@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <string>
 #include <string.h>
@@ -169,7 +169,8 @@ std::vector<std::pair<std::string, std::string> > GetShortcutsUsage(const std::s
   s.push_back(mp("Right arrow",    "Go to next time step"));
   s.push_back(mp("Up arrow",       "Make previous view visible"));
   s.push_back(mp("Down arrow",     "Make next view visible"));
-  s.push_back(mp("0",              "Reload project file"));
+  s.push_back(mp("0",              "Reload geometry"));
+  s.push_back(mp(cc + "0",         "Reload full project"));
   s.push_back(mp("1 or F1",        "Mesh lines"));
   s.push_back(mp("2 or F2",        "Mesh surfaces"));
   s.push_back(mp("3 or F3",        "Mesh volumes"));
@@ -231,6 +232,7 @@ std::vector<std::pair<std::string, std::string> > GetShortcutsUsage(const std::s
   s.push_back(mp("Alt+z",          "Set Z view"));
   s.push_back(mp("Alt+Shift+a",    "Hide/show small axes"));
   s.push_back(mp("Alt+Shift+b",    "Hide/show mesh volume faces"));
+  s.push_back(mp("Alt+Shift+c",    "Loop through predefined colormaps"));
   s.push_back(mp("Alt+Shift+d",    "Hide/show mesh surface faces"));
   s.push_back(mp("Alt+Shift+l",    "Hide/show mesh lines"));
   s.push_back(mp("Alt+Shift+p",    "Hide/show mesh points"));
@@ -283,9 +285,10 @@ void PrintUsage(const std::string &name)
 
 void GetOptions(int argc, char *argv[])
 {
-  // print messages on terminal
+  // print messages on terminal (use special 99 value so that we can detect if
+  // it was later set to 1 in the option file)
   int terminal = CTX::instance()->terminal;
-  CTX::instance()->terminal = 1;
+  CTX::instance()->terminal = 99;
 
 #if defined(HAVE_PARSER)
   if(argc && argv){
@@ -406,6 +409,10 @@ void GetOptions(int argc, char *argv[])
               i++;
               opt_mesh_partition_pyr_weight(0,GMSH_SET,atoi(argv[i]));
             }
+            else if (!strcmp(argv[i],"trihedron")) {
+              i++;
+              opt_mesh_partition_trih_weight(0,GMSH_SET,atoi(argv[i]));
+            }
             else if (!strcmp(argv[i],"hex")) {
               i++;
               opt_mesh_partition_hex_weight(0,GMSH_SET,atoi(argv[i]));
@@ -505,13 +512,6 @@ void GetOptions(int argc, char *argv[])
         else
           Msg::Fatal("Missing number of lloyd iterations");
       }
-      else if(!strcmp(argv[i] + 1, "bunin")) {
-        i++;
-        if(argv[i])
-          CTX::instance()->mesh.bunin = atoi(argv[i++]);
-        else
-          Msg::Fatal("Missing cavity size in bunin optimization");
-      }
 #if defined(HAVE_MESH)
       else if(!strcmp(argv[i] + 1, "microstructure")) {
         i++;
@@ -549,7 +549,7 @@ void GetOptions(int argc, char *argv[])
       else if(!strcmp(argv[i] + 1, "setstring")) {
         i++;
 	if (i + 1 < argc && argv[i][0] != '-' && argv[i + 1][0] != '-') {
-          gmsh_yystringsymbols[argv[i]] = argv[i + 1];
+          gmsh_yystringsymbols[argv[i]] = std::vector<std::string>(1, argv[i + 1]);
           Msg::GetCommandLineStrings()[argv[i]] = argv[i + 1];
           i += 2;
 	}
@@ -875,9 +875,9 @@ void GetOptions(int argc, char *argv[])
         i++;
         CTX::instance()->mesh.recombinationTestNoGreedyStrat = 1;
       }
-      else if(!strcmp(argv[i] + 1, "newstrat")) {
+      else if(!strcmp(argv[i] + 1, "meshdiscrete")) {
         i++;
-        CTX::instance()->mesh.recombinationTestNewStrat = 1;
+        CTX::instance()->meshDiscrete = 1;
       }
       else if(!strcmp(argv[i] + 1, "format") || !strcmp(argv[i] + 1, "f")) {
         i++;
@@ -912,22 +912,7 @@ void GetOptions(int argc, char *argv[])
       }
       else if(!strcmp(argv[i] + 1, "listen")) {
         i++;
-#if !defined(HAVE_ONELAB2)
         CTX::instance()->solver.listen = 1;
-#else
-        if(argv[i]) {
-          if(strstr(argv[i], "/") || strstr(argv[i], "\\") || !strstr(argv[i], ":")) // UNIX
-            CTX::instance()->onelab.unixSock = std::string(argv[i]);
-          else if(argv[i][0] == 'u') // UDT
-            CTX::instance()->onelab.udtSock = std::string(argv[i++]);
-          else if(argv[i][0] == 't') // TCP
-            CTX::instance()->onelab.tcpSock = std::string(argv[i++]);
-          else // TCP
-            CTX::instance()->onelab.tcpSock = std::string(argv[i]);
-        }
-        else // TCP
-          CTX::instance()->onelab.tcpSock = "127.0.0.1:1148";
-#endif
       }
       else if(!strcmp(argv[i] + 1, "minterpreter")) {
         i++;
@@ -984,13 +969,13 @@ void GetOptions(int argc, char *argv[])
                 MED_NUM_MINEUR, MED_NUM_RELEASE);
 #endif
         fprintf(stderr, "Packaged by      : %s\n", GMSH_PACKAGER);
-        fprintf(stderr, "Web site         : http://www.geuz.org/gmsh/\n");
-        fprintf(stderr, "Mailing list     : gmsh@geuz.org\n");
+        fprintf(stderr, "Web site         : http://gmsh.info\n");
+        fprintf(stderr, "Mailing list     : gmsh@onelab.info\n");
         Msg::Exit(0);
       }
       else if(!strcmp(argv[i] + 1, "help") || !strcmp(argv[i] + 1, "-help")) {
         fprintf(stderr, "Gmsh, a 3D mesh generator with pre- and post-processing facilities\n");
-        fprintf(stderr, "Copyright (C) 1997-2015 Christophe Geuzaine and Jean-Francois Remacle\n");
+        fprintf(stderr, "Copyright (C) 1997-2016 Christophe Geuzaine and Jean-Francois Remacle\n");
         PrintUsage(argv[0]);
         Msg::Exit(0);
       }
@@ -1131,5 +1116,6 @@ void GetOptions(int argc, char *argv[])
   else
     GModel::current()->setFileName(CTX::instance()->files[0]);
 
-  CTX::instance()->terminal = terminal;
+  if(CTX::instance()->terminal == 99)
+    CTX::instance()->terminal = terminal;
 }
