@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <stdlib.h>
 #include <string.h>
@@ -41,11 +41,11 @@ CTX::CTX() : gamepad(0)
     homeDir += "/";
 
   batch = batchAfterMesh = 0;
+  meshDiscrete=0;
   outputFileName = "";
   bgmFileName = "";
   createAppendMeshStatReport = 0;
   launchSolverAtStartup = -1;
-  solverToRun = -1;
   lc = 1.;
   min[0] = min[1] = min[2] = max[2] = 0.;
   max[0] = max[1] = 1.; // for nice view when adding point in new model
@@ -77,13 +77,13 @@ CTX::CTX() : gamepad(0)
   mesh.switchElementTags = 0;
   terminal = 0;
 
-  // need to initialize these too, since the corresponding opt_XXX
-  // routine uses the current value to set "mesh.changed"
+  // need to initialize these too, since the corresponding opt_XXX routines use
+  // the current value to detect changes
   mesh.changed = 0;
   mesh.qualityInf = mesh.qualitySup = mesh.qualityType = 0;
   mesh.radiusInf = mesh.radiusSup = 0;
   mesh.lines = mesh.triangles = mesh.tetrahedra = mesh.quadrangles = 0;
-  mesh.prisms = mesh.pyramids = mesh.hexahedra = 0;
+  mesh.prisms = mesh.pyramids = mesh.hexahedra = mesh.trihedra = 0;
   mesh.volumesEdges = mesh.volumesFaces = mesh.surfacesEdges = mesh.surfacesFaces = 0;
   mesh.volumesFaces = mesh.surfacesEdges = mesh.surfacesFaces = 0;
   mesh.hoOptimize = mesh.smoothNormals = mesh.reverseAllNormals = 0;
@@ -93,10 +93,19 @@ CTX::CTX() : gamepad(0)
   mesh.ignorePartBound = 0;
   mesh.saveTri = 0;
   color.mesh.tangents = color.mesh.tetrahedron = color.mesh.triangle = 0;
-  color.mesh.prism = color.mesh.pyramid = color.mesh.hexahedron = 0;
+  color.mesh.prism = color.mesh.pyramid = color.mesh.hexahedron = color.mesh.trihedron = 0;
   color.mesh.tangents = color.mesh.line = color.mesh.quadrangle = 0;
   for (int i = 0; i < 20; i++)
     color.mesh.carousel[i] = 0;
+  mesh.optimize = mesh.optimizeNetgen = 0;
+  mesh.refineSteps = mesh.scalingFactor = mesh.lcFactor = mesh.lcMin = mesh.lcMax = 0;
+  mesh.toleranceEdgeLength = mesh.toleranceInitialDelaunay = 0;
+  mesh.lcFromCurvature = mesh.lcFromPoints = mesh.lcExtendFromBoundary = 0;
+  mesh.lcIntegrationPrecision = mesh.randFactor = 0;
+  mesh.algo2d = mesh.algo3d = mesh.algoRecombine = mesh.recombineAll = 0;
+  mesh.recombine3DAll = mesh.algoSubdivide = mesh.meshOnlyVisible = 0;
+  mesh.minCircPoints = mesh.order = 0;
+  mesh.secondOrderLinear = mesh.secondOrderIncomplete = 0;
 }
 
 CTX::~CTX()
@@ -151,4 +160,3 @@ int CTX::unpackAlpha(unsigned int X)
   else
     return ( ( (X) >> 24 ) & 0xff );
 }
-
