@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _CONTEXT_H_
 #define _CONTEXT_H_
@@ -20,11 +20,10 @@ class GamePad;
 
 struct contextMeshOptions {
   int draw, changed, light, lightTwoSide, lightLines, pointType;
-  int points, lines, triangles, quadrangles, tetrahedra, hexahedra, prisms, pyramids;
+  int points, lines, triangles, quadrangles, tetrahedra, hexahedra, prisms, pyramids, trihedra;
   int surfacesEdges, surfacesFaces, volumesEdges, volumesFaces, numSubEdges;
   int pointsNum, linesNum, surfacesNum, volumesNum, qualityType, labelType;
   int optimize, optimizeNetgen, optimizeLloyd, smoothCrossField, refineSteps;
-  int remove4triangles;
   double normals, tangents, explode, angleSmoothNormals, allowSwapEdgeAngle;
   double mshFileVersion, mshFilePartitioned, pointSize, lineWidth;
   double qualityInf, qualitySup, radiusInf, radiusSup;
@@ -34,7 +33,8 @@ struct contextMeshOptions {
   int lcFromPoints, lcFromCurvature, lcExtendFromBoundary;
   int dual, voronoi, drawSkinOnly, colorCarousel, labelSampling;
   int fileFormat, nbSmoothing, algo2d, algo3d, algoSubdivide;
-  int algoRecombine, recombineAll, recombine3DAll, flexibleTransfinite;
+  int algoRecombine, recombineAll, recombine3DAll, recombine3DLevel, recombine3DConformity;
+  int flexibleTransfinite;
   //-- for recombination test (amaury) --
     int doRecombinationTest, recombinationTestStart;
     int recombinationTestNoGreedyStrat, recombinationTestNewStrat;
@@ -55,9 +55,9 @@ struct contextMeshOptions {
   int cgnsImportOrder;
   std::map<int,int> algo2d_per_face;
   std::map<int,int> curvature_control_per_face;
-  int bunin;
   int ignorePartBound;
   int NewtonConvergenceTestXYZ;
+  int preserveNumberingMsh2;
 };
 
 struct contextGeometryOptions {
@@ -103,8 +103,6 @@ class CTX {
   int createAppendMeshStatReport;
   // should we launch a solver at startup?
   int launchSolverAtStartup ;
-  // solver to use with ONELAB2
-  int solverToRun ;
   // save session/option file on exit?
   int sessionSave, optionsSave;
   // ask confirmation when overwriting files?
@@ -141,6 +139,8 @@ class CTX {
   int batch;
   // batch operations to apply after meshing (1: partition mesh)
   int batchAfterMesh;
+  // mesh discrete faces / edges
+  int meshDiscrete;
   // initial menu (0: automatic, 1: geom, 2: mesh, 3: solver, 4: post)
   int initialContext;
   // never popup dialogs in scripts (use default values instead)?
@@ -258,15 +258,12 @@ class CTX {
     int plugins, listen;
     double timeout;
     std::string socketName, pythonInterpreter, octaveInterpreter;
-    std::string name[NUM_SOLVERS], executable[NUM_SOLVERS], remoteLogin[NUM_SOLVERS];
-    int autoSaveDatabase, autoArchiveOutputFiles, autoMesh, autoMergeFile;
+    std::string name[NUM_SOLVERS], extension[NUM_SOLVERS];
+    std::string executable[NUM_SOLVERS], remoteLogin[NUM_SOLVERS];
+    int autoSaveDatabase, autoLoadDatabase;
+    int autoArchiveOutputFiles, autoMesh, autoMergeFile;
     int autoShowViews, autoShowLastStep, autoCheck, showInvisibleParameters;
   }solver;
-  // onelab options
-  struct{
-    std::string unixSock, tcpSock, udtSock;
-    bool unixConnected, tcpConnected, udtConnected;
-  }onelab;
   // print options
   struct{
     int fileFormat, epsQuality, epsCompress, epsPS3Shading;
@@ -293,7 +290,7 @@ class CTX {
     } geom;
     struct{
       unsigned int vertex, vertexSup, line, triangle, quadrangle;
-      unsigned int tetrahedron, hexahedron, prism, pyramid;
+      unsigned int tetrahedron, hexahedron, prism, pyramid, trihedron;
       unsigned int carousel[20];
       unsigned int tangents, normals;
     } mesh;
