@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _DEFAULT_OPTIONS_H_
 #define _DEFAULT_OPTIONS_H_
@@ -189,6 +189,27 @@ StringXString SolverOptions_String[] = {
   { F|S, "Name9" , opt_solver_name9 , "" ,
     "Name of solver 9" },
 
+  { F|S, "Extension0" , opt_solver_extension0 , ".pro" ,
+    "File extension for solver 0" },
+  { F|S, "Extension1" , opt_solver_extension1 , "" ,
+    "File extension for solver 1" },
+  { F|S, "Extension2" , opt_solver_extension2 , "" ,
+    "File extension for solver 2" },
+  { F|S, "Extension3" , opt_solver_extension3 , "" ,
+    "File extension for solver 3" },
+  { F|S, "Extension4" , opt_solver_extension4 , "" ,
+    "File extension for solver 4" },
+  { F|S, "Extension5" , opt_solver_extension5 , "" ,
+    "File extension for solver 5" },
+  { F|S, "Extension6" , opt_solver_extension6 , "" ,
+    "File extension for solver 6" },
+  { F|S, "Extension7" , opt_solver_extension7 , "" ,
+    "File extension for solver 7" },
+  { F|S, "Extension8" , opt_solver_extension8 , "" ,
+    "File extension for solver 8" },
+  { F|S, "Extension9" , opt_solver_extension9 , "" ,
+    "File extension for solver 9" },
+
   { F|S, "OctaveInterpreter" , opt_solver_octave_interpreter , "octave" ,
     "Name of the Octave interpreter (used to run .m files)" },
   { F|S, "PythonInterpreter" , opt_solver_python_interpreter , "python" ,
@@ -892,7 +913,7 @@ StringXNumber GeometryOptions_Number[] = {
 
   { F|O, "Tangents" , opt_geometry_tangents , 0. ,
     "Display size of tangent vectors (in pixels)" },
-  { F|O, "Tolerance" , opt_geometry_tolerance, 1.e-6 ,
+  { F|O, "Tolerance" , opt_geometry_tolerance, 1.e-8 ,
     "Geometrical tolerance" },
   { F,   "Transform" , opt_geometry_transform , 0. ,
     "Transform model display coordinates (0=no, 1=scale)" },
@@ -945,9 +966,6 @@ StringXNumber MeshOptions_Number[] = {
     "Field format for Nastran BDF files (0=free, 1=small, 2=large)" },
   { F|O, "Binary" , opt_mesh_binary , 0. ,
     "Write mesh files in binary format (if possible)" },
-  { F|O, "Bunin" , opt_mesh_bunin , 0. ,
-    "Apply Bunin optimization on quad meshes (the parameter is the maximal size of "
-    "a cavity that may be remeshed)" },
   { F|O, "Lloyd" , opt_mesh_lloyd , 0. ,
     "Apply lloyd optimization on surface meshes" },
   { F|O, "SmoothCrossField" , opt_mesh_smooth_cross_field , 0. ,
@@ -1033,8 +1051,8 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "NewtonConvergenceTestXYZ" , opt_mesh_newton_convergence_test_xyz , 0. ,
     "Force inverse surface mapping algorithm (Newton-Raphson) to converge in real coordinates (experimental)" },
   { F|O, "Format" , opt_mesh_file_format , FORMAT_AUTO ,
-    "Mesh output format (1=msh, 2=unv, 10=automatic, 19=vrml, 27=stl, 30=mesh, 31=bdf, "
-    "32=cgns, 33=med, 40=ply2)" },
+    "Mesh output format (1=msh, 2=unv, 10=automatic, 19=vrml, 26=pos statistics, 27=stl, "
+    "30=mesh, 31=bdf, 32=cgns, 33=med, 39=inp, 40=ply2)" },
 
   { F|O, "Hexahedra" , opt_mesh_hexahedra , 1. ,
     "Display mesh hexahedra?" },
@@ -1096,6 +1114,8 @@ StringXNumber MeshOptions_Number[] = {
     "Weight of prismatic element (wedge) for METIS load balancing" },
   { F|O, "PartitionPyramidWeight" , opt_mesh_partition_pyr_weight , 1 ,
     "Weight of pyramidal element for METIS load balancing" },
+  { F|O, "PartitionTrihedronWeight" , opt_mesh_partition_trih_weight , 0 ,
+    "Weight of trihedron element for METIS load balancing" },
   { F|O, "PartitionQuadWeight"    , opt_mesh_partition_qua_weight , 1 ,
     "Weight of quadrangle for METIS load balancing" },
   { F|O, "PartitionTetWeight"     , opt_mesh_partition_tet_weight , 1 ,
@@ -1113,6 +1133,8 @@ StringXNumber MeshOptions_Number[] = {
     "Number of prisms in the current mesh (read-only)" },
   { F, "NbPyramids" , opt_mesh_nb_pyramids , 0. ,
     "Number of pyramids in the current mesh (read-only)" },
+  { F, "NbTrihedra" , opt_mesh_nb_trihedra , 0. ,
+    "Number of trihedra in the current mesh (read-only)" },
   { F, "NbQuadrangles" , opt_mesh_nb_quadrangles , 0. ,
     "Number of quadrangles in the current mesh (read-only)" },
   { F, "NbTetrahedra" , opt_mesh_nb_tetrahedra , 0. ,
@@ -1144,6 +1166,8 @@ StringXNumber MeshOptions_Number[] = {
     "Display mesh prisms?" },
   { F|O, "Pyramids" , opt_mesh_pyramids , 1. ,
     "Display mesh pyramids?" },
+  { F|O, "Trihedra" , opt_mesh_trihedra , 1. ,
+    "Display mesh trihedra?" },
 
   { F|O, "Quadrangles" , opt_mesh_quadrangles , 1. ,
     "Display mesh quadrangles?" },
@@ -1164,6 +1188,8 @@ StringXNumber MeshOptions_Number[] = {
     "RandomFactor * size(triangle)/size(model) approaches machine accuracy)" },
   { F|O, "IgnorePartitionBoundary" , opt_mesh_ignore_part_bound , 0. ,
     "Ignore partitions boundaries (0=no, 1=yes)"},
+  { F|O, "PreserveNumberingMsh2" , opt_mesh_preserve_numbering_msh2 , 0. ,
+    "Preserve element numbering in MSH2 format (will break meshes with multiple physical groups for a single elementary entity)"},
 #if defined(HAVE_BLOSSOM)
   { F|O, "RecombinationAlgorithm" , opt_mesh_algo_recombine , 1 ,
 #else
@@ -1174,14 +1200,16 @@ StringXNumber MeshOptions_Number[] = {
     "Apply recombination algorithm to all surfaces, ignoring per-surface spec" },
   { F|O, "Recombine3DAll" , opt_mesh_recombine3d_all , 0 ,
     "Apply recombination3D algorithm to all volumes, ignoring per-volume spec" },
+  { F|O, "Recombine3DLevel" , opt_mesh_recombine3d_level , 0 ,
+    "3d recombination level (0: hex, 1: hex+prisms, 2: hex+prism+pyramids)" },
+  { F|O, "Recombine3DConformity" , opt_mesh_recombine3d_conformity , 0 ,
+    "3d recombination conformity type (0: nonconforming, 1: trihedra, 2: pyramids+trihedra, 3:pyramids+hexSplit+trihedra, 4:hexSplit+trihedra)" },
   { F|O, "DoRecombinationTest" , opt_mesh_do_recombination_test , 0 ,
     "Apply recombination algorithm for test" },
   { F|O, "RecombinationTestHorizStart" , opt_mesh_recombination_test_start , 1 ,
     "Depth start" },
   { F|O, "RecombinationTestNoGreedyStrat" , opt_mesh_recombination_no_greedy_strat , 0 ,
     "No greedy (global) strategies" },
-  { F|O, "RecombinationTestNewStrat" , opt_mesh_recombination_new_strat , 0 ,
-    "New strategies" },
   { F|O, "RemeshAlgorithm" , opt_mesh_remesh_algo , 0 ,
     "Remeshing algorithm (0=no split, 1=automatic, 2=automatic only with metis)" },
   { F|O, "RemeshParametrization" , opt_mesh_remesh_param , 4 ,
@@ -1189,8 +1217,6 @@ StringXNumber MeshOptions_Number[] = {
 
   { F|O, "RefineSteps" , opt_mesh_refine_steps , 10 ,
     "Number of refinement steps in the MeshAdapt-based 2D algorithms" },
-  { F|O, "Remove4Triangles" , opt_mesh_remove_4_triangles , 0 ,
-    "Try to remove nodes surrounded by 4 triangles in 2D triangular meshes" },
   { F|O, "ReverseAllNormals" , opt_mesh_reverse_all_normals , 0. ,
     "Reverse all the mesh normals (for display)" },
 
@@ -1263,8 +1289,10 @@ StringXNumber SolverOptions_Number[] = {
     "Automatically archive output files after each computation" },
   { F|O, "AutoCheck" , opt_solver_auto_check , 1. ,
     "Automatically check model every time a parameter is changed" },
-  { F|O, "AutoSaveDatabase" , opt_solver_auto_save_database , 0. ,
-    "Automatically save database after each computation" },
+  { F|O, "AutoLoadDatabase" , opt_solver_auto_load_database , 0. ,
+    "Automatically load the ONELAB database when launching a solver" },
+  { F|O, "AutoSaveDatabase" , opt_solver_auto_save_database , 1. ,
+    "Automatically save the ONELAB database after each computation" },
   { F|O, "AutoMesh" , opt_solver_auto_mesh , 1. ,
     "Automatically mesh if necesssary (0: never remesh; 1: on startup, use existing "
     "mesh on disk if available; 2: always remesh)" },
@@ -1401,7 +1429,11 @@ StringXNumber ViewOptions_Number[] = {
   { F|O, "ColormapInvert" , opt_view_colormap_invert , 0. ,
     "Invert the color values, i.e., replace x with (255-x) in the colormap?" },
   { F|O, "ColormapNumber" , opt_view_colormap_number , 2. ,
-    "Default colormap number" },
+    "Default colormap number (0: black, 1: vis5d, 2: jet, 3: lucie, 4: rainbow, "
+    "5: emc2000, 6: incadescent, 7: hot, 8: pink, 9: grayscale, 10: french, "
+    "11: hsv, 12: spectrum, 13: bone, 14: spring, 15: summer, 16: autumm, "
+    "17: winter, 18: cool, 19: copper, 20: magma, 21: inferno, 22: plasma, "
+    "23: viridis)"},
   { F|O, "ColormapRotation" , opt_view_colormap_rotation , 0. ,
     "Incremental colormap rotation" },
   { F|O, "ColormapSwap" , opt_view_colormap_swap , 0. ,
@@ -1445,6 +1477,8 @@ StringXNumber ViewOptions_Number[] = {
     "Display post-processing prisms?" },
   { F|O, "DrawPyramids" , opt_view_draw_pyramids , 1. ,
     "Display post-processing pyramids?" },
+  { F|O, "DrawTrihedra" , opt_view_draw_trihedra , 1. ,
+    "Display post-processing trihedra?" },
   { F|O, "DrawQuadrangles" , opt_view_draw_quadrangles , 1. ,
     "Display post-processing quadrangles?" },
   { F|O, "DrawScalars" , opt_view_draw_scalars , 1. ,
@@ -1798,6 +1832,7 @@ StringXColor GeometryOptions_Color[] = {
 #define COLQ  {130, 120, 225, 255}
 #define COLP  {232, 210, 23, 255}
 #define COLY  {217, 113, 38, 255}
+#define COLR  {20, 255, 0, 255}
 
 #define COL0  {255, 120, 0, 255}
 #define COL2  {255, 160, 0, 255}
@@ -1848,6 +1883,9 @@ StringXColor MeshOptions_Color[] = {
   { F|O, "Pyramids" , opt_mesh_color_pyramid ,
     COLY, COLY, COLW, COLY,
      "Mesh pyramid color (if Mesh.ColorCarousel=0)" },
+  { F|O, "Trihedra" , opt_mesh_color_trihedron ,
+    COLR, COLR, COLW, COLR,
+     "Mesh trihedron color (if Mesh.ColorCarousel=0)" },
   { F|O, "Tangents" , opt_mesh_color_tangents ,
     {255, 255, 0, 255}, {255, 255, 0, 255}, {0, 0, 0, 255}, {255, 255, 0, 255},
     "Tangent mesh vector color" },
@@ -1916,6 +1954,7 @@ StringXColor ViewOptions_Color[] = {
   { F|O, "Hexahedra" , opt_view_color_hexahedra , ELECOL, "Hexahedron color" },
   { F|O, "Prisms" , opt_view_color_prisms , ELECOL, "Prism color" },
   { F|O, "Pyramids" , opt_view_color_pyramids , ELECOL, "Pyramid color" },
+  { F|O, "Trihedra" , opt_view_color_trihedra , ELECOL, "Trihedron color" },
   { F|O, "Tangents" , opt_view_color_tangents ,
     {255, 255, 0, 255}, {255, 255, 0, 255}, {0, 0, 0, 255}, {255, 255, 0, 255},
     "Tangent vector color" },
