@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <string>
 #include <time.h>
@@ -23,9 +23,11 @@
 #include "robustPredicates.h"
 #if defined(HAVE_POST)
 #include "PView.h"
+#include "PViewData.h"
+#include "PViewOptions.h"
 #endif
 
-#if defined(HAVE_ONELAB) && !defined(HAVE_ONELAB2)
+#if defined(HAVE_ONELAB)
 #include "gmshLocalNetworkClient.h"
 #endif
 
@@ -43,16 +45,8 @@
 #include "FlGui.h"
 #include "graphicWindow.h"
 #include "drawContext.h"
-#if defined(HAVE_ONELAB2)
-#include "onelab2Group.h"
-#else
 #include "onelabGroup.h"
 #endif
-#endif
-
-#if defined(HAVE_ONELAB2)
-#include "OnelabServer.h"
-#endif
 
 int GmshInitialize(int argc, char **argv)
 {
@@ -85,8 +79,8 @@ int GmshInitialize(int argc, char **argv)
   PluginManager::instance()->registerDefaultPlugins();
 #endif
 
-  // Initialize robust predicates
-  robustPredicates::exactinit();
+  // Initialize robust predicates (no static filter for now, we do not know the size of the domain)
+  robustPredicates::exactinit(0,1.0,1.0,1.0);
 
   if(dummy) delete dummy;
   return 1;
@@ -227,6 +221,21 @@ int GmshWriteFile(const std::string &fileName)
 
 int GmshFinalize()
 {
+#if defined(HAVE_POST)
+  // Delete all PViewData stored in static list of PView class
+  for(unsigned int i = 0; i < PView::list.size(); i++) {
+    delete PView::list[i];
+  }
+  PView::list.clear();
+
+  // Delete static _interpolationSchemes of PViewData class
+  PViewData::removeAllInterpolationSchemes();
+#endif
+  // Delete all Gmodels
+  for(unsigned int i = 0; i < GModel::list.size(); i++)
+    delete GModel::list[i];
+  GModel::list.clear();
+
   return 1;
 }
 
@@ -237,13 +246,6 @@ int GmshBatch()
             Msg::GetCommSize(), Msg::GetCommSize() > 1 ? "s" : "",
             Msg::GetMaxThreads(), Msg::GetMaxThreads() > 1 ? "s" : "");
   Msg::Info("Started on %s", Msg::GetLaunchDate().c_str());
-#if defined(HAVE_ONELAB2)
-  if(CTX::instance()->onelab.listen_port > 0) {
-    OnelabServer::instance(0, CTX::instance()->onelab.listen_port);
-    OnelabServer::instance()->Run();
-    Msg::Exit(0);
-  }
-#endif
 
   OpenProject(GModel::current()->getFileName());
   bool open = false;
@@ -273,9 +275,7 @@ int GmshBatch()
 #endif
 
   if(CTX::instance()->batch == -3){
-#if !defined(HAVE_ONELAB2)
     GmshRemote();
-#endif
   }
   else if(CTX::instance()->batch == -2){
     GModel::current()->checkMeshCoherence(CTX::instance()->geom.tolerance);
@@ -389,13 +389,12 @@ int GmshFLTK(int argc, char **argv)
     else
       Msg::Error("Invalid background mesh (no view)");
   }
-#if !defined(HAVE_ONELAB2)
+
   // listen to external solvers
   if(CTX::instance()->solver.listen){
     gmshLocalNetworkClient *c = new gmshLocalNetworkClient("Listen", "");
     c->run();
   }
-#endif
 
   // launch solver (if requested) and fill onelab tree
   solver_cb(0, (void*)CTX::instance()->launchSolverAtStartup);
