@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include "GmshConfig.h"
 
@@ -28,14 +28,8 @@
 #if defined(HAVE_ONELAB)
 #include "onelab.h"
 #endif
-#if !defined(HAVE_ONELAB2)
-#include "gmshLocalNetworkClient.h"
-#endif
 
-#if defined(HAVE_ONELAB2)
-#include "OnelabDatabase.h"
-#include "NetworkUtils.h"
-#endif
+#include "gmshLocalNetworkClient.h"
 
 #if defined(HAVE_PETSC)
 #include <petsc.h>
@@ -67,17 +61,10 @@ std::string Msg::_commandLine;
 std::string Msg::_launchDate;
 std::map<std::string, std::vector<double> > Msg::_commandLineNumbers;
 std::map<std::string, std::string> Msg::_commandLineStrings;
-#if !defined(HAVE_ONELAB2)
 GmshClient *Msg::_client = 0;
-#endif
 std::string Msg::_execName;
-#if defined(HAVE_ONELAB2)
-OnelabDatabase *OnelabDatabase::_instance = NULL;
-OnelabDatabase *Msg::_onelabClient = 0;
-#elif defined(HAVE_ONELAB)
-onelab::client *Msg::_onelabClient = 0;
-#endif
 #if defined(HAVE_ONELAB)
+onelab::client *Msg::_onelabClient = 0;
 onelab::server *onelab::server::_server = 0;
 #endif
 
@@ -275,13 +262,11 @@ void Msg::Fatal(const char *fmt, ...)
   va_end(args);
 
   if(_callback) (*_callback)("Fatal", str);
-#if !defined(HAVE_ONELAB2)
   if(_client) _client->Error(str);
-#endif
 
 #if defined(HAVE_FLTK)
   if(FlGui::available()){
-    if(FlGui::instance()->in_main_thread()) FlGui::instance()->check();
+    FlGui::instance()->check();
     std::string tmp = std::string("@C1@.") + "Fatal   : " + str;
     FlGui::instance()->addMessage(tmp.c_str());
     if(_firstError.empty()) _firstError = str;
@@ -325,13 +310,11 @@ void Msg::Error(const char *fmt, ...)
   va_end(args);
 
   if(_callback) (*_callback)("Error", str);
-#if !defined(HAVE_ONELAB2)
   if(_client) _client->Error(str);
-#endif
 
 #if defined(HAVE_FLTK)
   if(FlGui::available()){
-    if(FlGui::instance()->in_main_thread()) FlGui::instance()->check();
+    FlGui::instance()->check();
     std::string tmp = std::string(CTX::instance()->guiColorScheme ? "@B72@." : "@C1@.")
       + "Error   : " + str;
     FlGui::instance()->addMessage(tmp.c_str());
@@ -367,12 +350,11 @@ void Msg::Warning(const char *fmt, ...)
   va_end(args);
 
   if(_callback) (*_callback)("Warning", str);
-#if !defined(HAVE_ONELAB2)
   if(_client) _client->Warning(str);
-#endif
+
 #if defined(HAVE_FLTK)
   if(FlGui::available()){
-    if(FlGui::instance()->in_main_thread()) FlGui::instance()->check();
+    FlGui::instance()->check();
     std::string tmp = std::string(CTX::instance()->guiColorScheme ? "@B152@." : "@C5@.")
       + "Warning : " + str;
     FlGui::instance()->addMessage(tmp.c_str());
@@ -405,9 +387,7 @@ void Msg::Info(const char *fmt, ...)
   va_end(args);
 
   if(_callback) (*_callback)("Info", str);
-#if !defined(HAVE_ONELAB2)
   if(_client) _client->Info(str);
-#endif
 
 #if defined(HAVE_FLTK)
 #if defined(_OPENMP)
@@ -415,7 +395,7 @@ void Msg::Info(const char *fmt, ...)
 #endif
   {
     if(FlGui::available()){
-      if(FlGui::instance()->in_main_thread()) FlGui::instance()->check();
+      FlGui::instance()->check();
       std::string tmp = std::string("Info    : ") + str;
       FlGui::instance()->addMessage(tmp.c_str());
     }
@@ -430,6 +410,7 @@ void Msg::Info(const char *fmt, ...)
     fflush(stdout);
   }
 }
+
 void Msg::RequestRender()
 {
   if(_callback) (*_callback)("RequestRender", "");
@@ -446,9 +427,7 @@ void Msg::Direct(const char *fmt, ...)
   va_end(args);
 
   if(_callback) (*_callback)("Direct", str);
-#if !defined(HAVE_ONELAB2)
   if(_client) _client->Info(str);
-#endif
 
 #if defined(HAVE_FLTK)
 #if defined(_OPENMP)
@@ -456,7 +435,7 @@ void Msg::Direct(const char *fmt, ...)
 #endif
   {
     if(FlGui::available()){
-      if(FlGui::instance()->in_main_thread()) FlGui::instance()->check();
+      FlGui::instance()->check();
       std::string tmp = std::string(CTX::instance()->guiColorScheme ? "@B136@." : "@C4@.")
         + str;
       FlGui::instance()->addMessage(tmp.c_str());
@@ -488,9 +467,7 @@ void Msg::StatusBar(bool log, const char *fmt, ...)
   va_end(args);
 
   if(_callback && log) (*_callback)("Info", str);
-#if !defined(HAVE_ONELAB2)
   if(_client && log) _client->Info(str);
-#endif
 
 #if defined(HAVE_FLTK)
 #if defined(_OPENMP)
@@ -498,7 +475,7 @@ void Msg::StatusBar(bool log, const char *fmt, ...)
 #endif
   {
     if(FlGui::available()){
-      if(log && FlGui::instance()->in_main_thread()) FlGui::instance()->check();
+      if(log) FlGui::instance()->check();
       if(!log || _verbosity > 4)
 	FlGui::instance()->setStatus(str);
       if(log){
@@ -532,6 +509,15 @@ void Msg::StatusGl(const char *fmt, ...)
 #endif
 }
 
+void Msg::SetWindowTitle(const std::string &title)
+{
+#if defined(HAVE_FLTK)
+  if(FlGui::available()){
+    FlGui::instance()->setGraphicTitle(title);
+  }
+#endif
+}
+
 void Msg::Debug(const char *fmt, ...)
 {
   if(_verbosity < 99) return;
@@ -543,9 +529,7 @@ void Msg::Debug(const char *fmt, ...)
   va_end(args);
 
   if(_callback) (*_callback)("Debug", str);
-#if !defined(HAVE_ONELAB2)
   if(_client) _client->Info(str);
-#endif
 
 #if defined(HAVE_FLTK)
   if(FlGui::available()){
@@ -578,13 +562,11 @@ void Msg::ProgressMeter(int n, int N, bool log, const char *fmt, ...)
     va_end(args);
     sprintf(str2, "%3d%%    : %s", _progressMeterCurrent, str);
 
-#if !defined(HAVE_ONELAB2)
     if(_client) _client->Progress(str2);
-#endif
 
 #if defined(HAVE_FLTK)
     if(FlGui::available() && _verbosity > 4){
-      if(FlGui::instance()->in_main_thread()) FlGui::instance()->check();
+      FlGui::instance()->check();
       FlGui::instance()->setProgress(str, (n > N - 1) ? 0 : n, 0, N);
     }
 #endif
@@ -752,107 +734,97 @@ int Msg::GetAnswer(const char *question, int defaultval, const char *zero,
 
 bool Msg::UseOnelab()
 {
-#if defined(HAVE_ONELAB2)
-  return true;
-#elif defined(HAVE_ONELAB)
+#if defined(HAVE_ONELAB)
   return _onelabClient ? true : false;
 #else
   return false;
 #endif
 }
 
-void Msg::SetOnelabNumber(std::string name, double val, bool visible)
+void Msg::SetOnelabNumber(std::string name, double val, bool visible,
+                          bool persistent, bool readOnly, bool neverChanged)
 {
 #if defined(HAVE_ONELAB)
   if(_onelabClient){
     std::vector<onelab::number> numbers;
-#if defined(HAVE_ONELAB2)
-    _onelabClient->get(numbers, name, "Gmsh");
-#else
     _onelabClient->get(numbers, name);
-#endif
     if(numbers.empty()){
       numbers.resize(1);
       numbers[0].setName(name);
     }
     numbers[0].setValue(val);
     numbers[0].setVisible(visible);
-#if defined(HAVE_ONELAB2)
-    _onelabClient->set(numbers[0], "Gmsh");
-#else
+    if(persistent) numbers[0].setAttribute("Persistent", "1");
+    numbers[0].setReadOnly(readOnly);
+    numbers[0].setNeverChanged(neverChanged);
     _onelabClient->set(numbers[0]);
-#endif
   }
 #endif
 }
 
-void Msg::SetOnelabString(std::string name, std::string val, bool visible)
+void Msg::SetOnelabString(std::string name, std::string val, bool visible,
+                          bool persistent, bool readOnly, bool neverChanged,
+                          const std::string &kind)
 {
 #if defined(HAVE_ONELAB)
   if(_onelabClient){
     std::vector<onelab::string> strings;
-#if defined(HAVE_ONELAB2)
-    _onelabClient->get(strings, name, "Gmsh");
-#else
     _onelabClient->get(strings, name);
-#endif
     if(strings.empty()){
       strings.resize(1);
       strings[0].setName(name);
     }
     strings[0].setValue(val);
     strings[0].setVisible(visible);
-#if defined(HAVE_ONELAB2)
-    _onelabClient->set(strings[0], "Gmsh");
-#else
+    if(persistent) strings[0].setAttribute("Persistent", "1");
+    strings[0].setReadOnly(readOnly);
+    strings[0].setNeverChanged(neverChanged);
+    if(kind.size()) strings[0].setKind(kind);
     _onelabClient->set(strings[0]);
-#endif
   }
 #endif
 }
 
-double Msg::GetOnelabNumber(std::string name)
+double Msg::GetOnelabNumber(std::string name, double defaultValue,
+                            bool errorIfMissing)
 {
 #if defined(HAVE_ONELAB)
   if(_onelabClient){
     std::vector<onelab::number> numbers;
-#if defined(HAVE_ONELAB2)
-    _onelabClient->get(numbers, name, "Gmsh");
-#else
     _onelabClient->get(numbers, name);
-#endif
     if(numbers.empty()){
-      Msg::Error("Unknown ONELAB number parameter '%s'", name.c_str());
-      return 0.;
+      if(errorIfMissing)
+        Msg::Error("Unknown ONELAB number parameter '%s'", name.c_str());
+      return defaultValue;
     }
     else
       return numbers[0].getValue();
   }
 #endif
-  Msg::Error("GetNumber requires a ONELAB client");
-  return 0.;
+  if(errorIfMissing)
+    Msg::Error("GetNumber requires a ONELAB client");
+  return defaultValue;
 }
 
-std::string Msg::GetOnelabString(std::string name)
+std::string Msg::GetOnelabString(std::string name, const std::string &defaultValue,
+                                 bool errorIfMissing)
 {
 #if defined(HAVE_ONELAB)
   if(_onelabClient){
     std::vector<onelab::string> strings;
-#if defined(HAVE_ONELAB2)
-    _onelabClient->get(strings, name, "Gmsh");
-#else
     _onelabClient->get(strings, name);
-#endif
     if(strings.empty()){
-      Msg::Error("Unknown ONELAB string parameter '%s'", name.c_str());
-      return "";
+      if(errorIfMissing)
+        Msg::Error("Unknown ONELAB string parameter '%s'", name.c_str());
+      return defaultValue;
     }
     else
       return strings[0].getValue();
   }
 #endif
-  Msg::Error("GetString requires a ONELAB client");
-  return "";
+  if(errorIfMissing)
+    Msg::Error("GetString requires a ONELAB client");
+  return defaultValue;
 }
 
 #if defined(HAVE_ONELAB)
@@ -885,52 +857,7 @@ public:
 
 void Msg::InitializeOnelab(const std::string &name, const std::string &sockname)
 {
-#if defined(HAVE_ONELAB2)
-  if(sockname.empty()){
-    if(name != "Gmsh"){ // load db from file:
-      FILE *fp = Fopen(name.c_str(), "rb");
-      if(fp){
-        OnelabDatabase::instance()->fromFile(fp);
-        fclose(fp);
-      }
-      else
-        Error("Error loading onelab database '%s'", name.c_str());
-    }
-    _onelabClient = OnelabDatabase::instance();
-  }
-  else{
-    UInt32 address = 0;
-    UInt16 port = 1148;
-    size_t colon = sockname.find(':');
-    if(colon != std::string::npos) {
-      address = ip4_inet_pton(sockname.substr(0,colon).c_str());
-      port = atoi(sockname.substr(colon+1).c_str());
-    }
-    GmshNetworkClient *c = OnelabDatabase::instance()->useAsNetworkClient(address, port, name);
-    if(c == NULL) {
-      Error("Unable to connect ONELAB server (%s)", sockname.c_str());
-      Exit(1);
-    }
-#if defined(HAVE_FLTK)
-    else if(FlGui::available()) // FIXME
-      c->setCallback(FlGui::instance()->onelab);
-    else
-      Warning("FlGUI not available");
-#endif
-    _onelabClient = OnelabDatabase::instance();
-
-    SetOnelabNumber(name + "/UseCommandLine", 1, false);
-    SetOnelabString(name + "/FileExtension", ".geo", false);
-    SetOnelabString(name + "/9CheckCommand", "-", false);
-    SetOnelabString(name + "/9ComputeCommand", "-3", false);
-    std::vector<onelab::string> ps;
-    _onelabClient->get(ps, name + "/Action", "Gmsh");
-    if(ps.size()){
-      Info("Performing ONELAB '%s'", ps[0].getValue().c_str());
-      if(ps[0].getValue() == "initialize") Exit(0);
-    }
-  }
-#elif defined(HAVE_ONELAB)
+#if defined(HAVE_ONELAB)
   if(_onelabClient) delete _onelabClient;
   if(sockname.empty()){
     _onelabClient = new localGmsh();
@@ -949,8 +876,8 @@ void Msg::InitializeOnelab(const std::string &name, const std::string &sockname)
     _onelabClient = c;
     _client = c->getGmshClient();
 
-    SetOnelabNumber(name + "/UseCommandLine", 1, false);
-    SetOnelabString(name + "/FileExtension", ".geo", false);
+    SetOnelabNumber(name + "/Use command line", 1, false);
+    SetOnelabString(name + "/File extension", ".geo", false);
     SetOnelabString(name + "/9CheckCommand", "-", false);
     SetOnelabString(name + "/9ComputeCommand", "-3", false);
 
@@ -982,13 +909,10 @@ std::string Msg::GetOnelabAction()
   if(_onelabClient){
     std::vector<onelab::string> ps;
     _onelabClient->get(ps, _onelabClient->getName() + "/Action");
-#ifdef HAVE_ONELAB2
-    _onelabClient->recvfrom();
-#endif
     if(ps.size()) return ps[0].getValue();
   }
-  return "";
 #endif
+  return "";
 }
 
 void Msg::LoadOnelabClient(const std::string &clientName, const std::string &sockName)
@@ -1096,11 +1020,7 @@ void Msg::ExchangeOnelabParameter(const std::string &key,
   }
 
   std::vector<onelab::number> ps;
-#if defined(HAVE_ONELAB2)
-  _onelabClient->get(ps, name, "Gmsh");
-#else
   _onelabClient->get(ps, name);
-#endif
   bool noRange = true, noChoices = true, noLoop = true;
   bool noGraph = true, noClosed = true;
   if(ps.size()){
@@ -1180,12 +1100,8 @@ void Msg::ExchangeOnelabParameter(const std::string &key,
   if(noClosed && fopt.count("Closed"))
     ps[0].setAttribute("Closed", fopt["Closed"][0] ? "1" : "0");
   _setStandardOptions(&ps[0], fopt, copt);
-#if defined(HAVE_ONELAB2)
-  _onelabClient->set(ps[0], "Gmsh");
-#else
   _onelabClient->set(ps[0]);
 #endif
-#endif
 }
 
 void Msg::ExchangeOnelabParameter(const std::string &key,
@@ -1238,12 +1154,8 @@ void Msg::ExchangeOnelabParameter(const std::string &key,
   if(noMultipleSelection && copt.count("MultipleSelection"))
     ps[0].setAttribute("MultipleSelection", copt["MultipleSelection"][0]);
   _setStandardOptions(&ps[0], fopt, copt);
-#if defined(HAVE_ONELAB2)
-  _onelabClient->set(ps[0], "Gmsh");
-#else
   _onelabClient->set(ps[0]);
 #endif
-#endif
 }
 
 void Msg::UndefineOnelabParameter(const std::string &name)
@@ -1254,30 +1166,54 @@ void Msg::UndefineOnelabParameter(const std::string &name)
 #endif
 }
 
-void Msg::ImportPhysicalsAsOnelabRegions()
+void Msg::ImportPhysicalGroupsInOnelab()
 {
 #if defined(HAVE_ONELAB)
   if(_onelabClient){
     std::map<int, std::vector<GEntity*> > groups[4];
     GModel::current()->getPhysicalGroups(groups);
+    int size = 0;
+    for(int dim = 0; dim <= 3; dim++)
+      size += groups[dim].size();
+    onelab::number n("Gmsh/Number of physical groups", size);
+    n.setReadOnly(true);
+    n.setVisible(false);
+    n.setAttribute("Closed", "1");
+    _onelabClient->set(n);
+
+    onelab::number nd("Gmsh/Model dimension", GModel::current()->getDim());
+    nd.setReadOnly(true);
+    nd.setVisible(false);
+    nd.setAttribute("Closed", "1");
+    _onelabClient->set(nd);
+
+    int index = 1;
     for(int dim = 0; dim <= 3; dim++){
       for(std::map<int, std::vector<GEntity*> >::iterator it = groups[dim].begin();
           it != groups[dim].end(); it++){
-        // create "read-only" onelab region
+        int num = it->first;
         std::string name = GModel::current()->getPhysicalName(dim, it->first);
-        std::ostringstream num;
-        num << it->first;
-        if(name.empty())
-          name = std::string("Physical") +
-            ((dim == 3) ? "Volume" : (dim == 2) ? "Surface" :
-             (dim == 1) ? "Line" : "Point") + num.str();
-        name.insert(0, "Gmsh parameters/Physical groups/");
-        onelab::region p(name, num.str());
-        p.setDimension(dim);
-        p.setReadOnly(true);
-        p.setVisible(false);
-        p.setAttribute("Closed", "1");
-        _onelabClient->set(p);
+        char tmp[256];
+        if(name.empty()){
+          sprintf(tmp, "Physical %s %d", (dim == 3) ? "Volume " : (dim == 2) ? "Surface " :
+                  (dim == 1) ? "Line " : "Point ", num);
+          name = tmp;
+        }
+        sprintf(tmp, "Gmsh/Physical group %d/", index);
+        std::string str = tmp;
+        onelab::number n1(str + "Dimension", dim);
+        n1.setReadOnly(true);
+        n1.setVisible(false);
+        _onelabClient->set(n1);
+        onelab::number n2(str + "Number", num);
+        n2.setReadOnly(true);
+        n2.setVisible(false);
+        _onelabClient->set(n2);
+        onelab::string s(str + "Name", name);
+        s.setReadOnly(true);
+        s.setVisible(false);
+        _onelabClient->set(s);
+        index++;
       }
     }
   }
@@ -1286,8 +1222,9 @@ void Msg::ImportPhysicalsAsOnelabRegions()
 
 void Msg::RunOnelabClient(const std::string &name, const std::string &command)
 {
-#if defined(HAVE_ONELAB) && !defined(HAVE_ONELAB2)
-  onelab::remoteNetworkClient *c = dynamic_cast<onelab::remoteNetworkClient*>(_onelabClient);
+#if defined(HAVE_ONELAB)
+  onelab::remoteNetworkClient *c =
+    dynamic_cast<onelab::remoteNetworkClient*>(_onelabClient);
   if(c){
     c->runSubClient(name, command);
   }
@@ -1298,15 +1235,16 @@ void Msg::RunOnelabClient(const std::string &name, const std::string &command)
 #endif
 }
 
+void Msg::SetOnelabChanged(bool value, const std::string &client)
+{
+#if defined(HAVE_ONELAB)
+  onelab::server::instance()->setChanged(value, client);
+#endif
+}
+
 void Msg::FinalizeOnelab()
 {
-#if defined(HAVE_ONELAB2)
-  if(_onelabClient) {
-    _onelabClient->finalize();
-    delete _onelabClient;
-    _onelabClient = 0;
-  }
-#elif defined(HAVE_ONELAB)
+#if defined(HAVE_ONELAB)
   if(_onelabClient){
     delete _onelabClient;
     _onelabClient = 0;
