@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _GMSH_MESSAGE_H_
 #define _GMSH_MESSAGE_H_
@@ -16,10 +16,6 @@
 class GmshClient;
 namespace onelab{ class client; }
 
-#if defined(HAVE_ONELAB2)
-class OnelabDatabase;
-#endif
-
 // the external message handler
 class GmshMessage{
  public:
@@ -50,14 +46,9 @@ class Msg {
   // command-line-specified numbers and strings
   static std::map<std::string, std::vector<double> > _commandLineNumbers;
   static std::map<std::string, std::string> _commandLineStrings;
-#if !defined(HAVE_ONELAB2)
   // communication with Gmsh when run remotely
   static GmshClient *_client;
-#endif
-#if defined(HAVE_ONELAB2)
-  // communication with onelab server (replace _client and old _onelabClient)
-  static OnelabDatabase *_onelabClient;
-#elif defined(HAVE_ONELAB)
+#if defined(HAVE_ONELAB)
   // communication with onelab server
   static onelab::client *_onelabClient;
 #endif
@@ -97,6 +88,7 @@ class Msg {
   static void Direct(const char *fmt, ...);
   static void StatusBar(bool log, const char *fmt, ...);
   static void StatusGl(const char *fmt, ...);
+  static void SetWindowTitle(const std::string &title);
   static void Debug(const char *fmt, ...);
   static void ProgressMeter(int n, int N, bool log, const char *fmt, ...);
   static void SetProgressMeterStep(int step){ _progressMeterStep = step; }
@@ -118,22 +110,22 @@ class Msg {
   static void SetExecutableName(const std::string &name) { _execName.assign(name); }
   static std::string GetExecutableName() { return _execName; }
   static void LoadOnelabClient(const std::string &name, const std::string &sockName);
-#if !defined(HAVE_ONELAB2)
   static GmshClient *GetGmshClient(){ return _client; }
-#else
-  static int GetGmshClient(){ return 1; }
-#endif
-#if defined(HAVE_ONELAB2)
-  static OnelabDatabase *GetOnelabClient(){ return _onelabClient; }
-#elif defined(HAVE_ONELAB)
+#if defined(HAVE_ONELAB)
   static onelab::client *GetOnelabClient(){ return _onelabClient; }
 #endif
   static void FinalizeOnelab();
   static bool UseOnelab();
-  static void SetOnelabNumber(std::string name, double val, bool visible=true);
-  static void SetOnelabString(std::string name, std::string val, bool visible=true);
-  static double GetOnelabNumber(std::string name);
-  static std::string GetOnelabString(std::string name);
+  static void SetOnelabNumber(std::string name, double val, bool visible=true,
+                              bool persistent=false, bool readOnly=false,
+                              bool neverChanged=false);
+  static void SetOnelabString(std::string name, std::string val, bool visible=true,
+                              bool persistent=false, bool readOnly=false,
+                              bool neverChanged=false, const std::string &kind="");
+  static double GetOnelabNumber(std::string name, double defaultValue=0.,
+                                bool errorIfMissing=false);
+  static std::string GetOnelabString(std::string name, const std::string &defaultValue="",
+                                     bool errorIfMissing=false);
   static void SetOnelabAction(const std::string &action);
   static std::string GetOnelabAction();
   static void ExchangeOnelabParameter(const std::string &key,
@@ -146,7 +138,8 @@ class Msg {
                                       std::map<std::string, std::vector<std::string> > &copt);
   static void UndefineOnelabParameter(const std::string &name);
   static void RunOnelabClient(const std::string &name, const std::string &exe="");
-  static void ImportPhysicalsAsOnelabRegions();
+  static void SetOnelabChanged(bool value=true, const std::string &client="Gmsh");
+  static void ImportPhysicalGroupsInOnelab();
 };
 
 #endif
