@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 // This file contains a bunch of functions that depend on OS-dependent
 // features and/or system calls
@@ -583,13 +583,7 @@ int SystemCallExe(const std::string &exe, const std::string &argsOrCommand,
     return 1;
   }
   if(!blocking) cmd += " &";
-#if defined(HAVE_FLTK) // FIXME: remove
-  if(FlGui::available()) FlGui::instance()->lock();
-#endif
   Msg::Info("Calling '%s'", cmd.c_str());
-#if defined(HAVE_FLTK) // FIXME: remove
-  if(FlGui::available()) FlGui::instance()->unlock();
-#endif
   if(!system(cmd.c_str())) return 1;
 #endif
   return 0;
