@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <sstream>
 #include <string.h>
@@ -43,11 +43,7 @@
 #if defined(HAVE_FLTK)
 #include <FL/fl_ask.H>
 #include "FlGui.h"
-#if defined(HAVE_ONELAB2)
-#include "onelab2Group.h"
-#else
 #include "onelabGroup.h"
-#endif
 #include "graphicWindow.h"
 #include "drawContext.h"
 #endif
@@ -276,6 +272,15 @@ void ParseString(const std::string &str, bool inCurrentModelDir)
   }
 }
 
+static std::string getSolverForExtension(const std::string &ext)
+{
+  for(int i = 0; i < NUM_SOLVERS; i++){
+    if(opt_solver_extension(i, GMSH_GET, "") == ext)
+      return opt_solver_name(i, GMSH_GET, "");
+  }
+  return "";
+}
+
 static int defineSolver(const std::string &name)
 {
   for(int i = 0; i < NUM_SOLVERS; i++){
@@ -286,19 +291,9 @@ static int defineSolver(const std::string &name)
   return NUM_SOLVERS - 1;
 }
 
-int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTitle,
-              bool setBoundingBox)
+int MergeFile(const std::string &fileName, bool warnIfMissing, bool setBoundingBox,
+              bool importPhysicalsInOnelab)
 {
-  if(GModel::current()->getName() == ""){
-    GModel::current()->setFileName(fileName);
-    GModel::current()->setName(SplitFileName(fileName)[1]);
-  }
-
-#if defined(HAVE_FLTK)
-  if(FlGui::available() && setWindowTitle)
-    FlGui::instance()->setGraphicTitle(GModel::current()->getFileName());
-#endif
-
   // added 'b' for pure Windows programs, since some of these files
   // contain binary data
   gmshFILE fp = gmshopen(fileName.c_str(), "rb");
@@ -324,11 +319,11 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
     if(ext != ".geo" && ext != ".GEO" &&
        ext != ".unv" && ext != ".UNV"){
       if(doSystemUncompress(fileName, noExt))
-        return MergeFile(noExt, false, setWindowTitle);
+        return MergeFile(noExt, false);
     }
 #else
     if(doSystemUncompress(fileName, noExt))
-      return MergeFile(noExt, false, setWindowTitle);
+      return MergeFile(noExt, false);
 #endif
   }
 
@@ -343,6 +338,38 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
 #endif
 
   int status = 0;
+
+#if defined(HAVE_ONELAB)
+  std::string solver = getSolverForExtension(ext);
+  if(solver.size()){
+    int num = defineSolver(solver);
+    Msg::SetOnelabString(solver + "/Model name", fileName, true, true,
+                         false, false, "file");
+    if(GModel::current()->getName() == "" ||
+       Msg::GetOnelabString("Gmsh/Model name").empty()){
+      GModel::current()->setFileName(split[0] + split[1] + ".geo");
+      GModel::current()->setName(split[1] + ".geo");
+    }
+    CTX::instance()->launchSolverAtStartup = num;
+    CTX::instance()->geom.draw = 1;
+    return 1;
+  }
+  else if(ext == ".py" || ext == ".PY" ||
+          ext == ".m" || ext == ".M" ||
+          ext == ".exe" || ext == ".EXE"){
+    int num = defineSolver(split[1]);
+    opt_solver_executable(num, GMSH_SET, fileName);
+    CTX::instance()->launchSolverAtStartup = num;
+    CTX::instance()->geom.draw = 1;
+    return 1;
+  }
+#endif
+
+  if(GModel::current()->getName() == ""){
+    GModel::current()->setFileName(fileName);
+    GModel::current()->setName(SplitFileName(fileName)[1]);
+  }
+
   if(ext == ".stl" || ext == ".STL"){
     status = GModel::current()->readSTL(fileName, CTX::instance()->geom.tolerance);
   }
@@ -433,25 +460,6 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
   else if(ext == ".csv"){
     status = readFile3M(fileName);
   }
-#endif
-#if defined(HAVE_ONELAB)
-  else if(ext == ".pro" || ext == ".PRO"){
-    int num = defineSolver("GetDP");
-    GModel::current()->setName(split[1] + ".geo");
-    GModel::current()->setFileName(split[0] + split[1] + ".geo");
-    CTX::instance()->launchSolverAtStartup = num;
-    CTX::instance()->solverToRun = num; // used in ONELAB2
-    return 1;
-  }
-  else if(ext == ".py" || ext == ".PY" ||
-          ext == ".m" || ext == ".M" ||
-          ext == ".exe" || ext == ".EXE"){
-    int num = defineSolver(split[1]);
-    opt_solver_executable(num, GMSH_SET, fileName);
-    CTX::instance()->launchSolverAtStartup = num;
-    CTX::instance()->solverToRun = num; // used in ONELAB2
-    return 1;
-  }
 #endif
   else {
     CTX::instance()->geom.draw = 1;
@@ -495,9 +503,6 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
 #endif
     else {
       status = GModel::readGEO(fileName);
-#ifdef HAVE_ONELAB2
-      Msg::SetOnelabString("Gmsh/1ModelName", fileName, false);
-#endif
     }
   }
 
@@ -506,6 +511,8 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
   CTX::instance()->geom.draw = 1;
   CTX::instance()->mesh.changed = ENT_ALL;
 
+  if(importPhysicalsInOnelab) Msg::ImportPhysicalGroupsInOnelab();
+
 #if defined(HAVE_FLTK) && defined(HAVE_POST)
   if(FlGui::available()){
     // go directly to the first non-empty step after the one that is requested
@@ -517,8 +524,6 @@ int MergeFile(const std::string &fileName, bool warnIfMissing, bool setWindowTit
   }
 #endif
 
-  Msg::ImportPhysicalsAsOnelabRegions();
-
   if(!status) Msg::Error("Error loading '%s'", fileName.c_str());
   Msg::StatusBar(true, "Done reading '%s'", fileName.c_str());
 
@@ -573,8 +578,10 @@ int MergePostProcessingFile(const std::string &fileName, int showViews,
     GModel *m = new GModel();
     GModel::setCurrent(m);
   }
-  int ret = MergeFile(fileName, warnIfMissing, true,
-                      old->bounds().empty() ? true : false);
+  // FIXME: disabled onelab physical group import for now, as the number of
+  // groups in mesh-bases post-pro files can be different from the # in the
+  // model, which will trigger setChanged(Gmsh), leading undesirable remeshing
+  int ret = MergeFile(fileName, warnIfMissing, old->bounds().empty() ? true : false, false);
   GModel::setCurrent(old);
   old->setVisibility(1);
 
@@ -637,7 +644,6 @@ void ClearProject()
   GModel::current()->setName("");
 #if defined(HAVE_FLTK)
   if(FlGui::available()){
-    FlGui::instance()->setGraphicTitle(GModel::current()->getFileName());
     FlGui::instance()->resetVisibility();
     FlGui::instance()->updateViews(true, true);
     FlGui::instance()->updateFields();
@@ -647,7 +653,7 @@ void ClearProject()
   Msg::ResetErrorCounter();
 }
 
-void OpenProject(const std::string &fileName, bool setWindowTitle)
+void OpenProject(const std::string &fileName)
 {
   if(CTX::instance()->lock) {
     Msg::Info("I'm busy! Ask me that later...");
@@ -675,11 +681,13 @@ void OpenProject(const std::string &fileName, bool setWindowTitle)
   gmsh_yysymbols.clear();
   gmsh_yystringsymbols.clear();
   std::map<std::string, std::vector<double> > cln(Msg::GetCommandLineNumbers());
-  for(std::map<std::string, std::vector<double> >::iterator it = cln.begin(); it != cln.end(); it++)
+  for(std::map<std::string, std::vector<double> >::iterator it = cln.begin();
+      it != cln.end(); it++)
     gmsh_yysymbols[it->first].value = it->second;
   std::map<std::string, std::string> cls(Msg::GetCommandLineStrings());
-  for(std::map<std::string, std::string>::iterator it = cls.begin(); it != cls.end(); it++)
-    gmsh_yystringsymbols[it->first] = it->second;
+  for(std::map<std::string, std::string>::iterator it = cls.begin();
+      it != cls.end(); it++)
+    gmsh_yystringsymbols[it->first] = std::vector<std::string>(1, it->second);
   FunctionManager::Instance()->clear();
 #endif
 
@@ -687,7 +695,7 @@ void OpenProject(const std::string &fileName, bool setWindowTitle)
   ResetTemporaryBoundingBox();
 
   // merge the file
-  MergeFile(fileName, false, setWindowTitle);
+  MergeFile(fileName, false);
 
   // fill recent opened file list
   std::vector<std::string> tmp = CTX::instance()->recentFiles;
