@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _OPTIONS_H_
 #define _OPTIONS_H_
@@ -74,6 +74,17 @@ std::string opt_solver_name6(OPT_ARGS_STR);
 std::string opt_solver_name7(OPT_ARGS_STR);
 std::string opt_solver_name8(OPT_ARGS_STR);
 std::string opt_solver_name9(OPT_ARGS_STR);
+std::string opt_solver_extension(OPT_ARGS_STR);
+std::string opt_solver_extension0(OPT_ARGS_STR);
+std::string opt_solver_extension1(OPT_ARGS_STR);
+std::string opt_solver_extension2(OPT_ARGS_STR);
+std::string opt_solver_extension3(OPT_ARGS_STR);
+std::string opt_solver_extension4(OPT_ARGS_STR);
+std::string opt_solver_extension5(OPT_ARGS_STR);
+std::string opt_solver_extension6(OPT_ARGS_STR);
+std::string opt_solver_extension7(OPT_ARGS_STR);
+std::string opt_solver_extension8(OPT_ARGS_STR);
+std::string opt_solver_extension9(OPT_ARGS_STR);
 std::string opt_solver_executable(OPT_ARGS_STR);
 std::string opt_solver_executable0(OPT_ARGS_STR);
 std::string opt_solver_executable1(OPT_ARGS_STR);
@@ -392,7 +403,6 @@ double opt_geometry_match_geom_and_mesh(OPT_ARGS_NUM);
 double opt_mesh_label_sampling(OPT_ARGS_NUM);
 double opt_mesh_optimize(OPT_ARGS_NUM);
 double opt_mesh_optimize_netgen(OPT_ARGS_NUM);
-double opt_mesh_remove_4_triangles(OPT_ARGS_NUM);
 double opt_mesh_refine_steps(OPT_ARGS_NUM);
 double opt_mesh_normals(OPT_ARGS_NUM);
 double opt_mesh_num_sub_edges(OPT_ARGS_NUM);
@@ -423,6 +433,7 @@ double opt_mesh_tetrahedra(OPT_ARGS_NUM);
 double opt_mesh_hexahedra(OPT_ARGS_NUM);
 double opt_mesh_prisms(OPT_ARGS_NUM);
 double opt_mesh_pyramids(OPT_ARGS_NUM);
+double opt_mesh_trihedra(OPT_ARGS_NUM);
 double opt_mesh_surfaces_edges(OPT_ARGS_NUM);
 double opt_mesh_surfaces_faces(OPT_ARGS_NUM);
 double opt_mesh_volumes_edges(OPT_ARGS_NUM);
@@ -449,11 +460,11 @@ double opt_mesh_msh_file_partitioned(OPT_ARGS_NUM);
 double opt_mesh_partition_hex_weight(OPT_ARGS_NUM);
 double opt_mesh_partition_pri_weight(OPT_ARGS_NUM);
 double opt_mesh_partition_pyr_weight(OPT_ARGS_NUM);
+double opt_mesh_partition_trih_weight(OPT_ARGS_NUM);
 double opt_mesh_partition_qua_weight(OPT_ARGS_NUM);
 double opt_mesh_partition_tet_weight(OPT_ARGS_NUM);
 double opt_mesh_partition_tri_weight(OPT_ARGS_NUM);
 double opt_mesh_binary(OPT_ARGS_NUM);
-double opt_mesh_bunin(OPT_ARGS_NUM);
 double opt_mesh_lloyd(OPT_ARGS_NUM);
 double opt_mesh_smooth_cross_field(OPT_ARGS_NUM);
 double opt_mesh_bdf_field_format(OPT_ARGS_NUM);
@@ -463,11 +474,12 @@ double opt_mesh_algo3d(OPT_ARGS_NUM);
 double opt_mesh_algo_recombine(OPT_ARGS_NUM);
 double opt_mesh_recombine_all(OPT_ARGS_NUM);
 double opt_mesh_recombine3d_all(OPT_ARGS_NUM);
+double opt_mesh_recombine3d_level(OPT_ARGS_NUM);
+double opt_mesh_recombine3d_conformity(OPT_ARGS_NUM);
 double opt_mesh_flexible_transfinite(OPT_ARGS_NUM);
 double opt_mesh_do_recombination_test(OPT_ARGS_NUM);
 double opt_mesh_recombination_test_start(OPT_ARGS_NUM);
 double opt_mesh_recombination_no_greedy_strat(OPT_ARGS_NUM);
-double opt_mesh_recombination_new_strat(OPT_ARGS_NUM);
 double opt_mesh_remesh_algo(OPT_ARGS_NUM);
 double opt_mesh_remesh_param(OPT_ARGS_NUM);
 double opt_mesh_algo_subdivide(OPT_ARGS_NUM);
@@ -503,6 +515,7 @@ double opt_mesh_nb_tetrahedra(OPT_ARGS_NUM);
 double opt_mesh_nb_hexahedra(OPT_ARGS_NUM);
 double opt_mesh_nb_prisms(OPT_ARGS_NUM);
 double opt_mesh_nb_pyramids(OPT_ARGS_NUM);
+double opt_mesh_nb_trihedra(OPT_ARGS_NUM);
 double opt_mesh_cpu_time(OPT_ARGS_NUM);
 double opt_mesh_partition_partitioner(OPT_ARGS_NUM);
 double opt_mesh_partition_num(OPT_ARGS_NUM);
@@ -527,9 +540,11 @@ double opt_mesh_partition_metis_edge_matching(OPT_ARGS_NUM);
 double opt_mesh_partition_metis_refine_algorithm(OPT_ARGS_NUM);
 double opt_mesh_clip(OPT_ARGS_NUM);
 double opt_mesh_ignore_part_bound(OPT_ARGS_NUM);
+double opt_mesh_preserve_numbering_msh2(OPT_ARGS_NUM);
 double opt_solver_listen(OPT_ARGS_NUM);
 double opt_solver_timeout(OPT_ARGS_NUM);
 double opt_solver_plugins(OPT_ARGS_NUM);
+double opt_solver_auto_load_database(OPT_ARGS_NUM);
 double opt_solver_auto_save_database(OPT_ARGS_NUM);
 double opt_solver_auto_archive_output_files(OPT_ARGS_NUM);
 double opt_solver_auto_check(OPT_ARGS_NUM);
@@ -652,6 +667,7 @@ double opt_view_draw_tetrahedra(OPT_ARGS_NUM);
 double opt_view_draw_hexahedra(OPT_ARGS_NUM);
 double opt_view_draw_prisms(OPT_ARGS_NUM);
 double opt_view_draw_pyramids(OPT_ARGS_NUM);
+double opt_view_draw_trihedra(OPT_ARGS_NUM);
 double opt_view_draw_scalars(OPT_ARGS_NUM);
 double opt_view_draw_vectors(OPT_ARGS_NUM);
 double opt_view_draw_tensors(OPT_ARGS_NUM);
@@ -747,6 +763,7 @@ unsigned int opt_mesh_color_tetrahedra(OPT_ARGS_COL);
 unsigned int opt_mesh_color_hexahedra(OPT_ARGS_COL);
 unsigned int opt_mesh_color_prisms(OPT_ARGS_COL);
 unsigned int opt_mesh_color_pyramid(OPT_ARGS_COL);
+unsigned int opt_mesh_color_trihedron(OPT_ARGS_COL);
 unsigned int opt_mesh_color_tangents(OPT_ARGS_COL);
 unsigned int opt_mesh_color_normals(OPT_ARGS_COL);
 unsigned int opt_mesh_color_0(OPT_ARGS_COL);
@@ -777,6 +794,7 @@ unsigned int opt_view_color_tetrahedra(OPT_ARGS_COL);
 unsigned int opt_view_color_hexahedra(OPT_ARGS_COL);
 unsigned int opt_view_color_prisms(OPT_ARGS_COL);
 unsigned int opt_view_color_pyramids(OPT_ARGS_COL);
+unsigned int opt_view_color_trihedra(OPT_ARGS_COL);
 unsigned int opt_view_color_tangents(OPT_ARGS_COL);
 unsigned int opt_view_color_normals(OPT_ARGS_COL);
 unsigned int opt_view_color_text2d(OPT_ARGS_COL);
