@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include "GmshConfig.h"
 #include "Gmsh.h"
@@ -225,12 +225,6 @@ bool gmshLocalNetworkClient::receiveMessage(gmshLocalNetworkClient *master)
 	}
         set(p);
       }
-      else if(ptype == "region"){
-        onelab::region p; p.fromChar(message); set(p);
-      }
-      else if(ptype == "function"){
-        onelab::function p; p.fromChar(message); set(p);
-      }
       else
         Msg::Error("Unknown ONELAB parameter type: %s", ptype.c_str());
     }
@@ -251,14 +245,6 @@ bool gmshLocalNetworkClient::receiveMessage(gmshLocalNetworkClient *master)
         std::vector<onelab::string> par; get(par, name);
         if(par.size() == 1) reply = par[0].toChar();
       }
-      else if(ptype == "region"){
-        std::vector<onelab::region> par; get(par, name);
-        if(par.size() == 1) reply = par[0].toChar();
-      }
-      else if(ptype == "function"){
-        std::vector<onelab::function> par; get(par, name);
-        if(par.size() == 1) reply = par[0].toChar();
-      }
       else
         Msg::Error("Unknown ONELAB parameter type in query: %s", ptype.c_str());
 
@@ -292,16 +278,6 @@ bool gmshLocalNetworkClient::receiveMessage(gmshLocalNetworkClient *master)
         for(std::vector<onelab::string>::iterator it = strings.begin();
             it != strings.end(); it++) replies.push_back((*it).toChar());
       }
-      else if(ptype == "region"){
-        std::vector<onelab::region> regions; get(regions);
-        for(std::vector<onelab::region>::iterator it = regions.begin();
-            it != regions.end(); it++) replies.push_back((*it).toChar());
-      }
-      else if(ptype == "function"){
-        std::vector<onelab::function> functions; get(functions);
-        for(std::vector<onelab::function>::iterator it = functions.begin();
-            it != functions.end(); it++) replies.push_back((*it).toChar());
-      }
       else
         Msg::Error("Unknown ONELAB parameter type in query: %s", ptype.c_str());
 
@@ -648,6 +624,15 @@ void saveDb(const std::string &fileName)
   }
   else
     Msg::Error("Could not save database '%s'", fileName.c_str());
+
+  // test
+  fp = Fopen((fileName + ".json").c_str(), "wb");
+  if(fp){
+    std::string json;
+    onelab::server::instance()->toJSON(json);
+    fwrite(json.c_str(), sizeof(char), json.size(), fp);
+    fclose(fp);
+  }
 }
 
 void archiveOutputFiles(const std::string &fileName)
@@ -763,22 +748,6 @@ void resetDb(bool runGmshClient)
       persistentStrings.push_back(allStrings[i]);
   }
 
-  // TODO FIXME: this will be removed once the new stable version of getdp is
-  // released
-  for(onelab::server::citer it = onelab::server::instance()->firstClient();
-      it != onelab::server::instance()->lastClient(); it++){
-    onelab::client *c = *it;
-    std::vector<onelab::number> ps;
-    c->get(ps, c->getName() + "/UseCommandLine");
-    if(ps.size()) persistentNumbers.push_back(ps[0]);
-    c->get(ps, c->getName() + "/GuessModelName");
-    if(ps.size()) persistentNumbers.push_back(ps[0]);
-    std::vector<onelab::string> ps2;
-    c->get(ps2, c->getName() + "/FileExtension");
-    if(ps2.size()) persistentStrings.push_back(ps2[0]);
-  }
-  // END TODO
-
   // clear the db
   onelab::server::instance()->clear();
 
@@ -788,13 +757,13 @@ void resetDb(bool runGmshClient)
     onelabUtils::runGmshClient("reset", CTX::instance()->solver.autoMesh);
 
   for(unsigned int i = 0; i < persistentNumbers.size(); i++){
-    Msg::Info("Restoring persistent parameter %s",
-              persistentNumbers[i].getName().c_str());
+    Msg::Debug("Restoring persistent parameter %s",
+               persistentNumbers[i].getName().c_str());
     onelab::server::instance()->set(persistentNumbers[i]);
   }
   for(unsigned int i = 0; i < persistentStrings.size(); i++){
-    Msg::Info("Restoring persistent parameter %s",
-              persistentStrings[i].getName().c_str());
+    Msg::Debug("Restoring persistent parameter %s",
+               persistentStrings[i].getName().c_str());
     onelab::server::instance()->set(persistentStrings[i]);
   }
 }
@@ -848,8 +817,9 @@ void solver_batch_cb(void *data)
   }
 
   // load db
-  if(CTX::instance()->solver.autoSaveDatabase){
-    std::string db = SplitFileName(GModel::current()->getFileName())[0] + "onelab.db";
+  if(CTX::instance()->solver.autoLoadDatabase){
+    std::vector<std::string> split = SplitFileName(GModel::current()->getFileName());
+    std::string db = split[0] + split[1] + ".db";
     if(!StatFile(db)) loadDb(db);
   }
 
@@ -877,7 +847,8 @@ void solver_batch_cb(void *data)
 
   if(CTX::instance()->solver.autoSaveDatabase ||
      CTX::instance()->solver.autoArchiveOutputFiles){
-    std::string db = SplitFileName(GModel::current()->getFileName())[0] + "onelab.db";
+    std::vector<std::string> split = SplitFileName(GModel::current()->getFileName());
+    std::string db = split[0] + split[1] + ".db";
     if(CTX::instance()->solver.autoArchiveOutputFiles) archiveOutputFiles(db);
     if(CTX::instance()->solver.autoSaveDatabase) saveDb(db);
   }
