@@ -1,4 +1,4 @@
-// ONELAB - Copyright (C) 2011-2014 ULg-UCL
+// ONELAB - Copyright (C) 2011-2016 ULg-UCL
 //
 // Permission is hereby granted, free of charge, to any person
 // obtaining a copy of this software and associated documentation
@@ -23,7 +23,7 @@
 // OF THIS SOFTWARE.
 //
 // Please report all bugs and problems to the public mailing list
-// <gmsh@geuz.org>.
+// <gmsh@onelab.info>.
 
 #ifndef _ONELAB_H_
 #define _ONELAB_H_
@@ -38,19 +38,10 @@
 #include <sstream>
 #include "GmshSocket.h"
 
-#ifdef HAVE_ONELAB2
-#include "NetworkUtils.h"
-#include "OnelabAttributes.h"
-#endif
-
 namespace onelab{
 
   // The base parameter class.
-#ifdef HAVE_ONELAB2
-  class parameter : public OnelabAttr{
-#else
   class parameter {
-#endif
   private:
     // the name of the parameter, including its '/'-separated path in the
     // parameter hierarchy. Parameters or subpaths can start with numbers to
@@ -276,8 +267,7 @@ namespace onelab{
       type = getNextToken(msg, first);
       name = getNextToken(msg, first);
     }
-    static bool fromFile(std::vector<std::string> &msg,
-                         FILE *fp)
+    static bool fromFile(std::vector<std::string> &msg, FILE *fp)
     {
       msg.clear();
       char tmp[1000];
@@ -293,8 +283,7 @@ namespace onelab{
       }
       return true;
     }
-    static bool toFile(const std::vector<std::string> &msg,
-                       FILE *fp,
+    static bool toFile(const std::vector<std::string> &msg, FILE *fp,
                        const std::string &creator)
     {
       time_t now;
@@ -309,86 +298,36 @@ namespace onelab{
       }
       return true;
     }
-#ifdef HAVE_ONELAB2
-    static UInt16 attributeType() {return 0x05;}
-    virtual inline UInt16 getAttributeType() const {return this->attributeType();}
-    virtual inline UInt16 getAttributeLength() const {
-      UInt16 len = _name.length()+_label.length()+_help.length()+10;
-      for(std::map<std::string, bool>::const_iterator it = getClients().begin(); it != getClients().end(); it++)
-        len += it->first.size()+2;
-      for(std::map<std::string, std::string>::const_iterator it = _attributes.begin(); it != _attributes.end(); it++)
-        len += it->first.size()+it->second.size()+2;
-      return len;
-    }
-    virtual UInt8 *encodeAttribute(UInt8 *dst)
+    virtual std::string toJSON() const
     {
-      dst = encode(dst, getAttributeType());
-      dst = encode(dst, getAttributeLength());
-
-      dst = encode(dst, (UInt8 *)_name.c_str(), this->_name.length()+1);
-      dst = encode(dst, (UInt8 *)_label.c_str(), this->_label.length()+1);
-      dst = encode(dst, (UInt8 *)_help.c_str(), this->_help.length()+1);
-      dst = encode(dst, (UInt8)_readOnly);
-      dst = encode(dst, (UInt8)_neverChanged);
-      dst = encode(dst, (UInt8)_visible);
-
-      dst = encode(dst, (UInt16)_attributes.size());
-      for(std::map<std::string, std::string>::const_iterator it = _attributes.begin(); it != _attributes.end(); it++) {
-        dst = encode(dst, (UInt8 *)it->first.c_str(), it->first.size()+1);
-        dst = encode(dst, (UInt8 *)it->second.c_str(), it->second.size()+1);
+      std::ostringstream sstream;
+      sstream
+        << "\"type\":\"" << getType() << "\""
+        << ", \"version\":\"" << version() << "\""
+        << ", \"name\":\"" << sanitize(getName()) << "\""
+        << ", \"label\":\"" << sanitize(getLabel()) << "\""
+        << ", \"help\":\"" << sanitize(getHelp()) << "\""
+        << ", \"neverChanged\":" << (getNeverChanged() ? "true" : "false") << "\""
+        << ", \"visible\":" << (getVisible() ? "true" : "false") << "\""
+        << ", \"readOnly\":" << (getReadOnly() ? "true" : "false") << "\"";
+      sstream << ", \"attributes\":{ ";
+      for(std::map<std::string, std::string>::const_iterator it = _attributes.begin();
+          it != _attributes.end(); it++){
+        if(it != _attributes.begin()) sstream << ", ";
+        sstream << "\"" << sanitize(it->first) << "\":\""
+                << sanitize(it->second) << "\"";
       }
-
-      dst = encode(dst, (UInt16)_clients.size());
-      for(std::map<std::string, bool>::const_iterator it = getClients().begin(); it != getClients().end(); it++) {
-        dst = encode(dst, (UInt8 *)it->first.c_str(), it->first.size()+1);
-        dst = encode(dst, (UInt8)it->second);
+      sstream << " }";
+      sstream << ", \"clients\":{ ";
+      for(std::map<std::string, bool>::const_iterator it = getClients().begin();
+          it != getClients().end(); it++){
+        if(it != getClients().begin()) sstream << ", ";
+        sstream << "\"" << sanitize(it->first) << "\":"
+                << (it->second ? "true" : "false");
       }
-
-      return dst;
+      sstream << " }";
+      return sstream.str();
     }
-    virtual UInt8 *parseAttribute(UInt8 *src, UInt32 length)
-    {
-      UInt8 tmp;
-      UInt16 n;
-
-      src = parse(src, _name, '\0');
-      src = parse(src, _label, '\0');
-      src = parse(src, _help, '\0');
-      src = parse(src, tmp);
-      this->_readOnly = (bool)tmp;
-      src = parse(src, tmp);
-      this->_neverChanged = (bool)tmp;
-      src = parse(src, tmp);
-      this->_visible = (bool)tmp;
-
-      src = parse(src, n);
-      for(int i=0; i<n; i++) {
-        std::string key, value;
-        src = parse(src, key, '\0');
-        src = parse(src, value, '\0');
-        setAttribute(key, value);
-      }
-
-      src = parse(src, n);
-      for(int i=0; i<n; i++) {
-        std::string client;
-        src = parse(src, client, '\0');
-        src = parse(src, tmp);
-        addClient(client, (bool)tmp);
-      }
-
-      return src;
-    }
-    void showAttribute() const
-    {
-      std::cout << "Name: " << getName() << std::endl
-        << "Label: " << getLabel() << std::endl
-        << "Help: " << getHelp() << std::endl
-        << "Never changed: " << getNeverChanged() << std::endl
-        << "Changed: " << getChanged() << std::endl
-        << "Visible: " << getVisible() << std::endl;
-    }
-#endif
   };
 
   class parameterLessThan{
@@ -400,8 +339,8 @@ namespace onelab{
   };
 
   // The number class. Numbers are stored internally as double precision real
-  // numbers. All more complicated types (complex numbers, vectors, etc.) are
-  // supposed to be either exchanged as strings or encapsulated in functions.
+  // numbers. All more complicated types (complex numbers, vectors, expressions,
+  // functions, etc.) are supposed to be exchanged as strings.
   class number : public parameter{
   private:
     double _value, _min, _max, _step;
@@ -509,73 +448,32 @@ namespace onelab{
       }
       return pos;
     }
-#ifdef HAVE_ONELAB2
-    static UInt16 attributeType() {return 0x06;}
-    virtual inline UInt16 getAttributeType() const {return this->attributeType();}
-    virtual inline UInt16 getAttributeLength() const {
-      UInt16 len = parameter::getAttributeLength()+sizeof(double)*4+8+sizeof(double)*_choices.size();
-      for(std::map<double, std::string>::const_iterator it = _valueLabels.begin(); it != _valueLabels.end(); it++)
-        len += it->second.size()+1+sizeof(double);
-      return len;
-    }
-    UInt8 *encodeAttribute(UInt8 *dst)
+    std::string toJSON() const
     {
-      dst = parameter::encodeAttribute(dst);
-
-      dst = encode(dst, _value);
-      dst = encode(dst, _min);
-      dst = encode(dst, _max);
-      dst = encode(dst, _step);
-      dst = encode(dst, (UInt32)_index);
-
-      dst = encode(dst, (UInt16)_choices.size());
-      for(unsigned int i = 0; i < _choices.size(); i++)
-        dst = encode(dst, _choices[i]);
-
-      dst = encode(dst, (UInt16)_valueLabels.size());
-      for(std::map<double, std::string>::const_iterator it = _valueLabels.begin(); it != _valueLabels.end(); it++) {
-        dst = encode(dst, it->first);
-        dst = encode(dst, (UInt8 *)it->second.c_str(), it->second.size()+1);
+      std::ostringstream sstream;
+      sstream.precision(16);
+      sstream
+        << "{ " << parameter::toJSON()
+        << ", \"value\":" << _value
+        << ", \"min\":" << _min
+        << ", \"max\":" << _max
+        << ", \"step\":" << _step
+        << ", \"index\":" << _index
+        << ", \"choices\":[ ";
+      for(unsigned int i = 0; i < _choices.size(); i++){
+        if(i) sstream << ", ";
+        sstream << _choices[i];
       }
-
-      return dst;
-    }
-    UInt8 *parseAttribute(UInt8 *src, UInt32 length)
-    {
-      UInt16 n;
-
-      src = parameter::parseAttribute(src, length);
-
-      src = parse(src, _value);
-      src = parse(src, _min);
-      src = parse(src, _max);
-      src = parse(src, _step);
-      src = parse(src, *(UInt32 *)&_index);
-
-      src = parse(src, n);
-      _choices.resize(n);
-      for(unsigned int i = 0; i < n; i++)
-        src = parse(src, _choices[i]);
-
-      src = parse(src, n);
-      for(int i=0; i<n; i++) {
-        double value;
-        std::string label;
-        src = parse(src, value);
-        src = parse(src, label, '\0');
-        setValueLabel(value, label);
+      sstream << " ]";
+      sstream << ", \"valueLabels\":{ ";
+      for(std::map<double, std::string>::const_iterator it = _valueLabels.begin();
+          it != _valueLabels.end(); it++){
+        if(it != _valueLabels.begin()) sstream << ", ";
+        sstream << "\"" << sanitize(it->second) << "\":" << it->first;
       }
-
-      return src;
+      sstream << " } }";
+      return sstream.str();
     }
-    void showAttribute() const
-    {
-      parameter::showAttribute();
-      std::cout << "Value: " << this->_value << std::endl
-        << "Min: " << this->_min << std::endl
-        << "Max: " << this->_max << std::endl;
-    }
-#endif
   };
 
   // The string class. A string has a mutable "kind": we do not derive
@@ -640,337 +538,21 @@ namespace onelab{
         _choices[i] = getNextToken(msg, pos);
       return pos;
     }
-#ifdef HAVE_ONELAB2
-    static UInt16 attributeType() {return 0x07;}
-    virtual inline UInt16 getAttributeType() const {return this->attributeType();}
-    virtual inline UInt16 getAttributeLength() const
-    {
-      UInt16 len =  parameter::getAttributeLength();
-      len += _value.size()+_kind.size()+4;
-      for(unsigned int i = 0; i < _choices.size(); i++)
-        len += _choices[i].size()+1;
-      return len;
-    }
-    UInt8 *encodeAttribute(UInt8 *dst)
-    {
-      dst = parameter::encodeAttribute(dst);
-
-      dst = encode(dst, (UInt8 *)_value.c_str(), _value.size()+1);
-      dst = encode(dst, (UInt8 *)_kind.c_str(), _kind.size()+1);
-
-      dst = encode(dst, (UInt16)_choices.size());
-      for(unsigned int i = 0; i < _choices.size(); i++)
-        dst = encode(dst, (UInt8 *)_choices[i].c_str(), _choices[i].size()+1);
-
-      return dst;
-     }
-    UInt8 *parseAttribute(UInt8 *src, UInt32 length)
-    {
-      UInt16 n;
-      src = parameter::parseAttribute(src, length);
-      src = parse(src, _value, '\0');
-      src = parse(src, _kind, '\0');
-
-      src = parse(src, n);
-      _choices.resize(n);
-      for(unsigned int i=0; i<n; i++) {
-        src = parse(src, _choices[i], '\0');
-      }
-
-      return src;
-    }
-    void showAttribute() const
-    {
-      parameter::showAttribute();
-      std::cout << "Value: " << this->_value << std::endl;
-    }
-#endif
-  };
-
-  // The region class. A region can be any kind of geometrical entity,
-  // represented as identifiers of physical regions. Operations on regions will
-  // include union, intersection, etc.
-  class region : public parameter{
-  private:
-    std::set<std::string> _value;
-    // optional geometrical dimension
-    int _dimension;
-    std::vector<std::set<std::string> > _choices;
-  public:
-    region(const std::string &name="",
-           const std::set<std::string> &value = std::set<std::string>(),
-           const std::string &label="", const std::string &help="")
-      : parameter(name, label, help), _value(value), _dimension(-1) {}
-    region(const std::string &name, const std::string &value,
-           const std::string &label="", const std::string &help="")
-      : parameter(name, label, help), _dimension(-1)
-    {
-      if(value.size()) _value.insert(value);
-    }
-    void setValue(const std::set<std::string> &value){ _value = value; }
-    void setDimension(int dim){ _dimension = dim; }
-    void setChoices(const std::vector<std::set<std::string> > &choices)
-    {
-      _choices = choices;
-    }
-    std::string getType() const { return "region"; }
-    const std::set<std::string> &getValue() const { return _value; }
-    int getDimension() const { return _dimension; }
-    const std::vector<std::set<std::string> > &getChoices() const
-    {
-      return _choices;
-    }
-    void update(const region &p)
-    {
-      addClients(p.getClients());
-      setLabel(p.getLabel());
-      setHelp(p.getHelp());
-      setAttributes(p.getAttributes());
-      if(p.getValue() != getValue()){
-        setValue(p.getValue());
-        setChanged(true);
-      }
-      setDimension(p.getDimension());
-      setChoices(p.getChoices());
-      if(getNeverChanged()) setChanged(false);
-    }
-    std::string toChar() const
+    std::string toJSON() const
     {
       std::ostringstream sstream;
-      sstream << parameter::toChar() << _value.size() << charSep();
-      for(std::set<std::string>::const_iterator it = _value.begin();
-          it != _value.end(); it++)
-        sstream << sanitize(*it) << charSep();
-      sstream << _dimension << charSep();
-      sstream << _choices.size() << charSep();
+      sstream
+        << "{ " << parameter::toJSON()
+        << ", \"value\":\"" << sanitize(_value) << "\""
+        << ", \"kind\":\"" << sanitize(_kind) <<  "\""
+        << ", \"choices\":[ ";
       for(unsigned int i = 0; i < _choices.size(); i++){
-        sstream << _choices[i].size() << charSep();
-        for(std::set<std::string>::const_iterator it = _choices[i].begin();
-            it != _choices[i].end(); it++)
-          sstream << sanitize(*it) << charSep();
+        if(i) sstream << ", ";
+        sstream << "\"" << sanitize(_choices[i]) << "\"";
       }
+      sstream << " ] }";
       return sstream.str();
     }
-    std::string::size_type fromChar(const std::string &msg)
-    {
-      std::string::size_type pos = parameter::fromChar(msg);
-      if(!pos) return 0;
-      int n = atoi(getNextToken(msg, pos).c_str());
-      for(int i = 0; i < n; i++)
-        _value.insert(getNextToken(msg, pos));
-      setDimension(atoi(getNextToken(msg, pos).c_str()));
-      _choices.resize(atoi(getNextToken(msg, pos).c_str()));
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        n = atoi(getNextToken(msg, pos).c_str());
-        for(int i = 0; i < n; i++)
-          _choices[i].insert(getNextToken(msg, pos));
-      }
-      return pos;
-    }
-#ifdef HAVE_ONELAB2
-    static UInt16 attributeType() {return 0x08;}
-    virtual inline UInt16 getAttributeType() const {return this->attributeType();}
-    virtual inline UInt16 getAttributeLength() const {
-      UInt16 len = parameter::getAttributeLength();
-      len += 2;
-      for(std::set<std::string>::const_iterator it = _value.begin(); it != _value.end(); it++)
-        len += it->size()+1;
-      len += 4;
-      len += 2;
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        len += 2;
-        for(std::set<std::string>::const_iterator it = _choices[i].begin(); it != _choices[i].end(); it++)
-          len += it->size()+1;
-      }
-      return len;
-    }
-    UInt8 *encodeAttribute(UInt8 *dst)
-    {
-      dst = parameter::encodeAttribute(dst);
-      dst = encode(dst, (UInt16)this->_value.size());
-      for(std::set<std::string>::const_iterator it = _value.begin(); it != _value.end(); it++)
-        dst = encode(dst, (UInt8 *)it->c_str(), it->size()+1);
-      dst = encode(dst, (UInt32)_dimension);
-      dst = encode(dst, (UInt16)_choices.size());
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        dst = encode(dst, (UInt16)_choices[i].size());
-        for(std::set<std::string>::const_iterator it = _choices[i].begin(); it != _choices[i].end(); it++)
-          dst = encode(dst, (UInt8 *)it->c_str(), it->size()+1);
-      }
-      return dst;
-    }
-    UInt8 *parseAttribute(UInt8 *src, UInt32 len)
-    {
-      src = parameter::parseAttribute(src, len);
-      UInt16 m = 0, n = 0;
-      std::string value;
-      src = parse(src, n);
-      for(int i=0; i<n; i++) {
-        src = parse(src, value, '\0');
-      	_value.insert(value);
-      }
-      src = parse(src, *(UInt32 *)&_dimension);
-      src = parse(src, n);
-      _choices.resize(n);
-      for(int i=0; i<n; i++) {
-        src = parse(src, m);
-        for(int j=0; j<m; j++) {
-          src = parse(src, value, '\0');
-          _choices[i].insert(value);
-        }
-      }
-      return src;
-    }
-    void showAttribute() const {}
-#endif
-
-  };
-
-  // The (possibly piece-wise defined on regions) function class. Functions are
-  // entirely client-dependent: they are just represented internally as onelab
-  // strings, defined on onelab regions.
-  class function : public parameter{
-  private:
-    std::map<std::string, std::string> _value;
-    std::vector<std::map<std::string, std::string> > _choices;
-  public:
-    function(const std::string &name="") : parameter(name, "", "") {}
-    function(const std::string &name, const std::map<std::string, std::string> &value,
-             const std::string &label="", const std::string &help="")
-      : parameter(name, label, help), _value(value) {}
-    void setValue(const std::map<std::string, std::string> &value)
-    {
-      _value = value;
-    }
-    void setChoices(const std::vector<std::map<std::string, std::string> > &choices)
-    {
-      _choices = choices;
-    }
-    std::string getType() const { return "function"; }
-    const std::map<std::string, std::string> &getValue() const { return _value; }
-    const std::string getValue(const std::string &region) const
-    {
-      std::map<std::string, std::string>::const_iterator it = _value.find(region);
-      if(it != _value.end()) return it->second;
-      return "";
-    }
-    const std::vector<std::map<std::string, std::string> > &getChoices() const
-    {
-      return _choices;
-    }
-    void update(const function &p)
-    {
-      addClients(p.getClients());
-      setLabel(p.getLabel());
-      setHelp(p.getHelp());
-      setAttributes(p.getAttributes());
-      if(p.getValue() != getValue()){
-        setValue(p.getValue());
-        setChanged(true);
-      }
-      setChoices(p.getChoices());
-      if(getNeverChanged()) setChanged(false);
-    }
-    std::string toChar() const
-    {
-      std::ostringstream sstream;
-      sstream << parameter::toChar() << _value.size() << charSep();
-      for(std::map<std::string, std::string>::const_iterator it = _value.begin();
-          it != _value.end(); it++)
-        sstream << sanitize(it->first) << charSep()
-                << sanitize(it->second) << charSep();
-      sstream << _choices.size() << charSep();
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        sstream << _choices[i].size() << charSep();
-        for(std::map<std::string, std::string>::const_iterator it = _choices[i].begin();
-            it != _choices[i].end(); it++)
-          sstream << sanitize(it->first) << charSep()
-                  << sanitize(it->second) << charSep();
-      }
-      return sstream.str();
-    }
-    std::string::size_type fromChar(const std::string &msg)
-    {
-      std::string::size_type pos = parameter::fromChar(msg);
-      if(!pos) return 0;
-      int n = atoi(getNextToken(msg, pos).c_str());
-      for(int i = 0; i < n; i++){
-        std::string key = getNextToken(msg, pos);
-        _value[key] = getNextToken(msg, pos);
-      }
-      _choices.resize(atoi(getNextToken(msg, pos).c_str()));
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        n = atoi(getNextToken(msg, pos).c_str());
-        for(int i = 0; i < n; i++){
-          std::string key = getNextToken(msg, pos);
-          _choices[i][key] = getNextToken(msg, pos);
-        }
-      }
-      return pos;
-    }
-#ifdef HAVE_ONELAB2
-    static UInt16 attributeType() {return 0x09;}
-    virtual inline UInt16 getAttributeType() const {return this->attributeType();}
-    virtual inline UInt16 getAttributeLength() const {
-      UInt16 len = parameter::getAttributeLength();
-      len += 2;
-      for(std::map<std::string, std::string>::const_iterator it = _value.begin();
-          it != _value.end(); it++)
-        len += 2+it->first.size()+it->second.size();
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        len += 2;
-        for(std::map<std::string, std::string>::const_iterator it = _choices[i].begin();
-            it != _choices[i].end(); it++) {
-          len += 2+it->first.size()+it->second.size();
-        }
-      }
-      return len;
-    }
-    UInt8 *encodeAttribute(UInt8 *dst)
-    {
-      dst = parameter::encodeAttribute(dst),
-      dst = encode(dst, (UInt16)this->_value.size());
-      for(std::map<std::string, std::string>::const_iterator it = _value.begin();
-          it != _value.end(); it++) {
-        dst = encode(dst, (UInt8 *)it->first.c_str(), it->first.size()+1);
-        dst = encode(dst, (UInt8 *)it->second.c_str(), it->second.size()+1);
-      }
-      dst = encode(dst, (UInt16)_choices.size());
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        dst = encode(dst, (UInt16)_choices[i].size());
-        for(std::map<std::string, std::string>::const_iterator it = _choices[i].begin();
-            it != _choices[i].end(); it++) {
-          dst = encode(dst, (UInt8 *)it->first.c_str(), it->first.size()+1);
-          dst = encode(dst, (UInt8 *)it->second.c_str(), it->second.size()+1);
-        }
-      }
-      return dst;
-    }
-    UInt8 *parseAttribute(UInt8 *src, UInt32 len)
-    {
-      src = parameter::parseAttribute(src, len);
-      UInt16 m = 0, n = 0;
-      std::string key, value;
-      src = parse(src, n);
-      for(int i=0; i<n; i++) {
-        src = parse(src, key, '\0');
-        src = parse(src, value, '\0');
-      	_value[key] = value;
-      }
-      src = parse(src, n);
-      _choices.resize(n);
-      for(int i=0; i<n; i++) {
-        src = parse(src, m);
-        for(int j=0; j<m; j++) {
-          src = parse(src, key, '\0');
-          src = parse(src, value, '\0');
-          _choices[i][key] = value;
-        }
-      }
-      return src;
-    }
-    void showAttribute() const {}
-#endif
   };
 
   // The parameter space, i.e., the set of parameters stored and handled by the
@@ -979,8 +561,6 @@ namespace onelab{
   private:
     std::set<number*, parameterLessThan> _numbers;
     std::set<string*, parameterLessThan> _strings;
-    std::set<region*, parameterLessThan> _regions;
-    std::set<function*, parameterLessThan> _functions;
     // delete a parameter from the parameter space
     template <class T> bool _clear(const std::string &name,
                                    const std::string &client,
@@ -1028,9 +608,6 @@ namespace onelab{
       }
       else{
         T* newp = new T(p);
-#ifdef HAVE_ONELAB2
-        newp->isInDatabase(true);
-#endif
         if(client.size()) newp->addClient(client, true);
         ps.insert(newp);
       }
@@ -1060,7 +637,8 @@ namespace onelab{
       }
       return true;
     }
-    template <class T> T* _getPtr(std::string name, const std::string client, std::set<T*, parameterLessThan> ps)
+    template <class T> T* _getPtr(std::string name, const std::string client,
+                                  std::set<T*, parameterLessThan> ps)
     {
       T tmp(name);
       typename std::set<T*, parameterLessThan>::iterator it = ps.find(&tmp);
@@ -1074,8 +652,6 @@ namespace onelab{
     {
       ps.insert(_numbers.begin(), _numbers.end());
       ps.insert(_strings.begin(), _strings.end());
-      ps.insert(_regions.begin(), _regions.end());
-      ps.insert(_functions.begin(), _functions.end());
     }
   public:
     parameterSpace(){}
@@ -1090,50 +666,36 @@ namespace onelab{
           delete *it;
         _numbers.clear();
         _strings.clear();
-        _regions.clear();
-        _functions.clear();
       }
       else{
         bool done = _clear(name, client, _numbers);
         if(!done) done = _clear(name, client, _strings);
-        if(!done) done = _clear(name, client, _regions);
-        if(!done) done = _clear(name, client, _functions);
       }
     }
     bool set(const number &p,
              const std::string &client=""){ return _set(p, client, _numbers); }
     bool set(const string &p,
              const std::string &client=""){ return _set(p, client, _strings); }
-    bool set(const region &p,
-             const std::string &client=""){ return _set(p, client, _regions); }
-    bool set(const function &p,
-             const std::string &client=""){ return _set(p, client, _functions); }
     bool get(std::vector<number> &ps, const std::string &name="",
              const std::string &client=""){ return _get(ps, name, client, _numbers); }
     bool get(std::vector<onelab::string> &ps, const std::string &name="",
              const std::string &client=""){ return _get(ps, name, client, _strings); }
-    bool get(std::vector<region> &ps, const std::string &name="",
-             const std::string &client=""){ return _get(ps, name, client, _regions); }
-    bool get(std::vector<function> &ps, const std::string &name="",
-             const std::string &client=""){ return _get(ps, name, client, _functions); }
     void getPtr(number **ptr, const std::string name, const std::string client="")
-      {*ptr = _getPtr(name, client, _numbers);}
+    {
+      *ptr = _getPtr(name, client, _numbers);
+    }
     void getPtr(string **ptr, const std::string name, const std::string client="")
-      {*ptr = _getPtr(name, client, _strings);}
-    void getPtr(region **ptr, const std::string name, const std::string client="")
-      {*ptr = _getPtr(name, client, _regions);}
-    void getPtr(function **ptr, const std::string name, const std::string client="")
-      {*ptr = _getPtr(name, client, _functions);}
+    {
+      *ptr = _getPtr(name, client, _strings);
+    }
     void getAllParameters(std::set<parameter*, parameterLessThan> &ps) const
     {
       ps.insert(_numbers.begin(), _numbers.end());
       ps.insert(_strings.begin(), _strings.end());
-      ps.insert(_regions.begin(), _regions.end());
-      ps.insert(_functions.begin(), _functions.end());
     }
     unsigned int getNumParameters()
     {
-      return (int)(_numbers.size() + _strings.size() + _regions.size() + _functions.size());
+      return (int)(_numbers.size() + _strings.size());
     }
     // check if at least one parameter depends on the given client
     bool hasClient(const std::string &client) const
@@ -1197,17 +759,34 @@ namespace onelab{
         else if(type == "string"){
           onelab::string p; p.fromChar(msg[i]); set(p, client);
         }
-        else if(type == "region"){
-          onelab::region p; p.fromChar(msg[i]); set(p, client);
-        }
-        else if(type == "function"){
-          onelab::function p; p.fromChar(msg[i]); set(p, client);
-        }
         else
           return false;
       }
       return true;
     }
+    void toJSON(std::string &json, const std::string &creator="",
+                const std::string &client="") const
+    {
+      time_t now;
+      time(&now);
+      std::string t(ctime(&now));
+      t.resize(t.size() - 1);
+      json.clear();
+      json += "{ \"onelab\":{\n";
+      json += "  \"creator\":\"" + creator + "\",\n";
+      json += "  \"date\":\"" + t + "\",\n";
+      json += "  \"parameters\":[ \n";
+      std::set<parameter*, parameterLessThan> ps;
+      _getAllParameters(ps);
+      for(std::set<parameter*, parameterLessThan>::const_iterator it = ps.begin();
+          it != ps.end(); it++){
+        if(client.empty() || (*it)->hasClient(client)){
+	  if((*it)->getAttribute("NotInDb") != "True")
+	    json += "    " + (*it)->toJSON() + "\n";
+	}
+      }
+      json += "] }\n";
+    }
   };
 
   // The onelab client: a class that communicates with the onelab server. Each
@@ -1243,23 +822,15 @@ namespace onelab{
     virtual bool clear(const std::string &name) = 0;
     virtual bool set(const number &p) = 0;
     virtual bool set(const string &p) = 0;
-    virtual bool set(const region &p) = 0;
-    virtual bool set(const function &p) = 0;
     virtual bool get(std::vector<number> &ps, const std::string &name="") = 0;
     virtual bool get(std::vector<onelab::string> &ps, const std::string &name="") = 0;
-    virtual bool get(std::vector<region> &ps, const std::string &name="") = 0;
-    virtual bool get(std::vector<function> &ps, const std::string &name="") = 0;
     std::vector<std::string> toChar()
     {
       std::vector<std::string> out;
       std::vector<number> n; get(n);
       for(unsigned int i = 0; i < n.size(); i++) out.push_back(n[i].toChar());
-      std::vector<number> s; get(s);
+      std::vector<string> s; get(s);
       for(unsigned int i = 0; i < s.size(); i++) out.push_back(s[i].toChar());
-      std::vector<region> r; get(r);
-      for(unsigned int i = 0; i < r.size(); i++) out.push_back(r[i].toChar());
-      std::vector<region> f; get(f);
-      for(unsigned int i = 0; i < f.size(); i++) out.push_back(f[i].toChar());
       return out;
     }
     bool fromChar(const std::vector<std::string> &msg)
@@ -1274,12 +845,6 @@ namespace onelab{
         else if(type == "string"){
           onelab::string p; p.fromChar(msg[i]); set(p);
         }
-        else if(type == "region"){
-          onelab::region p; p.fromChar(msg[i]); set(p);
-        }
-        else if(type == "function"){
-          onelab::function p; p.fromChar(msg[i]); set(p);
-        }
         else
           return false;
       }
@@ -1373,6 +938,10 @@ namespace onelab{
       if(parameter::fromFile(msg, fp)) return fromChar(msg, client);
       return false;
     }
+    void toJSON(std::string &json, const std::string &client="")
+    {
+      _parameterSpace.toJSON(json, client);
+    }
   };
 
   // A local client, which lives in the same memory space as the server.
@@ -1405,16 +974,10 @@ namespace onelab{
     }
     virtual bool set(const number &p){ return _set(p); }
     virtual bool set(const string &p){ return _set(p); }
-    virtual bool set(const function &p){ return _set(p); }
-    virtual bool set(const region &p){ return _set(p); }
     virtual bool get(std::vector<number> &ps,
                      const std::string &name=""){ return _get(ps, name); }
     virtual bool get(std::vector<onelab::string> &ps,
                      const std::string &name=""){ return _get(ps, name); }
-    virtual bool get(std::vector<function> &ps,
-                     const std::string &name=""){ return _get(ps, name); }
-    virtual bool get(std::vector<region> &ps,
-                     const std::string &name=""){ return _get(ps, name); }
   };
 
   // The local part of a network client.
@@ -1478,8 +1041,7 @@ namespace onelab{
       _gmshClient->SendMessage(GmshSocket::GMSH_PARAMETER, msg.size(), &msg[0]);
       return true;
     }
-    template <class T> bool _get(std::vector<T> &ps,
-                                 const std::string &name="")
+    template <class T> bool _get(std::vector<T> &ps, const std::string &name="")
     {
       ps.clear();
       if(!_gmshClient) return false;
@@ -1604,16 +1166,10 @@ namespace onelab{
     }
     virtual bool set(const number &p){ return _set(p); }
     virtual bool set(const string &p){ return _set(p); }
-    virtual bool set(const function &p){ return _set(p); }
-    virtual bool set(const region &p){ return _set(p); }
     virtual bool get(std::vector<number> &ps,
                      const std::string &name=""){ return _get(ps, name); }
     virtual bool get(std::vector<onelab::string> &ps,
                      const std::string &name=""){ return _get(ps, name); }
-    virtual bool get(std::vector<function> &ps,
-                     const std::string &name=""){ return _get(ps, name); }
-    virtual bool get(std::vector<region> &ps,
-                     const std::string &name=""){ return _get(ps, name); }
     void sendInfo(const std::string &msg)
     {
       if(_gmshClient) _gmshClient->Info(msg.c_str());
