@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include "GmshConfig.h"
 
@@ -22,98 +22,21 @@
 #include "PViewOptions.h"
 #endif
 
-#if defined(HAVE_ONELAB2)
-#include "OnelabDatabase.h"
-#include "OnelabServer.h"
-#endif
-
 namespace onelabUtils {
 
   // get command line arguments for the client if "UseCommandLine" is set for
   // this client
-#ifdef HAVE_ONELAB2
-  std::vector<std::string> getCommandLine(const std::string &client)
-  {
-    std::vector<std::string> args;
-    std::vector<onelab::number> n;
-    OnelabDatabase::instance()->get(n, client + "/UseCommandLine");
-    if(n.size() && n[0].getValue()){
-      std::vector<onelab::string> ps;
-      OnelabDatabase::instance()->get(ps, client + "/Action");
-      std::string action = (ps.empty() ? "" : ps[0].getValue());
-      OnelabDatabase::instance()->get(ps, client + "/1ModelName");
-      std::string modelName = (ps.empty() ? "" : ps[0].getValue());
-      OnelabDatabase::instance()->get(ps, client + "/9CheckCommand");
-      std::string checkCommand = (ps.empty() ? "" : ps[0].getValue());
-      OnelabDatabase::instance()->get(ps, client + "/9ComputeCommand");
-      std::string computeCommand = (ps.empty() ? "" : ps[0].getValue());
-      if(modelName.size()) args.push_back(" \"" + modelName + "\"");
-      if(action == "check")
-        args.push_back(" " + checkCommand) ;
-      else if(action == "compute")
-        args.push_back(" " + computeCommand);
-    }
-    return args;
-  }
-  std::string getMshFileName()
-  {
-    std::string name;
-    std::vector<onelab::string> ps;
-    OnelabDatabase::instance()->get(ps, "Gmsh/MshFileName");
-    if(ps.size()){
-      name = ps[0].getValue();
-    }
-    else{
-      name = CTX::instance()->outputFileName;
-      if(name.empty()){
-        if(CTX::instance()->mesh.fileFormat == FORMAT_AUTO)
-          name = GetDefaultFileName(FORMAT_MSH);
-        else
-          name = GetDefaultFileName(CTX::instance()->mesh.fileFormat);
-      }
-      onelab::string o("Gmsh/MshFileName", name, "Mesh name");
-      o.setKind("file");
-      o.setAttribute("Closed", "1");
-      OnelabDatabase::instance()->set(o, std::string("Gmsh"));
-    }
-    return name;
-  }
-
-  void guessModelName(const std::string &client)
-  {
-    std::vector<onelab::number> n;
-    OnelabDatabase::instance()->get(n, client + "/GuessModelName");
-    if(n.size() && n[0].getValue()){
-      std::vector<onelab::string> ps;
-      OnelabDatabase::instance()->get(ps, client + "/1ModelName");
-      if(ps.empty()){
-        OnelabDatabase::instance()->get(ps, "Gmsh/1ModelName");
-        std::vector<std::string> split;
-        if(!ps.empty()) split = SplitFileName(ps[0].getValue());
-        else split = SplitFileName(GModel::current()->getFileName());
-        std::string ext = "";
-        OnelabDatabase::instance()->get(ps, client + "/FileExtension");
-        if(ps.size()) ext = ps[0].getValue();
-        std::string name(split[0] + split[1] + ext);
-        onelab::string o(client + "/1ModelName", name, "Model name");
-        o.setKind("file");
-        OnelabDatabase::instance()->set(o, client);
-      }
-    }
-  }
-
-#else
   std::vector<std::string> getCommandLine(onelab::client *c)
   {
     std::vector<std::string> args;
     std::string name(c->getName());
     std::vector<onelab::number> n;
-    c->get(n, name + "/UseCommandLine");
+    c->get(n, name + "/Use command line");
     if(n.size() && n[0].getValue()){
       std::vector<onelab::string> ps;
       c->get(ps, name + "/Action");
       std::string action = (ps.empty() ? "" : ps[0].getValue());
-      c->get(ps, name + "/1ModelName");
+      c->get(ps, name + "/Model name");
       std::string modelName = (ps.empty() ? "" : ps[0].getValue());
       c->get(ps, name + "/9CheckCommand");
       std::string checkCommand = (ps.empty() ? "" : ps[0].getValue());
@@ -129,7 +52,8 @@ namespace onelabUtils {
       std::ostringstream sstream;
       sstream.precision(16);
       std::map<std::string, std::vector<double> > cln(Msg::GetCommandLineNumbers());
-      for(std::map<std::string, std::vector<double> >::iterator it = cln.begin(); it != cln.end(); it++){
+      for(std::map<std::string, std::vector<double> >::iterator it = cln.begin();
+          it != cln.end(); it++){
         if(it->second.size() == 1){
           sstream << " -setnumber " << it->first << " " << it->second[0];
         }
@@ -140,7 +64,8 @@ namespace onelabUtils {
         }
       }
       std::map<std::string, std::string> cls(Msg::GetCommandLineStrings());
-      for(std::map<std::string, std::string>::iterator it = cls.begin(); it != cls.end(); it++)
+      for(std::map<std::string, std::string>::iterator it = cls.begin();
+          it != cls.end(); it++)
         sstream << " -setstring " << it->first << " " << it->second;
       args.push_back(sstream.str());
     }
@@ -182,26 +107,31 @@ namespace onelabUtils {
 
   void guessModelName(onelab::client *c)
   {
+    std::string geo = GModel::current()->getFileName();
     std::vector<onelab::number> n;
-    c->get(n, c->getName() + "/GuessModelName");
+    c->get(n, c->getName() + "/Guess model name");
     if(n.size() && n[0].getValue()){
       std::vector<onelab::string> ps;
-      c->get(ps, c->getName() + "/1ModelName");
+      c->get(ps, c->getName() + "/Model name");
       if(ps.empty()){
-        std::vector<std::string> split = SplitFileName(GModel::current()->getFileName());
+        std::vector<std::string> split = SplitFileName(geo);
         std::string ext = "";
-        onelab::server::instance()->get(ps, c->getName() + "/FileExtension");
+        onelab::server::instance()->get(ps, c->getName() + "/File extension");
         if(ps.size()) ext = ps[0].getValue();
         std::string name(split[0] + split[1] + ext);
-        onelab::string o(c->getName() + "/1ModelName", name, "Model name");
+        onelab::string o(c->getName() + "/Model name", name);
         o.setKind("file");
+        o.setAttribute("Persistent", "1");
         c->set(o);
+        geo += std::string(" - ") + name;
+      }
+      else{
+        geo += std::string(" - ") + ps[0].getValue();
       }
     }
+    Msg::SetWindowTitle(geo);
   }
 
-#endif
-
   void initializeLoop(const std::string &level)
   {
     bool changed = false;
@@ -392,78 +322,6 @@ namespace onelabUtils {
   static bool _firstComputation = true;
   void setFirstComputationFlag(bool val){ _firstComputation = val; }
   bool getFirstComputationFlag(){ return _firstComputation; }
-#ifdef HAVE_ONELAB2
-  bool runGmshClient(const std::string &action, int meshAuto)
-  {
-    bool redraw = false;
-
-    // do nothing in case of a python metamodel
-    std::vector<onelab::number> pn;
-    OnelabDatabase::instance()->get(pn, "IsPyMetamodel");
-    if(pn.size() && pn[0].getValue()) return redraw;
-
-    OnelabLocalNetworkClient *c = OnelabServer::instance()->getClient("Gmsh");
-    if(c != NULL) return redraw; // Gmsh is remote TODO
-
-    std::string mshFileName = onelabUtils::getMshFileName();
-
-    Msg::SetOnelabAction(action);
-
-    static std::string modelName = GModel::current()->getName();
-
-    if(action == "initialize"){
-      // nothing to do
-    }
-    else if(action == "reset"){
-      setFirstComputationFlag(false);
-      // nothing more to do: "check" will be called right afterwards
-    }
-    else if(action == "check"){
-      if(OnelabServer::instance()->getChanged("Gmsh") ||
-         modelName != GModel::current()->getName()){
-        // reload geometry if Gmsh parameters have been modified or
-        // if the model name has changed
-        modelName = GModel::current()->getName();
-        redraw = true;
-        OpenProject(GModel::current()->getFileName(), false);
-        OnelabServer::instance()->setChanged(true, "Gmsh");
-      }
-    }
-    else if(action == "compute"){
-      if(OnelabServer::instance()->getChanged("Gmsh") ||
-         modelName != GModel::current()->getName()){
-        // reload the geometry, mesh it and save the mesh if Gmsh parameters
-        // have been modified or if the model name has changed
-        modelName = GModel::current()->getName();
-        redraw = true;
-        OpenProject(GModel::current()->getFileName(), false);
-        if(getFirstComputationFlag() && !StatFile(mshFileName) && meshAuto != 2){
-          Msg::Info("Skipping mesh generation: assuming '%s' is up-to-date "
-                    "(use Solver.AutoMesh=2 to force mesh generation)",
-                    mshFileName.c_str());
-        }
-        else if(!GModel::current()->empty() && meshAuto){
-          GModel::current()->mesh(3);
-          CreateOutputFile(mshFileName, CTX::instance()->mesh.fileFormat);
-        }
-      }
-      else if(StatFile(mshFileName)){
-        // mesh+save if the mesh file does not exist
-        if(meshAuto){
-          redraw = true;
-          GModel::current()->mesh(3);
-          CreateOutputFile(mshFileName, CTX::instance()->mesh.fileFormat);
-        }
-      }
-      setFirstComputationFlag(false);
-      OnelabServer::instance()->setChanged(false, "Gmsh");
-    }
-
-    Msg::SetOnelabAction("");
-
-    return redraw;
-  }
-#else
   bool runGmshClient(const std::string &action, int meshAuto)
   {
     bool redraw = false;
@@ -497,7 +355,7 @@ namespace onelabUtils {
         // if the model name has changed
         modelName = GModel::current()->getName();
         redraw = true;
-        OpenProject(GModel::current()->getFileName(), false);
+        OpenProject(GModel::current()->getFileName());
       }
     }
     else if(action == "compute"){
@@ -507,7 +365,7 @@ namespace onelabUtils {
         // have been modified or if the model name has changed
         modelName = GModel::current()->getName();
         redraw = true;
-        OpenProject(GModel::current()->getFileName(), false);
+        OpenProject(GModel::current()->getFileName());
         if(getFirstComputationFlag() && !StatFile(mshFileName) && meshAuto != 2){
           Msg::Info("Skipping mesh generation: assuming '%s' is up-to-date "
                     "(use Solver.AutoMesh=2 to force mesh generation)",
@@ -534,7 +392,7 @@ namespace onelabUtils {
 
     return redraw;
   }
-#endif
+
   // update x using y, giving priority to any settings in x that can be set in
   // the GUI. The value of x is only changed if y is read-only.
   double updateNumber(onelab::number &x, onelab::number &y, const bool readOnlyRange)
@@ -602,6 +460,20 @@ namespace onelabUtils {
     return val;
   }
 
+  bool haveSolverToRun()
+  {
+    for(onelab::server::citer it = onelab::server::instance()->firstClient();
+	it != onelab::server::instance()->lastClient(); it++){
+      onelab::client *c = *it;
+      if(c->getName() != "Gmsh" &&
+	 c->getName() != "Listen" &&
+	 c->getName() != "GmshRemote" &&
+         c->getName().find("NoAutoRun") == std::string::npos)
+	return true;
+    }
+    return false;
+  }
+
   // update x using y, giving priority to any settings in x that can be set in
   // the GUI. The value of x is only changed if y is read-only.
   std::string updateString(onelab::string &x, onelab::string &y)
