@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include "GmshConfig.h"
 #if !defined(HAVE_NO_STDINT_H)
@@ -29,12 +29,7 @@ typedef unsigned long intptr_t;
 #include "clippingWindow.h"
 #include "manipWindow.h"
 #include "contextWindow.h"
-#ifdef HAVE_ONELAB2
-#include "onelab2Group.h"
-#include "OnelabWindow.h"
-#else
 #include "onelabGroup.h"
-#endif
 #include "helpWindow.h"
 #include "colorbarWindow.h"
 #include "fileDialogs.h"
@@ -76,18 +71,6 @@ static int globalShortcut(int event)
   return FlGui::instance()->testGlobalShortcuts(event);
 }
 
-int FlGui::lock()
-{
-  _in_main_thread++;
-  return Fl::lock();
-}
-
-void FlGui::unlock()
-{
-  Fl::unlock();
-  _in_main_thread--;
-}
-
 static void simple_right_box_draw(int x, int y, int w, int h, Fl_Color c)
 {
   fl_color(c); fl_rectf(x, y, w, h);
@@ -199,6 +182,22 @@ static void gmsh_search(Fl_Color col)
   fl_line_style(FL_SOLID);
 }
 
+static void gmsh_colormap(Fl_Color col)
+{
+  fl_color(FL_RED);
+  fl_begin_polygon();
+  vv(-0.8,-0.8); vv(-0.3,-0.8); vv(-0.3,0.8); vv(-0.8,0.8);
+  fl_end_polygon();
+  fl_color(FL_GREEN);
+  fl_begin_polygon();
+  vv(-0.3,-0.8); vv(0.2,-0.8); vv(0.2,0.8); vv(-0.3,0.8);
+  fl_end_polygon();
+  fl_color(FL_BLUE);
+  fl_begin_polygon();
+  vv(0.2,-0.8); vv(0.7,-0.8); vv(0.7,0.8); vv(0.2,0.8);
+  fl_end_polygon();
+}
+
 #undef vv
 #undef bl
 #undef el
@@ -347,6 +346,7 @@ FlGui::FlGui(int argc, char **argv)
   fl_add_symbol("gmsh_gear", gmsh_gear, 1);
   fl_add_symbol("gmsh_graph", gmsh_graph, 1);
   fl_add_symbol("gmsh_search", gmsh_search, 1);
+  fl_add_symbol("gmsh_colormap", gmsh_colormap, 1);
 
   // load default system icons (for file browser)
   Fl_File_Icon::load_system_icons();
@@ -411,6 +411,7 @@ FlGui::FlGui(int argc, char **argv)
     g->getWindow()->show();
     graph.push_back(g);
   }
+  setGraphicTitle(GModel::current()->getFileName());
 
   // create fullscreen window
   fullscreen = new openglWindow(100, 100, 100, 100);
@@ -429,9 +430,6 @@ FlGui::FlGui(int argc, char **argv)
 
   // create all other windows
   options = new optionWindow(CTX::instance()->deltaFontSize);
-#if defined(HAVE_ONELAB2)
-  onelab2 = new onelabWindow();
-#endif
   fields = new fieldWindow(CTX::instance()->deltaFontSize);
   plugins = new pluginWindow(CTX::instance()->deltaFontSize);
   stats = new statisticsWindow(CTX::instance()->deltaFontSize);
@@ -439,15 +437,14 @@ FlGui::FlGui(int argc, char **argv)
   highordertools = new highOrderToolsWindow(CTX::instance()->deltaFontSize);
   clipping = new clippingWindow(CTX::instance()->deltaFontSize);
   manip = new manipWindow(CTX::instance()->deltaFontSize);
-  geoContext = new geometryContextWindow(CTX::instance()->deltaFontSize);
+  elementaryContext = new elementaryContextWindow(CTX::instance()->deltaFontSize);
+  physicalContext = new physicalContextWindow(CTX::instance()->deltaFontSize);
   meshContext = new meshContextWindow(CTX::instance()->deltaFontSize);
   help = new helpWindow();
 
   // init solver plugin stuff
   callForSolverPlugin(-1);
 
-  _in_main_thread = 0;
-
   // draw
   for(unsigned int i = 0; i < graph.size(); i++)
     for(unsigned int j = 0; j < graph[i]->gl.size(); j++)
@@ -463,10 +460,6 @@ FlGui *FlGui::instance(int argc, char **argv)
     _instance = new FlGui(argc, argv);
     // set all options in the new GUI
     InitOptionsGUI(0);
-#ifdef HAVE_ONELAB2
-    // Enable multi-thread support by locking from the main thread
-    Fl::lock();
-#endif
 
     // say welcome!
     Msg::StatusBar(false, "Gmsh %s", GetGmshVersion());
@@ -671,6 +664,13 @@ int FlGui::testGlobalShortcuts(int event)
       (0, GMSH_SET | GMSH_GUI, opt_general_color_scheme(0, GMSH_GET, 0) + 1);
     status = 2;
   }
+  else if(Fl::test_shortcut(FL_ALT + FL_SHIFT + 'c')) {
+    for(unsigned int i = 0; i < PView::list.size(); i++)
+      if(opt_view_visible(i, GMSH_GET, 0))
+        opt_view_colormap_number
+          (i, GMSH_SET | GMSH_GUI, opt_view_colormap_number(i, GMSH_GET, 0) + 1);
+    status = 2;
+  }
   else if(Fl::test_shortcut(FL_ALT + 'w')) {
     opt_geometry_light
       (0, GMSH_SET | GMSH_GUI, !opt_geometry_light(0, GMSH_GET, 0));
@@ -1095,8 +1095,8 @@ void FlGui::storeCurrentWindowsInfo()
   CTX::instance()->clipPosition[1] = clipping->win->y();
   CTX::instance()->manipPosition[0] = manip->win->x();
   CTX::instance()->manipPosition[1] = manip->win->y();
-  CTX::instance()->ctxPosition[0] = geoContext->win->x();
-  CTX::instance()->ctxPosition[1] = meshContext->win->y();
+  CTX::instance()->ctxPosition[0] = elementaryContext->win->x();
+  CTX::instance()->ctxPosition[1] = elementaryContext->win->y();
 #if defined(HAVE_3M)
   storeWindowPosition3M();
 #endif
@@ -1200,8 +1200,10 @@ void window_cb(Fl_Widget *w, void *data)
       FlGui::instance()->plugins->win->show();
     if(FlGui::instance()->fields->win->shown())
       FlGui::instance()->fields->win->show();
-    if(FlGui::instance()->geoContext->win->shown())
-      FlGui::instance()->geoContext->win->show();
+    if(FlGui::instance()->elementaryContext->win->shown())
+      FlGui::instance()->elementaryContext->win->show();
+    if(FlGui::instance()->physicalContext->win->shown())
+      FlGui::instance()->physicalContext->win->show();
     if(FlGui::instance()->meshContext->win->shown())
       FlGui::instance()->meshContext->win->show();
     if(FlGui::instance()->visibility->win->shown())
