@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <FL/Fl.H>
 #include "GmshConfig.h"
@@ -59,7 +59,7 @@ void onelab_cb(Fl_Widget *w, void *data)
 
   if(action == "refresh"){
     updateGraphs();
-    FlGui::instance()->rebuildTree(false); // FIXME: true would be better
+    FlGui::instance()->rebuildTree(true);
     return;
   }
 
@@ -85,41 +85,34 @@ void onelab_cb(Fl_Widget *w, void *data)
   }
 
   if(action == "save"){
-    std::vector<std::string> db = onelab::server::instance()->toChar();
-    Msg::Direct("ONELAB database:");
-    for(unsigned int i = 0; i < db.size(); i++){
-      for(unsigned int j = 0; j < db[i].size(); j++)
-        if(db[i][j] == onelab::parameter::charSep()) db[i][j] = '|';
-      Msg::Direct("%s", db[i].c_str());
-    }
-
     std::string fileName = "onelab.db";
-    // add user defined tag, if any
+
+    // special handling for metamodels: add user defined tag, if any
     std::vector<onelab::string> ps;
     onelab::server::instance()->get(ps,"0Metamodel/9Tag");
     if(ps.size() && ps[0].getValue().size()){
       fileName.assign("onelab_" + ps[0].getValue() + ".db");
     }
 
-    // save db in "restore" mode"
-    double restoreMode=0;
+    // special handling for metamodels: save db in "restore" mode"
+    double restoreMode = 0.;
     std::vector<onelab::number> pn;
     onelab::server::instance()->get(pn,"0Metamodel/9Use restored solution");
     if(pn.size()){
-      restoreMode=pn[0].getValue();
+      restoreMode = pn[0].getValue();
       pn[0].setValue(2); // special value
       onelab::server::instance()->set(pn[0]);
     }
 
-    std::string s;
-    s.assign(SplitFileName(GModel::current()->getFileName())[0] + fileName);
-    if(fileChooser(FILE_CHOOSER_CREATE, "Save", "*.db", s.c_str())){
+    std::string db;
+    db.assign(SplitFileName(GModel::current()->getFileName())[0] + fileName);
+    if(fileChooser(FILE_CHOOSER_CREATE, "Save", "*.db", db.c_str())){
       if(!restoreMode)
 	archiveSolutionFiles(fileChooserGetName(1));
       saveDb(fileChooserGetName(1));
     }
 
-    // switch back to normal "run" mode"
+    // special handling for metamodels: switch back to normal "run" mode"
     onelab::server::instance()->get(pn,"0Metamodel/9Use restored solution");
     if(pn.size()){
       pn[0].setValue(0);
@@ -201,7 +194,8 @@ void onelab_cb(Fl_Widget *w, void *data)
 
   if(action == "compute" && (CTX::instance()->solver.autoSaveDatabase ||
                              CTX::instance()->solver.autoArchiveOutputFiles)){
-    std::string db = SplitFileName(GModel::current()->getFileName())[0] + "onelab.db";
+    std::vector<std::string> split = SplitFileName(GModel::current()->getFileName());
+    std::string db = split[0] + split[1] + ".db";
     if(CTX::instance()->solver.autoArchiveOutputFiles) archiveOutputFiles(db);
     if(CTX::instance()->solver.autoSaveDatabase) saveDb(db);
   }
@@ -221,6 +215,8 @@ void onelab_option_cb(Fl_Widget *w, void *data)
   double val = ((Fl_Menu_*)w)->mvalue()->value() ? 1. : 0.;
   if(what == "save")
     CTX::instance()->solver.autoSaveDatabase = val;
+  else if(what == "load")
+    CTX::instance()->solver.autoLoadDatabase = val;
   else if(what == "archive")
     CTX::instance()->solver.autoArchiveOutputFiles = val;
   else if(what == "check"){
@@ -344,16 +340,12 @@ static void setOpenedClosed(Fl_Tree_Item *item, int reason)
 {
   std::vector<onelab::number> numbers;
   std::vector<onelab::string> strings;
-  std::vector<onelab::region> regions;
-  std::vector<onelab::function> functions;
   std::string path = FlGui::instance()->onelab->getPath(item);
   switch(reason){
   case FL_TREE_REASON_OPENED:
     FlGui::instance()->onelab->removeFromManuallyClosed(path);
     setClosed(path, numbers, "0");
     setClosed(path, strings, "0");
-    setClosed(path, regions, "0");
-    setClosed(path, functions, "0");
     for(unsigned int i = 0; i < PView::list.size(); i++){
       if(getViewPath(i) == path) PView::list[i]->getOptions()->closed = 0;
     }
@@ -362,8 +354,6 @@ static void setOpenedClosed(Fl_Tree_Item *item, int reason)
     FlGui::instance()->onelab->insertInManuallyClosed(path);
     setClosed(path, numbers, "1");
     setClosed(path, strings, "1");
-    setClosed(path, regions, "1");
-    setClosed(path, functions, "1");
     for(unsigned int i = 0; i < PView::list.size(); i++){
       if(getViewPath(i) == path) PView::list[i]->getOptions()->closed = 1;
     }
@@ -470,7 +460,9 @@ onelabGroup::onelabGroup(int x, int y, int w, int h, const char *l)
 
   _gearOptionsStart = _gear->menu()->size();
 
-  _gear->add("Save && load database automatically", 0, onelab_option_cb, (void*)"save",
+  _gear->add("Save database automatically", 0, onelab_option_cb, (void*)"save",
+             FL_MENU_TOGGLE);
+  _gear->add("Load database automatically", 0, onelab_option_cb, (void*)"load",
              FL_MENU_TOGGLE);
   _gear->add("Archive output files automatically", 0, onelab_option_cb, (void*)"archive",
              FL_MENU_TOGGLE);
@@ -548,8 +540,9 @@ void onelabGroup::_addParameter(T &p)
   if(!_enableTreeWidgetResize) grp->resizable(0);
   _treeWidgets.push_back(grp);
   widget->copy_label(p.getShortName().c_str());
-  std::string help = p.getLabel().size() ? p.getLabel() : p.getShortName();
-  if(p.getHelp().size()) help += ":\n" + p.getHelp();
+  std::string help = p.getHelp();
+  if(help.empty()) help = p.getLabel();
+  if(help.empty()) help = p.getShortName();
   widget->copy_tooltip(help.c_str());
   n->widget(grp);
   _tree->end();
@@ -768,7 +761,7 @@ Fl_Widget *onelabGroup::_addParameterWidget(onelab::number &p, int ww, int hh,
       }
     }
     but->callback(onelab_number_choice_cb, (void*)path);
-    but->align(FL_ALIGN_RIGHT);
+    but->align(FL_ALIGN_RIGHT | FL_ALIGN_CLIP);
     if(p.getReadOnly()) but->deactivate();
     return but;
   }
@@ -782,6 +775,7 @@ Fl_Widget *onelabGroup::_addParameterWidget(onelab::number &p, int ww, int hh,
     but->color(_tree->color());
     but->value(p.getValue());
     but->callback(onelab_number_check_button_cb, (void*)path);
+    but->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE | FL_ALIGN_CLIP);
     if(highlight){
       but->color(c);
       but->labelcolor(fl_contrast(FL_FOREGROUND_COLOR, c));
@@ -795,7 +789,7 @@ Fl_Widget *onelabGroup::_addParameterWidget(onelab::number &p, int ww, int hh,
     outputRange *but = new outputRange(1, 1, ww, hh);
     but->callback(onelab_number_output_range_cb, (void*)path);
     but->value(p.getValue());
-    but->align(FL_ALIGN_RIGHT);
+    but->align(FL_ALIGN_RIGHT | FL_ALIGN_CLIP);
     but->graph(p.getAttribute("Graph"));
     if(highlight) but->color(c);
     return but;
@@ -813,7 +807,7 @@ Fl_Widget *onelabGroup::_addParameterWidget(onelab::number &p, int ww, int hh,
   but->graph(p.getAttribute("Graph"));
   but->callback(onelab_number_input_range_cb, (void*)path);
   but->when(FL_WHEN_RELEASE | FL_WHEN_ENTER_KEY);
-  but->align(FL_ALIGN_RIGHT);
+  but->align(FL_ALIGN_RIGHT | FL_ALIGN_CLIP);
   if(highlight) but->color(c);
   return but;
 }
@@ -952,7 +946,7 @@ Fl_Widget *onelabGroup::_addParameterWidget(onelab::string &p, int ww, int hh,
   if(p.getReadOnly()){
     Fl_Output *but = new Fl_Output(1, 1, ww, hh);
     but->value(p.getValue().c_str());
-    but->align(FL_ALIGN_RIGHT);
+    but->align(FL_ALIGN_RIGHT | FL_ALIGN_CLIP);
     if(highlight){
       but->color(c);
       but->textcolor(fl_contrast(FL_FOREGROUND_COLOR, c));
@@ -965,8 +959,8 @@ Fl_Widget *onelabGroup::_addParameterWidget(onelab::string &p, int ww, int hh,
     Fl_Input *but = new Fl_Input(1, 1, ww, hh);
     but->value(p.getValue().c_str());
     but->callback(onelab_string_input_cb, (void*)path);
-    but->when(FL_WHEN_ENTER_KEY);
-    but->align(FL_ALIGN_RIGHT);
+    but->when(FL_WHEN_RELEASE | FL_WHEN_ENTER_KEY);
+    but->align(FL_ALIGN_RIGHT | FL_ALIGN_CLIP);
     if(highlight){
       but->color(c);
       but->textcolor(fl_contrast(FL_FOREGROUND_COLOR, c));
@@ -1006,8 +1000,8 @@ Fl_Widget *onelabGroup::_addParameterWidget(onelab::string &p, int ww, int hh,
   but->menubutton()->copy(&menu[0]);
   but->value(p.getValue().c_str());
   but->callback(onelab_string_input_choice_cb, (void*)path);
-  but->input()->when(FL_WHEN_ENTER_KEY);
-  but->align(FL_ALIGN_RIGHT);
+  but->input()->when(FL_WHEN_RELEASE | FL_WHEN_ENTER_KEY);
+  but->align(FL_ALIGN_RIGHT | FL_ALIGN_CLIP);
   if(highlight){
     but->input()->color(c);
     but->input()->textcolor(fl_contrast(FL_FOREGROUND_COLOR, c));
@@ -1015,21 +1009,6 @@ Fl_Widget *onelabGroup::_addParameterWidget(onelab::string &p, int ww, int hh,
   return but;
 }
 
-static void onelab_region_input_cb(Fl_Widget *w, void *data)
-{
-  if(!data) return;
-  std::string name((char*)data);
-  std::vector<onelab::region> regions;
-  onelab::server::instance()->get(regions, name);
-  if(regions.size()){
-    inputRegion *o = (inputRegion*)w;
-    onelab::region old = regions[0];
-    regions[0].setValue(o->value());
-    onelab::server::instance()->set(regions[0]);
-    autoCheck(old, regions[0]);
-  }
-}
-
 static void view_group_cb(Fl_Widget *w, void *data)
 {
   if(!data) return;
@@ -1046,51 +1025,6 @@ static void view_group_cb(Fl_Widget *w, void *data)
   drawContext::global()->draw();
 }
 
-Fl_Widget *onelabGroup::_addParameterWidget(onelab::region &p, int ww, int hh,
-                                            Fl_Tree_Item *n, bool highlight, Fl_Color c)
-{
-  char *path = strdup(getPath(n).c_str());
-  _treeStrings.push_back(path);
-
-  // non-editable value
-  if(p.getReadOnly()){
-    inputRegion *but = new inputRegion(1, 1, ww, hh, true);
-    but->value(p.getValue());
-    but->align(FL_ALIGN_RIGHT);
-    if(highlight){
-      but->color(c);
-      but->labelcolor(fl_contrast(FL_FOREGROUND_COLOR, c));
-    }
-    return but;
-  }
-
-  inputRegion *but = new inputRegion(1, 1, ww, hh, false);
-  but->value(p.getValue());
-  but->align(FL_ALIGN_RIGHT);
-  but->callback(onelab_region_input_cb, (void*)path);
-  if(highlight){
-    but->color(c);
-    but->labelcolor(fl_contrast(FL_FOREGROUND_COLOR, c));
-  }
-  return but;
-}
-
-Fl_Widget *onelabGroup::_addParameterWidget(onelab::function &p, int ww, int hh,
-                                            Fl_Tree_Item *n, bool highlight, Fl_Color c)
-{
-  // non-editable value
-  if(1 || p.getReadOnly()){
-    Fl_Output *but = new Fl_Output(1, 1, ww, hh);
-    but->value("TODO function");
-    but->align(FL_ALIGN_RIGHT);
-    if(highlight){
-      but->color(c);
-      but->textcolor(fl_contrast(FL_FOREGROUND_COLOR, c));
-    }
-    return but;
-  }
-}
-
 void onelabGroup::rebuildTree(bool deleteWidgets)
 {
   FL_NORMAL_SIZE -= CTX::instance()->deltaFontSize;
@@ -1151,26 +1085,6 @@ void onelabGroup::rebuildTree(bool deleteWidgets)
     _addParameter(strings[i]);
   }
 
-  std::vector<onelab::region> regions;
-  onelab::server::instance()->get(regions);
-  for(unsigned int i = 0; i < regions.size(); i++){
-    if(!regions[i].getVisible() && !CTX::instance()->solver.showInvisibleParameters)
-      continue;
-    if(regions[i].getAttribute("Closed") == "1")
-      closed.insert(regions[i].getPath());
-    _addParameter(regions[i]);
-  }
-
-  std::vector<onelab::function> functions;
-  onelab::server::instance()->get(functions);
-  for(unsigned int i = 0; i < functions.size(); i++){
-    if(!functions[i].getVisible() && !CTX::instance()->solver.showInvisibleParameters)
-      continue;
-    if(functions[i].getAttribute("Closed") == "1")
-      closed.insert(functions[i].getPath());
-    _addParameter(functions[i]);
-  }
-
   for(Fl_Tree_Item *n = _tree->first(); n; n = n->next()){
     if(n->has_children()){
       int ww = (int)(_baseWidth - (n->depth() + 1) * _indent);
@@ -1183,13 +1097,13 @@ void onelabGroup::rebuildTree(bool deleteWidgets)
       but = new Fl_Button(1, 1, ww, hh);
       but->box(FL_NO_BOX);
       but->clear_visible_focus();
-      but->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE);
+      but->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE | FL_ALIGN_CLIP);
       but->callback(onelab_subtree_cb, (void*)n);
 #else
       if(getPath(n).find("0Modules/Post-processing") != std::string::npos){
         but = new Fl_Button(1, 1, ww, hh);
         but->box(FL_NO_BOX);
-        but->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE);
+        but->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE | FL_ALIGN_CLIP);
         but->tooltip("Toggle visibility of child views");
         char *group = strdup(getPath(n).substr(24).c_str());
         _treeStrings.push_back(group);
@@ -1197,7 +1111,7 @@ void onelabGroup::rebuildTree(bool deleteWidgets)
       }
       else{
         but = new Fl_Box(1, 1, ww, hh);
-        but->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE);
+        but->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE | FL_ALIGN_CLIP);
       }
 #endif
       _treeWidgets.push_back(but);
@@ -1338,7 +1252,8 @@ std::string onelabGroup::getPath(Fl_Tree_Item *item)
 void onelabGroup::updateGearMenu()
 {
   Fl_Menu_Item* menu = (Fl_Menu_Item*)_gear->menu();
-  int values[8] = {CTX::instance()->solver.autoSaveDatabase,
+  int values[9] = {CTX::instance()->solver.autoSaveDatabase,
+                   CTX::instance()->solver.autoLoadDatabase,
                    CTX::instance()->solver.autoArchiveOutputFiles,
                    CTX::instance()->solver.autoCheck,
                    CTX::instance()->solver.autoMesh,
@@ -1346,7 +1261,7 @@ void onelabGroup::updateGearMenu()
                    CTX::instance()->solver.autoShowViews,
                    CTX::instance()->solver.autoShowLastStep,
                    CTX::instance()->solver.showInvisibleParameters};
-  for(int i = 0; i < 8; i++){
+  for(int i = 0; i < 9; i++){
     int idx = _gearOptionsStart - 1 + i;
     if(values[i])
       menu[idx].set();
@@ -1450,8 +1365,9 @@ void solver_cb(Fl_Widget *w, void *data)
     FlGui::instance()->onelab->rebuildSolverList();
   }
 
-  if(CTX::instance()->solver.autoSaveDatabase){
-    std::string db = SplitFileName(GModel::current()->getFileName())[0] + "onelab.db";
+  if(CTX::instance()->solver.autoLoadDatabase){
+    std::vector<std::string> split = SplitFileName(GModel::current()->getFileName());
+    std::string db = split[0] + split[1] + ".db";
     if(!StatFile(db)){
       loadDb(db);
       CTX::instance()->launchSolverAtStartup = -1;
