@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <FL/fl_ask.H>
 #include "FlGui.h"
@@ -18,11 +18,7 @@
 #include "OpenFile.h"
 #include "Field.h"
 #include "OS.h"
-#ifdef HAVE_ONELAB2
-#include "onelab2Group.h"
-#else
 #include "onelabGroup.h"
-#endif
 #include "viewButton.h"
 
 static void view_toggle_cb(Fl_Widget *w, void *data)
