@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include "GmshConfig.h"
 #if !defined(HAVE_NO_STDINT_H)
@@ -24,6 +24,7 @@ typedef unsigned long intptr_t;
 #include "paletteWindow.h"
 #include "contextWindow.h"
 #include "graphicWindow.h"
+#include "openglWindow.h"
 #include "GmshDefines.h"
 #include "GmshMessage.h"
 #include "GModel.h"
@@ -325,14 +326,6 @@ static void _rebuild_list_browser()
     if(VisibilityList::instance()->getVisibility(i))
       FlGui::instance()->visibility->browser->select(i + 1);
   }
-
-#if 0
-  // activate/deactivate delete button
-  if(type == VisibilityList::PhysicalEntities)
-    FlGui::instance()->visibility->push[0]->activate();
-  else
-    FlGui::instance()->visibility->push[0]->deactivate();
-#endif
 }
 
 static void visibility_browser_apply_cb(Fl_Widget *w, void *data)
@@ -362,31 +355,6 @@ static void visibility_browser_apply_cb(Fl_Widget *w, void *data)
   }
 }
 
-#if 0
-static void visibility_delete_cb(Fl_Widget *w, void *data)
-{
-  bool all = true;
-  for(int i = 0; i < VisibilityList::instance()->getNumEntities(); i++){
-    if(!FlGui::instance()->visibility->browser->selected(i + 1)){
-      all = false;
-      break;
-    }
-  }
-  if(all){
-    GModel::current()->deletePhysicalGroups();
-  }
-  else{
-    for(int i = 0; i < VisibilityList::instance()->getNumEntities(); i++){
-      if(FlGui::instance()->visibility->browser->selected(i + 1)){
-        Vis *v = VisibilityList::instance()->getEntity(i);
-        GModel::current()->deletePhysicalGroup(v->getDim(), v->getTag());
-      }
-    }
-  }
-  visibility_cb(NULL, (void*)"redraw_only");
-}
-#endif
-
 static void visibility_sort_cb(Fl_Widget *w, void *data)
 {
   const char *str = (const char*)data;
@@ -431,12 +399,12 @@ static void visibility_sort_cb(Fl_Widget *w, void *data)
       if(FlGui::instance()->visibility->browser->selected(i + 1)){
         static char tmpstr[256];
         sprintf(tmpstr, "%d", VisibilityList::instance()->getTag(i));
-        FlGui::instance()->geoContext->input[1]->value(tmpstr);
+        FlGui::instance()->elementaryContext->input[1]->value(tmpstr);
         break;
       }
     }
-    FlGui::instance()->geoContext->input[0]->value("NewName");
-    FlGui::instance()->geoContext->show(0);
+    FlGui::instance()->elementaryContext->input[0]->value("NewName");
+    FlGui::instance()->elementaryContext->show(0);
   }
   else { // set new sorting mode
     VisibilityList::instance()->setSortMode(val);
@@ -1146,7 +1114,7 @@ visibilityWindow::visibilityWindow(int deltaFontSize)
 {
   FL_NORMAL_SIZE -= deltaFontSize;
 
-  int CC = (int)(1.1 * BB);
+  int CC = (int)(1.18 * BB);
 
   static int cols[5] = {3 * WB, CC, CC, 2 * CC, 0};
   int width = cols[0] + cols[1] + cols[2] + cols[3] + 7 * WB;
@@ -1238,13 +1206,6 @@ visibilityWindow::visibilityWindow(int deltaFontSize)
     browser_type->menu(browser_type_table);
     browser_type->value(2); // physicals
 
-    // "Delete" is out of place in a Visibility window - it's a destructive operation!
-#if 0
-    push[0] = new Fl_Button
-      (width - 2 * CC - 3 * WB, height - 2 * BH - 3 * WB, CC, BH, "Delete");
-    push[0]->callback(visibility_delete_cb);
-#endif
-
     Fl_Return_Button *b1 = new Fl_Return_Button
       (width - 1 * CC - 2 * WB, height - 2 * BH - 3 * WB, CC, BH, "Apply");
     b1->callback(visibility_browser_apply_cb);
