@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <sstream>
 #include <algorithm>
@@ -15,6 +15,9 @@
 #include "GaussLegendre1D.h"
 #include "Context.h"
 #include "closestPoint.h"
+#if defined(HAVE_MESH)
+#include "meshGEdge.h"
+#endif
 
 GEdge::GEdge(GModel *model, int tag, GVertex *_v0, GVertex *_v1)
   : GEntity(model, tag), _length(0.), _tooSmall(false), _cp(0),
@@ -562,7 +565,7 @@ void GEdge::relocateMeshVertices()
   }
 }
 
-SPoint3 GEdge :: closestPoint (SPoint3 &p, double tolerance)
+SPoint3 GEdge::closestPoint(SPoint3 &p, double tolerance)
 {
   if (!_cp || _cp->tol() != tolerance)    {
     if(_cp)printf("coucou %12.15E %22.15E \n",tolerance,_cp->tol());
@@ -627,3 +630,11 @@ void GEdge::discretize(double tol, std::vector<SPoint3> &dpts, std::vector<doubl
     ts.push_back(upts[p].t);
   }
 }
+
+void GEdge::mesh(bool verbose)
+{
+#if defined(HAVE_MESH)
+  meshGEdge mesher;
+  mesher(this);
+#endif
+}
