@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _GENTITY_H_
 #define _GENTITY_H_
@@ -107,6 +107,7 @@ class GEntity {
     SurfaceOfRevolution,
     BoundaryLayerSurface,
     DiscreteSurface,
+    DiscreteDiskSurface,
     CompoundSurface,
     Volume,
     DiscreteVolume,
@@ -157,6 +158,7 @@ class GEntity {
       "Surface of Revolution",
       "Boundary layer surface",
       "Discrete surface",
+      "Discrete surface (parametrizable, isomorphic to a disk)",
       "Compound surface",
       "Volume",
       "Discrete volume",
@@ -176,6 +178,10 @@ class GEntity {
 
   virtual ~GEntity(){}
 
+
+  // mesh generation of the entity
+  virtual void mesh(bool verbose) {}
+  
   // delete the mesh data
   virtual void deleteMesh(){}
 
