@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _GFACE_H_
 #define _GFACE_H_
@@ -74,6 +74,9 @@ class GFace : public GEntity{
   GFace(GModel *model, int tag);
   virtual ~GFace();
 
+
+  std::set<MVertex*> constr_vertices;
+
   // delete mesh data
   virtual void deleteMesh();
 
@@ -293,6 +296,9 @@ class GFace : public GEntity{
   // tells if it's a sphere, and if it is, returns parameters
   virtual bool isSphere(double &radius, SPoint3 &center) const { return false; }
 
+  // new interface for meshing
+  virtual void mesh(bool verbose);
+
   // add layers of quads
   void addLayersOfQuads(int nLayers, GVertex *start, double hmin, double factor);
 
@@ -323,7 +329,7 @@ class GFace : public GEntity{
   int getCurvatureControlParameter () const;
   void setCurvatureControlParameter(int);
   virtual double getMeshSize() const { return meshAttributes.meshSize; }
-
+  
   struct {
     mutable GEntity::MeshGenerationStatus status;
     double worst_element_shape, best_element_shape, average_element_shape;
