@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _GMODEL_H_
 #define _GMODEL_H_
@@ -119,6 +119,9 @@ class GModel
   // the fly if needed
   void _storeElementsInEntities(std::map<int, std::vector<MElement*> > &map);
 
+  // Discrete Entities have to have their mesh moved to a geometry container
+  void _createGeometryOfDiscreteEntities(bool force=false);
+
   // loop over all vertices connected to elements and associate
   // geometrical entity
   void _associateEntityWithMeshVertices();
@@ -220,11 +223,7 @@ class GModel
   std::string getName(){ return _name; }
 
   // get/set the model file name
-  void setFileName(std::string fileName)
-  {
-    _fileName = fileName;
-    _fileNames.insert(fileName);
-  }
+  void setFileName(std::string fileName);
   std::string getFileName(){ return _fileName; }
   bool hasFileName(const std::string &name)
   {
@@ -362,7 +361,7 @@ class GModel
 
   // get the number of each type of element in the mesh at the largest
   // dimension and return the dimension
-  int getNumMeshElements(unsigned c[5]);
+  int getNumMeshElements(unsigned c[6]);
 
   // access a mesh element by coordinates (using an octree search)
   MElement *getMeshElementByCoord(SPoint3 &p, int dim=-1, bool strict=true);
@@ -467,6 +466,7 @@ class GModel
 
   // Ensure that the Jacobian of all volume elements is positive
   bool setAllVolumesPositive();
+  void setAllVolumesPositiveTopology();
 
   // make the mesh a high order mesh at order N
   // linear is 1 if the high order points are not placed on the geometry of the model
@@ -486,7 +486,6 @@ class GModel
   void fillVertexArrays();
 
   // reclassify a mesh i.e. use an angle threshold to tag edges faces and regions
-  void detectEdges(double _tresholdAngle);
   void classifyFaces(std::set<GFace*> &_faces);
 
   // glue entities in the model (assume a tolerance eps and merge
@@ -613,6 +612,10 @@ class GModel
   int writeOCCSTEP(const std::string &name);
   int writeOCCBREP(const std::string &name);
   int importOCCShape(const void *shape);
+  GVertex *getVertexForOCCShape(const void *shape);
+  GEdge *getEdgeForOCCShape(const void *shape);
+  GFace *getFaceForOCCShape(const void *shape);
+  GRegion *getRegionForOCCShape(const void *shape);
 
   // ACIS Model
   int readACISSAT(const std::string &name);
