@@ -1,8 +1,10 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
+#include <sstream>
+#include <iomanip>
 #include "GModel.h"
 #include "OS.h"
 #include "GmshMessage.h"
@@ -15,38 +17,94 @@
 #include "MHexahedron.h"
 #include "MPrism.h"
 #include "MPyramid.h"
+#include "MTrihedron.h"
 #include "StringUtils.h"
+#include "discreteVertex.h"
+#include "discreteEdge.h"
+#include "discreteFace.h"
+#include "discreteRegion.h"
 
-void writeMSHPeriodicNodes(FILE *fp, std::vector<GEntity*> &entities)
+static int readMSHPhysicals(FILE *fp, GEntity *ge)
 {
-  int count = 0;
-  for (unsigned int i = 0; i < entities.size(); i++)
-    if (entities[i]->meshMaster() != entities[i]) count++;
-  if (!count) return;
-  fprintf(fp, "$Periodic\n");
-  fprintf(fp, "%d\n", count);
-  for(unsigned int i = 0; i < entities.size(); i++){
-    GEntity *g_slave  = entities[i];
-    GEntity *g_master = g_slave->meshMaster();
-    if (g_slave != g_master){
-      fprintf(fp,"%d %d %d\n", g_slave->dim(), g_slave->tag(), g_master->tag());
+  int nump;
+  if(fscanf(fp, "%d", &nump) != 1) return 0;
+  for(int i = 0; i < nump; i++){
+    int tag;
+    if(fscanf(fp, "%d", &tag) != 1) return 0;
+    ge->physicals.push_back(tag);
+  }
+  return 1;
+}
 
-      if (g_slave->affineTransform.size() == 16) {
-        fprintf(fp,"Affine");
-        for (int i=0;i<16;i++) fprintf(fp," %.16g",g_slave->affineTransform[i]);
-        fprintf(fp,"\n");
+static void readMSHEntities(FILE *fp, GModel *gm)
+{
+  int nv, ne, nf, nr;
+  int tag;
+  if(fscanf(fp, "%d %d %d %d", &nv, &ne, &nf, &nr) != 4) return;
+  for (int i = 0; i < nv; i++){
+    if(fscanf(fp, "%d", &tag) != 1) return;
+    GVertex *gv = gm->getVertexByTag(tag);
+    if (!gv){
+      gv = new discreteVertex(gm, tag);
+      gm->add(gv);
+    }
+    if(!readMSHPhysicals(fp, gv)) return;
+  }
+  for (int i = 0; i < ne; i++){
+    int n;
+    if(fscanf(fp, "%d %d", &tag, &n) != 2) return;
+    GEdge *ge = gm->getEdgeByTag(tag);
+    if (!ge){
+      GVertex *v1 = 0, *v2 = 0;
+      for(int j = 0; j < n; j++){
+        int tagv;
+        if(fscanf(fp, "%d", &tagv) != 1) return;
+        GVertex *v = gm->getVertexByTag(tagv);
+        if (!v) Msg::Error("Unknown GVertex %d", tagv);
+        if(j == 0) v1 = v;
+        if(j == 1) v2 = v;
+      }
+      ge = new discreteEdge(gm, tag, v1, v2);
+      gm->add(ge);
+    }
+    if(!readMSHPhysicals(fp, ge)) return;
+  }
+  for (int i = 0; i < nf; i++){
+    int n;
+    if(fscanf(fp, "%d %d", &tag, &n) != 2) return;
+    GFace *gf = gm->getFaceByTag(tag);
+    if (!gf){
+      discreteFace *df = new discreteFace(gm, tag);
+      std::vector<int> edges;
+      for (int j = 0; j < n; j++){
+	int tage;
+	if(fscanf(fp, "%d", &tage) != 1) return;
+	edges.push_back(tage);
       }
-
-      fprintf(fp,"%d\n", (int) g_slave->correspondingVertices.size());
-      for (std::map<MVertex*,MVertex*>::iterator it = g_slave->correspondingVertices.begin();
-           it != g_slave->correspondingVertices.end(); it++){
-        MVertex *v1 = it->first;
-        MVertex *v2 = it->second;
-        fprintf(fp,"%d %d\n", v1->getIndex(), v2->getIndex());
+      df->setBoundEdges(gm, edges);
+      gm->add(df);
+      gf = df;
+    }
+    if(!readMSHPhysicals(fp, gf)) return;
+  }
+  for (int i = 0; i < nr; i++){
+    int n;
+    if(fscanf(fp, "%d %d", &tag, &n) != 2) return;
+    GRegion *gr = gm->getRegionByTag(tag);
+    if (!gr){
+      discreteRegion *dr = new discreteRegion(gm, tag);
+      std::set<int> faces;
+      for (int j = 0; j < n; j++){
+	int tagf;
+	if(fscanf(fp, "%d", &tagf) != 1) return;
+	faces.insert(tagf);
       }
+      dr->setBoundFaces(faces);
+      gm->add(dr);
+      gr = dr;
     }
+    if(!readMSHPhysicals(fp, gr)) return;
   }
-  fprintf(fp, "$EndPeriodic\n");
 }
 
 void readMSHPeriodicNodes(FILE *fp, GModel *gm)
@@ -82,7 +140,7 @@ void readMSHPeriodicNodes(FILE *fp, GModel *gm)
       if(fscanf(fp, "%d", &numv) != 1) numv = 0;
       for(int j = 0; j < numv; j++){
         int v1, v2;
-        if(fscanf(fp,"%d %d", &v1, &v2) != 2) continue;
+        if(fscanf(fp, "%d %d", &v1, &v2) != 2) continue;
         MVertex *mv1 = gm->getMeshVertexByTag(v1);
         MVertex *mv2 = gm->getMeshVertexByTag(v2);
         s->correspondingVertices[mv1] = mv2;
@@ -101,6 +159,7 @@ int GModel::readMSH(const std::string &name)
 
   char str[256] = "";
 
+
   // detect prehistoric MSH files
   if(!fgets(str, sizeof(str), fp)){ fclose(fp); return 0; }
   if(!strncmp(&str[1], "NOD", 3) || !strncmp(&str[1], "NOE", 3)){
@@ -113,8 +172,7 @@ int GModel::readMSH(const std::string &name)
   double version = 0.;
   bool binary = false, swap = false, postpro = false;
   int minVertex = 0;
-  std::map<int, std::vector<int> > entities[4];
-  std::map<int, std::vector<MElement*> > elements[10];
+  std::map<int, std::vector<MElement*> > elements[11];
 
   while(1) {
 
@@ -150,12 +208,6 @@ int GModel::readMSH(const std::string &name)
       }
     }
 
-    // FIXME:
-    // change this into "$PhysicalAttributes"
-    //  #attribs
-    //    dim num #nums num1 num2... #str str1 str2...
-    //    dum num ...
-
     // $PhysicalNames section
     else if(!strncmp(&str[1], "PhysicalNames", 13)) {
       if(!fgets(str, sizeof(str), fp)){ fclose(fp); return 0; }
@@ -173,23 +225,7 @@ int GModel::readMSH(const std::string &name)
 
     // $Entities section
     else if(!strncmp(&str[1], "Entities", 8)) {
-      if(!fgets(str, sizeof(str), fp)){ fclose(fp); return 0; }
-      int numEntities;
-      if(sscanf(str, "%d", &numEntities) != 1){ fclose(fp); return 0; }
-      for(int i = 0; i < numEntities; i++) {
-        int num, dim, numPhysicals;
-        if(fscanf(fp, "%d %d %d", &num, &dim, &numPhysicals) != 3){
-          fclose(fp);
-          return 0;
-        }
-        if(numPhysicals > 0){
-          std::vector<int> physicals(numPhysicals);
-          for(int j = 0; j < numPhysicals; j++){
-            if(fscanf(fp, "%d", &physicals[j]) != 1){ fclose(fp); return 0; }
-          }
-          entities[dim][num] = physicals;
-        }
-      }
+      readMSHEntities(fp, this);
     }
 
     // $Nodes section
@@ -318,7 +354,6 @@ int GModel::readMSH(const std::string &name)
         _vertexMapCache.clear();
       }
     }
-
     // $Elements section
     else if(!strncmp(&str[1], "Elements", 8)) {
       if(!fgets(str, sizeof(str), fp)){ fclose(fp); return 0; }
@@ -372,14 +407,20 @@ int GModel::readMSH(const std::string &name)
         case TYPE_HEX: elements[5][entity].push_back(element); break;
         case TYPE_PRI: elements[6][entity].push_back(element); break;
         case TYPE_PYR: elements[7][entity].push_back(element); break;
-        case TYPE_POLYG: elements[8][entity].push_back(element); break;
-        case TYPE_POLYH: elements[9][entity].push_back(element); break;
+        case TYPE_TRIH: elements[8][entity].push_back(element); break;
+        case TYPE_POLYG: elements[9][entity].push_back(element); break;
+        case TYPE_POLYH: elements[10][entity].push_back(element); break;
         }
         if(numElements > 100000)
           Msg::ProgressMeter(i + 1, numElements, true, "Reading elements");
       }
     }
 
+    // $Periodical section
+    else if(!strncmp(&str[1], "Periodical", 10)) {
+      readMSHPeriodicNodes(fp, this);
+    }
+
     // Post-processing sections
     else if(!strncmp(&str[1], "NodeData", 8) ||
             !strncmp(&str[1], "ElementData", 11) ||
@@ -387,9 +428,6 @@ int GModel::readMSH(const std::string &name)
       postpro = true;
       break;
     }
-    else if(!strncmp(&str[1], "Periodical", 10)) {
-      readMSHPeriodicNodes (fp, this);
-    }
 
     do {
       if(!fgets(str, sizeof(str), fp) || feof(fp))
@@ -401,7 +439,7 @@ int GModel::readMSH(const std::string &name)
   // entity does not exist, create a new (discrete) one.
   for(int i = 0; i < (int)(sizeof(elements) / sizeof(elements[0])); i++)
     _storeElementsInEntities(elements[i]);
-
+  
   // associate the correct geometrical entity with each mesh vertex
   _associateEntityWithMeshVertices();
 
@@ -411,27 +449,63 @@ int GModel::readMSH(const std::string &name)
   else
     _storeVerticesInEntities(_vertexMapCache);
 
-  // store physicals in entities
-  for(int dim = 0; dim < 4; dim++){
-    for(std::map<int, std::vector<int> >::iterator it = entities[dim].begin();
-        it != entities[dim].end(); it++){
-      GEntity *ge = 0;
-      switch(dim){
-      case 0: ge = getVertexByTag(it->first); break;
-      case 1: ge = getEdgeByTag(it->first); break;
-      case 2: ge = getFaceByTag(it->first); break;
-      case 3: ge = getRegionByTag(it->first); break;
-      }
-      if(ge)
-        ge->physicals = it->second;
-    }
-  }
+  _createGeometryOfDiscreteEntities() ;
 
   fclose(fp);
 
   return postpro ? 2 : 1;
 }
 
+static void writeMSHPhysicals(FILE *fp, GEntity *ge)
+{
+  std::vector<int> phys = ge->getPhysicalEntities();
+  fprintf(fp, "%d ", (int)phys.size());
+  for(std::vector<int>::iterator itp = phys.begin(); itp != phys.end(); itp++)
+    fprintf(fp, "%d ", *itp);
+}
+
+static void writeMSHEntities(FILE *fp, GModel *gm)
+{
+  fprintf(fp, "$Entities\n");
+  fprintf (fp, "%d %d %d %d\n", gm->getNumVertices(), gm->getNumEdges(),
+           gm->getNumFaces(), gm->getNumRegions());
+  for (GModel::viter it = gm->firstVertex(); it != gm->lastVertex(); ++it) {
+    fprintf(fp, "%d ", (*it)->tag());
+    writeMSHPhysicals(fp, *it);
+    fprintf(fp, "\n");
+  }
+  for (GModel::eiter it = gm->firstEdge(); it != gm->lastEdge(); ++it) {
+    std::list<GVertex*> vertices;
+    if((*it)->getBeginVertex()) vertices.push_back((*it)->getBeginVertex());
+    if((*it)->getEndVertex()) vertices.push_back((*it)->getEndVertex());
+    fprintf(fp, "%d %d ", (*it)->tag(), (int)vertices.size());
+    for(std::list<GVertex*>::iterator itv = vertices.begin(); itv != vertices.end(); itv++){
+      fprintf(fp, "%d ", (*itv)->tag());
+    }
+    writeMSHPhysicals(fp, *it);
+    fprintf(fp, "\n");
+  }
+  for (GModel::fiter it = gm->firstFace(); it != gm->lastFace(); ++it) {
+    std::list<GEdge*> edges = (*it)->edges();
+    fprintf(fp, "%d %d ", (*it)->tag(), (int)edges.size());
+    for(std::list<GEdge*>::iterator ite = edges.begin(); ite != edges.end(); ite++){
+      fprintf(fp, "%d ", (*ite)->tag());
+    }
+    writeMSHPhysicals(fp, *it);
+    fprintf(fp, "\n");
+  }
+  for (GModel::riter it = gm->firstRegion(); it != gm->lastRegion(); ++it) {
+    std::list<GFace*> faces = (*it)->faces();
+    fprintf(fp, "%d %d ", (*it)->tag(), (int)faces.size());
+    for(std::list<GFace*>::iterator itf = faces.begin(); itf != faces.end(); itf++){
+      fprintf(fp, "%d ", (*itf)->tag());
+    }
+    writeMSHPhysicals(fp, *it);
+    fprintf(fp, "\n");
+  }
+  fprintf(fp, "$EndEntities\n");
+}
+
 static int getNumElementsMSH(GEntity *ge, bool saveAll, int saveSinglePartition)
 {
   if(!saveAll && ge->physicals.empty()) return 0;
@@ -476,6 +550,38 @@ static void writeElementsMSH(FILE *fp, GModel *model, GEntity *ge, std::vector<T
   }
 }
 
+void writeMSHPeriodicNodes(FILE *fp, std::vector<GEntity*> &entities)
+{
+  int count = 0;
+  for (unsigned int i = 0; i < entities.size(); i++)
+    if (entities[i]->meshMaster() != entities[i]) count++;
+  if (!count) return;
+  fprintf(fp, "$Periodic\n");
+  fprintf(fp, "%d\n", count);
+  for(unsigned int i = 0; i < entities.size(); i++){
+    GEntity *g_slave  = entities[i];
+    GEntity *g_master = g_slave->meshMaster();
+    if (g_slave != g_master){
+      fprintf(fp, "%d %d %d\n", g_slave->dim(), g_slave->tag(), g_master->tag());
+
+      if (g_slave->affineTransform.size() == 16) {
+        fprintf(fp, "Affine");
+        for (int i = 0; i < 16;i++) fprintf(fp, " %.16g", g_slave->affineTransform[i]);
+        fprintf(fp, "\n");
+      }
+
+      fprintf(fp, "%d\n", (int) g_slave->correspondingVertices.size());
+      for (std::map<MVertex*,MVertex*>::iterator it = g_slave->correspondingVertices.begin();
+           it != g_slave->correspondingVertices.end(); it++){
+        MVertex *v1 = it->first;
+        MVertex *v2 = it->second;
+        fprintf(fp, "%d %d\n", v1->getIndex(), v2->getIndex());
+      }
+    }
+  }
+  fprintf(fp, "$EndPeriodic\n");
+}
+
 int GModel::writeMSH(const std::string &name, double version, bool binary,
                      bool saveAll, bool saveParametric,
                      double scalingFactor, int elementStartNum,
@@ -526,33 +632,16 @@ int GModel::writeMSH(const std::string &name, double version, bool binary,
   if(numPhysicalNames()){
     fprintf(fp, "$PhysicalNames\n");
     fprintf(fp, "%d\n", numPhysicalNames());
-    for(piter it = firstPhysicalName(); it != lastPhysicalName(); it++)
+    for(piter it = firstPhysicalName(); it != lastPhysicalName(); it++){
+      std::string name = it->second;
+      if(name.size() > 254) name.resize(254);
       fprintf(fp, "%d %d \"%s\"\n", it->first.first, it->first.second,
-              it->second.c_str());
+              name.c_str());
+    }
     fprintf(fp, "$EndPhysicalNames\n");
   }
 
-  fprintf(fp, "$Entities\n");
-  std::vector<GEntity*> entitiesWithPhysical;
-  if(saveAll)
-    entitiesWithPhysical = entities;
-  else{
-    for(unsigned int i = 0; i < entities.size(); i++){
-      GEntity *ge = entities[i];
-      std::vector<int> physicallist = ge->getPhysicalEntities();
-      if(physicallist.size()) entitiesWithPhysical.push_back(ge);
-    }
-  }
-  fprintf(fp, "%d\n", (int)entitiesWithPhysical.size());
-  for(unsigned int i = 0; i < entitiesWithPhysical.size(); i++){
-    GEntity *ge = entitiesWithPhysical[i];
-    std::vector<int> physicallist = ge->getPhysicalEntities();
-    fprintf(fp, "%d %d %d", ge->tag(), ge->dim(), (int)physicallist.size());
-    for(unsigned int j = 0; j < physicallist.size(); j++)
-      fprintf(fp, " %d", physicallist[j]);
-    fprintf(fp, "\n");
-  }
-  fprintf(fp, "$EndEntities\n");
+  writeMSHEntities(fp, this);
 
   fprintf(fp, "$Nodes\n");
   fprintf(fp, "%d\n", numVertices);
@@ -580,6 +669,9 @@ int GModel::writeMSH(const std::string &name, double version, bool binary,
   for(riter it = firstRegion(); it != lastRegion(); ++it)
     writeElementsMSH(fp, this, *it, (*it)->pyramids, saveAll, saveSinglePartition,
                      binary);
+  for(riter it = firstRegion(); it != lastRegion(); ++it)
+    writeElementsMSH(fp, this, *it, (*it)->trihedra, saveAll, saveSinglePartition,
+                     binary);
   for(fiter it = firstFace(); it != lastFace(); ++it)
     writeElementsMSH(fp, this, *it, (*it)->triangles, saveAll, saveSinglePartition,
                      binary);
@@ -598,9 +690,29 @@ int GModel::writeMSH(const std::string &name, double version, bool binary,
   fprintf(fp, "$EndElements\n");
 
   //save periodic nodes
-  writeMSHPeriodicNodes (fp,entities);
+  writeMSHPeriodicNodes(fp, entities);
 
   fclose(fp);
 
   return 1;
 }
+
+int GModel::writePartitionedMSH(const std::string &baseName, double version,
+                                bool binary, bool saveAll, bool saveParametric,
+                                double scalingFactor)
+{
+  if(version < 3)
+    return _writePartitionedMSH2(baseName, binary, saveAll, saveParametric,
+                                 scalingFactor);
+
+  for(std::set<int>::iterator it = meshPartitions.begin();
+      it != meshPartitions.end(); it++){
+    int partition = *it;
+    std::ostringstream sstream;
+    sstream << baseName << "_" << std::setw(6) << std::setfill('0') << partition;
+    Msg::Info("Writing partition %d in file '%s'", partition, sstream.str().c_str());
+    writeMSH(sstream.str(), version, binary, saveAll, saveParametric,
+             scalingFactor, 0, partition);
+  }
+  return 1;
+}
