@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <stdlib.h>
 #include <string.h>
@@ -296,7 +296,7 @@ int GModel::_readMSH2(const std::string &name)
           }
           if (iClasDim == 0){
             GVertex *gv = getVertexByTag(iClasTag);
-            if (gv) gv->deleteMesh();
+            if (gv) gv->mesh_vertices.clear();
             newVertex = new MVertex(xyz[0], xyz[1], xyz[2], gv, num);
           }
           else if (iClasDim == 1){
@@ -689,6 +689,7 @@ int GModel::_readMSH2(const std::string &name)
   for(int i = 0; i < 4; i++)
     _storePhysicalTagsInEntities(i, physicals[i]);
 
+  _createGeometryOfDiscreteEntities() ;
   fclose(fp);
 
   return postpro ? 2 : 1;
@@ -788,10 +789,8 @@ static int getNumElementsMSH(GEntity *ge, bool saveAll, int saveSinglePartition)
 
 static int getNumElementsMSH(GModel *m, bool saveAll, int saveSinglePartition)
 {
-
   int n = 0;
-  for(GModel::viter it = m->firstVertex(); it != m->lastVertex(); ++it)
-  {
+  for(GModel::viter it = m->firstVertex(); it != m->lastVertex(); ++it){
     n += getNumElementsMSH(*it, saveAll, saveSinglePartition);
     if(!CTX::instance()->mesh.saveTri){
       for(unsigned int i = 0; i < (*it)->points.size(); i++)
@@ -799,8 +798,7 @@ static int getNumElementsMSH(GModel *m, bool saveAll, int saveSinglePartition)
           n += (saveAll ? 1 : (*it)->physicals.size());
     }
   }
-  for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); ++it)
-  {
+  for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); ++it){
     n += getNumElementsMSH(*it, saveAll, saveSinglePartition);
     if(!CTX::instance()->mesh.saveTri){
       for(unsigned int i = 0; i < (*it)->lines.size(); i++)
@@ -808,18 +806,15 @@ static int getNumElementsMSH(GModel *m, bool saveAll, int saveSinglePartition)
           n += (saveAll ? 1 : (*it)->physicals.size());
     }
   }
-  for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it)
-  {
+  for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it){
     n += getNumElementsMSH(*it, saveAll, saveSinglePartition);
     if(CTX::instance()->mesh.saveTri){
-      for(unsigned int i = 0; i < (*it)->polygons.size(); i++)
-      {
+      for(unsigned int i = 0; i < (*it)->polygons.size(); i++){
         int nbC = (*it)->polygons[i]->getNumChildren()-1;
         n += (saveAll ? nbC : nbC * (*it)->physicals.size());
       }
     }
-    else
-    {
+    else{
       for(unsigned int i = 0; i < (*it)->triangles.size(); i++)
         if((*it)->triangles[i]->ownsParent())
           n += (saveAll ? 1 : (*it)->physicals.size());
@@ -828,18 +823,15 @@ static int getNumElementsMSH(GModel *m, bool saveAll, int saveSinglePartition)
           n += (saveAll ? 1 : (*it)->physicals.size());
     }
   }
-  for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it)
-  {
+  for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it){
     n += getNumElementsMSH(*it, saveAll, saveSinglePartition);
     if(CTX::instance()->mesh.saveTri){
-      for(unsigned int i = 0; i < (*it)->polyhedra.size(); i++)
-      {
+      for(unsigned int i = 0; i < (*it)->polyhedra.size(); i++){
         int nbC = (*it)->polyhedra[i]->getNumChildren()-1;
         n += (saveAll ? nbC : nbC * (*it)->physicals.size());
       }
     }
-    else
-    {
+    else{
       for(unsigned int i = 0; i < (*it)->tetrahedra.size(); i++)
         if((*it)->tetrahedra[i]->ownsParent())
           n += (saveAll ? 1 : (*it)->physicals.size());
@@ -847,6 +839,7 @@ static int getNumElementsMSH(GModel *m, bool saveAll, int saveSinglePartition)
         if((*it)->polyhedra[i]->ownsParent())
           n += (saveAll ? 1 : (*it)->physicals.size());
     }
+    n -= (*it)->trihedra.size();
   }
   return n;
 }
@@ -855,7 +848,6 @@ int GModel::_writeMSH2(const std::string &name, double version, bool binary,
                        bool saveAll, bool saveParametric, double scalingFactor,
                        int elementStartNum, int saveSinglePartition, bool multipleView)
 {
-
   FILE *fp;
   if(multipleView)
     fp = Fopen(name.c_str(), binary ? "ab" : "a");
@@ -895,9 +887,12 @@ int GModel::_writeMSH2(const std::string &name, double version, bool binary,
     if(numPhysicalNames()){
       fprintf(fp, "$PhysicalNames\n");
       fprintf(fp, "%d\n", numPhysicalNames());
-      for(piter it = firstPhysicalName(); it != lastPhysicalName(); it++)
+      for(piter it = firstPhysicalName(); it != lastPhysicalName(); it++){
+        std::string name = it->second;
+        if(name.size() > 128) name.resize(128);
         fprintf(fp, "%d %d \"%s\"\n", it->first.first, it->first.second,
-                it->second.c_str());
+                name.c_str());
+      }
       fprintf(fp, "$EndPhysicalNames\n");
     }
 
@@ -1063,11 +1058,12 @@ int GModel::_writeMSH2(const std::string &name, double version, bool binary,
   return 1;
 }
 
-int GModel::writePartitionedMSH(const std::string &baseName, bool binary,
-                                bool saveAll, bool saveParametric,
-                                double scalingFactor)
+int GModel::_writePartitionedMSH2(const std::string &baseName, bool binary,
+                                  bool saveAll, bool saveParametric,
+                                  double scalingFactor)
 {
-  int index = 0;
+  int numElements;
+  int startNum = 0;
   for(std::set<int>::iterator it = meshPartitions.begin();
       it != meshPartitions.end(); it++){
     int partition = *it;
@@ -1075,11 +1071,11 @@ int GModel::writePartitionedMSH(const std::string &baseName, bool binary,
     std::ostringstream sstream;
     sstream << baseName << "_" << std::setw(6) << std::setfill('0') << partition;
 
-    int startNum = index ? getNumElementsMSH(this, saveAll, partition) : 0;
+    numElements = getNumElementsMSH(this, saveAll, partition);
     Msg::Info("Writing partition %d in file '%s'", partition, sstream.str().c_str());
     _writeMSH2(sstream.str(), 2.2, binary, saveAll, saveParametric,
                scalingFactor, startNum, partition);
-    index++;
+    startNum += numElements; // update for next iteration in the loop
   }
 
 #if 0
