@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _MELEMENT_H_
 #define _MELEMENT_H_
@@ -9,7 +9,7 @@
 #include <stdio.h>
 #include <algorithm>
 #include <string>
-#include "GmshDefines.h"
+
 #include "GmshMessage.h"
 #include "MVertex.h"
 #include "MEdge.h"
@@ -19,6 +19,7 @@
 #include "JacobianBasis.h"
 #include "MetricBasis.h"
 #include "GaussIntegration.h"
+
 class GModel;
 
 // A mesh element.
