@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include "GmshConfig.h"
 #include "MTriangle.h"
@@ -65,7 +65,7 @@ double MTriangle::getOuterRadius()
 double MTriangle::angleShapeMeasure()
 {
 #if defined(HAVE_MESH)
-  return qmTriangleAngles(this);
+  return qmTriangle::angles(this);
 #else
   return 0.;
 #endif
@@ -73,20 +73,18 @@ double MTriangle::angleShapeMeasure()
 
 double MTriangle::etaShapeMeasure()
 {
-  double a1 = 180 * angle3Vertices(_v[0], _v[1], _v[2]) / M_PI;
-  double a2 = 180 * angle3Vertices(_v[1], _v[2], _v[0]) / M_PI;
-  double a3 = 180 * angle3Vertices(_v[2], _v[0], _v[1]) / M_PI;
-
-  double amin = std::min(std::min(a1,a2),a3);
-  double angle = fabs(60. - amin);
-  return 1.-angle/60;
+#if defined(HAVE_MESH)
+  return qmTriangle::eta(this);
+#else
+  return 0.;
+#endif
 }
 
 
 double MTriangle::gammaShapeMeasure()
 {
 #if defined(HAVE_MESH)
-  return qmTriangle(this, QMTRI_RHO);
+  return qmTriangle::gamma(this);
 #else
   return 0.;
 #endif
@@ -167,29 +165,32 @@ static void _myGetEdgeRep(MTriangle *t, int num, double *x, double *y, double *z
   }
 }
 
-void MTriangleN::getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MTriangleN::getEdgeRep(bool curved, int num,
+                            double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetEdgeRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MTriangle::getEdgeRep(false, num, x, y, z, n);
 }
 
-void MTriangle6::getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MTriangle6::getEdgeRep(bool curved, int num,
+                            double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetEdgeRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MTriangle::getEdgeRep(false, num, x, y, z, n);
 }
 
-int MTriangle6::getNumFacesRep(bool curved) {
+int MTriangle6::getNumFacesRep(bool curved)
+{
   return curved ? SQU(CTX::instance()->mesh.numSubEdges) : 1;
 }
-int MTriangleN::getNumFacesRep(bool curved) {
+int MTriangleN::getNumFacesRep(bool curved)
+{
   return curved ? SQU(CTX::instance()->mesh.numSubEdges) : 1;
 }
 
 static void _myGetFaceRep(MTriangle *t, int num, double *x, double *y, double *z,
                           SVector3 *n, int numSubEdges)
 {
-
   // on the first layer, we have (numSubEdges-1) * 2 + 1 triangles
   // on the second layer, we have (numSubEdges-2) * 2 + 1 triangles
   // on the ith layer, we have (numSubEdges-1-i) * 2 + 1 triangles
@@ -249,12 +250,14 @@ static void _myGetFaceRep(MTriangle *t, int num, double *x, double *y, double *z
   z[0] = pnt1.z(); z[1] = pnt2.z(); z[2] = pnt3.z();
 }
 
-void MTriangleN::getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MTriangleN::getFaceRep(bool curved, int num,
+                            double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetFaceRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MTriangle::getFaceRep(false, num, x, y, z, n);
 }
-void MTriangle6::getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n)
+void MTriangle6::getFaceRep(bool curved, int num,
+                            double *x, double *y, double *z, SVector3 *n)
 {
   if (curved) _myGetFaceRep(this, num, x, y, z, n, CTX::instance()->mesh.numSubEdges);
   else MTriangle::getFaceRep(false, num, x, y, z, n);
