@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <vector>
 #include <list>
@@ -22,6 +22,11 @@
 #include "Curvature.h"
 #include "GEdgeCompound.h"
 
+#if defined(HAVE_MESH)
+#include "meshGEdge.h"
+#include "Context.h"
+#endif
+
 discreteEdge::discreteEdge(GModel *model, int num, GVertex *_v0, GVertex *_v1)
   : GEdge(model, num, _v0, _v1)
 {
@@ -33,13 +38,14 @@ discreteEdge::discreteEdge(GModel *model, int num, GVertex *_v0, GVertex *_v1)
 
 void discreteEdge::createTopo()
 {
-  if(!createdTopo){ 
+  if(!createdTopo){
     orderMLines();
     setBoundVertices();
     createdTopo = true;
   }
 }
 
+// FULL OF BUGS !!!!!!
 void discreteEdge::orderMLines()
 {
   //printf("ordering line %d\n", tag());
@@ -54,7 +60,7 @@ void discreteEdge::orderMLines()
   }
 
   // find a lonly MLine
-  for (std::list<MLine*>::iterator it = segments.begin(); 
+  for (std::list<MLine*>::iterator it = segments.begin();
        it != segments.end(); ++it){
     MVertex *vL = (*it)->getVertex(0);
     MVertex *vR = (*it)->getVertex(1);
@@ -70,7 +76,7 @@ void discreteEdge::orderMLines()
   MLine *firstLine;
   if (boundv.size() == 2){   // non periodic
     firstLine = (boundv.begin())->second;
-    for (std::list<MLine*>::iterator it = segments.begin(); 
+    for (std::list<MLine*>::iterator it = segments.begin();
          it != segments.end(); ++it){
       if (*it == firstLine){
         segments.erase(it);
@@ -83,7 +89,7 @@ void discreteEdge::orderMLines()
     segments.erase(segments.begin());
   }
   else{
-    Msg::Error("EdgeCompound %d is wrong (it has %d end points)", 
+    Msg::Error("EdgeCompound %d is wrong (it has %d end points)",
                tag(), boundv.size());
   }
 
@@ -95,7 +101,7 @@ void discreteEdge::orderMLines()
   while (first != last){
     if (segments.empty())break;
     bool found = false;
-    for (std::list<MLine*>::iterator it = segments.begin(); 
+    for (std::list<MLine*>::iterator it = segments.begin();
          it != segments.end(); ++it){
       MLine *e = *it;
       if (e->getVertex(0) == last){
@@ -131,25 +137,23 @@ void discreteEdge::orderMLines()
 
   //lines is now a list of ordered MLines
   lines = _m;
-  
+
   //mesh_vertices
   mesh_vertices.clear();
   for (unsigned int i = 0; i < lines.size(); ++i){
     MVertex *v1 = lines[i]->getVertex(0);
     MVertex *v2 = lines[i]->getVertex(1);
-    if (std::find(mesh_vertices.begin(), mesh_vertices.end(), v1) ==  
+    if (std::find(mesh_vertices.begin(), mesh_vertices.end(), v1) ==
         mesh_vertices.end()) mesh_vertices.push_back(v1);
-    if (std::find(mesh_vertices.begin(), mesh_vertices.end(), v2) ==  
+    if (std::find(mesh_vertices.begin(), mesh_vertices.end(), v2) ==
         mesh_vertices.end()) mesh_vertices.push_back(v2);
-    v1->setEntity(this);
-    v2->setEntity(this);
   }
 
   //special case reverse orientation
   if (lines.size() < 2) return;
   if (_orientation[0] && lines[0]->getVertex(1) != lines[1]->getVertex(1)
       && lines[0]->getVertex(1) != lines[1]->getVertex(0)){
-    for (unsigned int i = 0; i < lines.size(); i++) 
+    for (unsigned int i = 0; i < lines.size(); i++)
       _orientation[i] = !_orientation[i];
   }
 }
@@ -158,11 +162,11 @@ void discreteEdge::setBoundVertices()
 {
   if (boundv.size() == 2){
     std::vector<GVertex*> bound_vertices;
-    for (std::map<MVertex*,MLine*>::const_iterator iter = boundv.begin(); 
+    for (std::map<MVertex*,MLine*>::const_iterator iter = boundv.begin();
          iter != boundv.end(); iter++){
       MVertex* vE = (iter)->first;
       bool existVertex  = false;
-      for(GModel::viter point = model()->firstVertex(); 
+      for(GModel::viter point = model()->firstVertex();
           point != model()->lastVertex(); point++){
         if ((*point)->mesh_vertices[0]->getNum() == vE->getNum()){
           bound_vertices.push_back((*point));
@@ -172,7 +176,7 @@ void discreteEdge::setBoundVertices()
       }
       if(!existVertex){
         GVertex *gvB = new discreteVertex
-          (model(), model()->getMaxElementaryNumber(0) + 1);      
+          (model(), model()->getMaxElementaryNumber(0) + 1);
         //printf("*** Created discreteVertex %d\n", gvB->tag());
         bound_vertices.push_back(gvB);
         vE->setEntity(gvB);
@@ -180,22 +184,22 @@ void discreteEdge::setBoundVertices()
         gvB->points.push_back(new MPoint(gvB->mesh_vertices.back()));
         model()->add(gvB);
       }
-      std::vector<MVertex*>::iterator itve = 
+      std::vector<MVertex*>::iterator itve =
         std::find(mesh_vertices.begin(), mesh_vertices.end(), vE);
       if (itve != mesh_vertices.end()) mesh_vertices.erase(itve);
 
       for(GModel::eiter edge = model()->firstEdge(); edge != model()->lastEdge(); edge++){
-        std::vector<MVertex*>::iterator itve = 
+        std::vector<MVertex*>::iterator itve =
           std::find((*edge)->mesh_vertices.begin(), (*edge)->mesh_vertices.end(), vE);
         if (itve != (*edge)->mesh_vertices.end()) (*edge)->mesh_vertices.erase(itve);
       }
       for(GModel::fiter face = model()->firstFace(); face != model()->lastFace(); face++){
-        std::vector<MVertex*>::iterator itve = 
+        std::vector<MVertex*>::iterator itve =
           std::find((*face)->mesh_vertices.begin(), (*face)->mesh_vertices.end(), vE);
         if (itve != (*face)->mesh_vertices.end()) (*face)->mesh_vertices.erase(itve);
       }
       for(GModel::riter reg = model()->firstRegion(); reg != model()->lastRegion(); reg++){
-        std::vector<MVertex*>::iterator itve = 
+        std::vector<MVertex*>::iterator itve =
           std::find((*reg)->mesh_vertices.begin(), (*reg)->mesh_vertices.end(), vE);
         if (itve != (*reg)->mesh_vertices.end()) (*reg)->mesh_vertices.erase(itve);
       }
@@ -210,7 +214,7 @@ void discreteEdge::setBoundVertices()
     std::vector<MLine*>::const_iterator it = lines.begin();
     MVertex* vE = (*it)->getVertex(0);
     bool existVertex  = false;
-    for(GModel::viter point = model()->firstVertex(); 
+    for(GModel::viter point = model()->firstVertex();
         point != model()->lastVertex(); point++){
       if ((*point)->mesh_vertices[0]->getNum() == vE->getNum()){
         bound_vertex = (*point);
@@ -229,25 +233,25 @@ void discreteEdge::setBoundVertices()
       gvB->points.push_back(new MPoint(gvB->mesh_vertices.back()));
       model()->add(gvB);
     }
-    std::vector<MVertex*>::iterator itve = 
+    std::vector<MVertex*>::iterator itve =
       std::find(mesh_vertices.begin(),mesh_vertices.end(), vE);
     if (itve != mesh_vertices.end()) mesh_vertices.erase(itve);
-   
-    for(GModel::eiter edge = model()->firstEdge(); 
+
+    for(GModel::eiter edge = model()->firstEdge();
         edge != model()->lastEdge(); edge++){
-      std::vector<MVertex*>::iterator itve = 
+      std::vector<MVertex*>::iterator itve =
         std::find((*edge)->mesh_vertices.begin(), (*edge)->mesh_vertices.end(), vE);
       if (itve != (*edge)->mesh_vertices.end()) (*edge)->mesh_vertices.erase(itve);
     }
-    for(GModel::fiter face = model()->firstFace(); 
+    for(GModel::fiter face = model()->firstFace();
         face != model()->lastFace(); face++){
-      std::vector<MVertex*>::iterator itve = 
+      std::vector<MVertex*>::iterator itve =
         std::find((*face)->mesh_vertices.begin(), (*face)->mesh_vertices.end(), vE);
       if (itve != (*face)->mesh_vertices.end()) (*face)->mesh_vertices.erase(itve);
     }
-    for(GModel::riter reg = model()->firstRegion(); 
+    for(GModel::riter reg = model()->firstRegion();
         reg != model()->lastRegion(); reg++){
-      std::vector<MVertex*>::iterator itve = 
+      std::vector<MVertex*>::iterator itve =
         std::find((*reg)->mesh_vertices.begin(), (*reg)->mesh_vertices.end(), vE);
       if (itve != (*reg)->mesh_vertices.end()) (*reg)->mesh_vertices.erase(itve);
     }
@@ -265,21 +269,21 @@ void discreteEdge::setBoundVertices()
     _pars[0]=0   _pars[1]=1    _pars[2]=2             _pars[N+1]=N+1
 */
 void discreteEdge::parametrize(std::map<GFace*, std::map<MVertex*, MVertex*,
-                               std::less<MVertex*> > > &face2Vert, 
+                               std::less<MVertex*> > > &face2Vert,
                                std::map<GRegion*, std::map<MVertex*, MVertex*,
                                std::less<MVertex*> > > &region2Vert)
-{ 
+{
   if (_pars.empty()){
     for (unsigned int i = 0; i < lines.size() + 1; i++){
       _pars.push_back(i);
     }
   }
   //Replace MVertex by MedgeVertex
-  std::map<MVertex*, MVertex*, std::less<MVertex*> > old2new;   
+  std::map<MVertex*, MVertex*, std::less<MVertex*> > old2new;
   old2new.clear();
   std::vector<MVertex* > newVertices;
   std::vector<MLine*> newLines;
-  
+
   MVertex *vL = getBeginVertex()->mesh_vertices[0];
   int i = 0;
   for(i = 0; i < (int)lines.size() - 1; i++){
@@ -287,7 +291,7 @@ void discreteEdge::parametrize(std::map<GFace*, std::map<MVertex*, MVertex*,
     if (_orientation[i] == 1 ) vR = lines[i]->getVertex(1);
     else vR = lines[i]->getVertex(0);
     int param = i+1;
-    MVertex *vNEW = new MEdgeVertex(vR->x(),vR->y(),vR->z(), this, 
+    MVertex *vNEW = new MEdgeVertex(vR->x(),vR->y(),vR->z(), this,
                                     param, -1., vR->getNum());
     old2new.insert(std::make_pair(vR,vNEW));
     newVertices.push_back(vNEW);
@@ -305,15 +309,15 @@ void discreteEdge::parametrize(std::map<GFace*, std::map<MVertex*, MVertex*,
   mesh_vertices = newVertices;
   lines.clear();
   lines = newLines;
-  
+
   //  we need to recreate lines, triangles and tets
   //  that contain those new MEdgeVertices
-  
-   for(std::list<GFace*>::iterator iFace = l_faces.begin(); 
+
+   for(std::list<GFace*>::iterator iFace = l_faces.begin();
        iFace != l_faces.end(); ++iFace){
 
      // for each face find correspondane face2Vertex
-     std::map<GFace*, std::map<MVertex*, MVertex*, 
+     std::map<GFace*, std::map<MVertex*, MVertex*,
        std::less<MVertex*> > >::iterator itmap = face2Vert.find(*iFace);
      if (itmap == face2Vert.end()) {
        face2Vert.insert(std::make_pair(*iFace, old2new));
@@ -329,38 +333,38 @@ void discreteEdge::parametrize(std::map<GFace*, std::map<MVertex*, MVertex*,
      // do the same for regions
      for ( int j = 0; j < (*iFace)->numRegions(); j++){
        GRegion *r = (*iFace)->getRegion(j);
-       std::map<GRegion*,  std::map<MVertex*, MVertex*, 
+       std::map<GRegion*,  std::map<MVertex*, MVertex*,
          std::less<MVertex*> > >::iterator itmap = region2Vert.find(r);
        if (itmap == region2Vert.end()) {
 	 region2Vert.insert(std::make_pair(r, old2new));
        }
        else{
 	 std::map<MVertex*, MVertex*, std::less<MVertex*> > mapVert = itmap->second;
-	 for (std::map<MVertex*, MVertex*, std::less<MVertex*> >::iterator it = 
+	 for (std::map<MVertex*, MVertex*, std::less<MVertex*> >::iterator it =
                 old2new.begin(); it != old2new.end(); it++)
 	   mapVert.insert(*it);
 	 itmap->second = mapVert;
        }
      }
- 
+
    }
 }
 
 void discreteEdge::computeNormals () const
 {
   _normals.clear();
-  for (unsigned int i= 0; i < mesh_vertices.size(); i++) 
+  for (unsigned int i= 0; i < mesh_vertices.size(); i++)
     _normals[mesh_vertices[i]] = SVector3(0.0,0.0,0.0);
   _normals[getBeginVertex()->mesh_vertices[0]] = SVector3(0.0,0.0,0.0);
   _normals[getBeginVertex()->mesh_vertices[0]] = SVector3(0.0,0.0,0.0);
   double J[3][3];
 
-  for(std::list<GFace*>::const_iterator iFace = l_faces.begin(); 
+  for(std::list<GFace*>::const_iterator iFace = l_faces.begin();
       iFace != l_faces.end(); ++iFace){
     for (unsigned int i = 0; i < (*iFace)->triangles.size(); ++i){
       MTriangle *t = (*iFace)->triangles[i];
       for (int j = 0; j < 3; j++){
-        std::map<MVertex*, SVector3, std::less<MVertex*> >::iterator 
+        std::map<MVertex*, SVector3, std::less<MVertex*> >::iterator
           itn = _normals.find(t->getVertex(j));
         if (itn != _normals.end()){
           t->getJacobian(0, 0, 0, J);
@@ -368,7 +372,7 @@ void discreteEdge::computeNormals () const
           SVector3 d2(J[1][0], J[1][1], J[1][2]);
           SVector3 n = crossprod(d1, d2);
           n.normalize();
-          _normals[t->getVertex(j)] += n; 
+          _normals[t->getVertex(j)] += n;
         }
       }
     }
@@ -377,35 +381,18 @@ void discreteEdge::computeNormals () const
   for ( ; itn != _normals.end(); ++itn){
     itn->second.normalize();
   }
-
-// //smooth normals
-//   smooth_normals *normals = 0;
-//   normals = new smooth_normals(180.);
-  
-//   for ( itn = _normals.begin();  itn != _normals.end(); ++itn){
-//     MVertex *v = itn->first;
-//     SVector3 n = itn->second;
-//     normals->add(v->x(),v->y(),v->z(),n.x(),n.y(),n.z());
-//   }
-//   for ( itn = _normals.begin();  itn != _normals.end(); ++itn){
-//     double nx,ny,nz;
-//     MVertex *v = itn->first;
-//     normals->get(v->x(),v->y(),v->z(),nx,ny,nz);
-//     itn->second = SVector3(nx,ny,nz);
-//   }
-//   delete normals;
 }
 
+/// WE REWRITE WHAT FOLLOWS : THIS WAS WRONG (PAB -JFR)
+
 bool discreteEdge::getLocalParameter(const double &t, int &iLine,
                                      double &tLoc) const
 {
-  if(_pars.empty()) return false;
-  for (iLine = 0; iLine < (int)lines.size(); iLine++){
+  for (iLine = 0; iLine < (int)discrete_lines.size(); iLine++){
     double tmin = _pars[iLine];
     double tmax = _pars[iLine+1];
     if (t >= tmin && t <= tmax){
-      tLoc = _orientation[iLine] ? (t-tmin)/(tmax-tmin) :
-        1 - (t-tmin)/(tmax-tmin);
+      tLoc = (t-tmin)/(tmax-tmin);
       return true;
     }
   }
@@ -419,58 +406,14 @@ GPoint discreteEdge::point(double par) const
   if(!getLocalParameter(par, iEdge, tLoc)) return GPoint();
 
   double x, y, z;
-  MVertex *vB = lines[iEdge]->getVertex(0);
-  MVertex *vE = lines[iEdge]->getVertex(1);
-
-  const bool LINEARMESH = true; //false;
-  
-  if (LINEARMESH){
-    //linear Lagrange mesh
-    x = vB->x() + tLoc * (vE->x()- vB->x());
-    y = vB->y() + tLoc * (vE->y()- vB->y());
-    z = vB->z() + tLoc * (vE->z()- vB->z());
-    return GPoint(x,y,z);
-  }
-  else{
-    //curved PN triangle
-
-    if (_normals.empty() ) computeNormals();
-
-    const SVector3 n1 = _normals[vB];
-    const SVector3 n2 = _normals[vE];
-    
-    SPoint3 v1(vB->x(),vB->y(),vB->z());  
-    SPoint3 v2(vE->x(),vE->y(),vE->z());
-    
-    SVector3 b300,b030,b003;
-    SVector3 b210,b120;
-    double  w12,w21;
-
-    b300 = v1;
-    b030 = v2;
-
-    w12 = dot(v2 - v1, n1);
-    w21 = dot(v1 - v2, n2);
-    b210 = (2*v1 + v2 -w12*n1)*0.333; 
-    b120 = (2*v2 + v1 -w21*n2)*0.333;
-    
-    // tagged PN trinagles (sigma=1)
-    double theta = 0.0;
-    SVector3 d1 = v1+.33*(1-theta)*(v2-v1);
-    SVector3 d2 = v2+.33*(1-theta)*(v1-v2);
-    SVector3 X1 = 1/norm(n1)*n1;
-    SVector3 X2 = 1/norm(n2)*n2;
-    b210 = d1 - dot(X1,d1-v1)*X1;
-    b120 = d2 - dot(X2,d2-v2)*X2;
-
-    double U = tLoc;
-    double W = 1-U;
-    SVector3 point = b300*W*W*W+b030*U*U*U+b210*3.*W*W*U+b120*3.*W*U*U;
-
-    SPoint3 PP(point.x(), point.y(), point.z());
-    return GPoint(PP.x(),PP.y(),PP.z());
-  }
+  MVertex *vB = discrete_lines[iEdge]->getVertex(0);
+  MVertex *vE = discrete_lines[iEdge]->getVertex(1);
 
+  //linear Lagrange mesh
+  x = vB->x() + tLoc * (vE->x()- vB->x());
+  y = vB->y() + tLoc * (vE->y()- vB->y());
+  z = vB->z() + tLoc * (vE->z()- vB->z());
+  return GPoint(x,y,z,this,par);
 }
 
 SVector3 discreteEdge::firstDer(double par) const
@@ -479,14 +422,14 @@ SVector3 discreteEdge::firstDer(double par) const
   int iEdge;
   if(!getLocalParameter(par, iEdge, tLoc)) return SVector3();
 
-  MVertex *vB = lines[iEdge]->getVertex(0);
-  MVertex *vE = lines[iEdge]->getVertex(1);
+  MVertex *vB = discrete_lines[iEdge]->getVertex(0);
+  MVertex *vE = discrete_lines[iEdge]->getVertex(1);
 
   double dx, dy, dz;
-  double dt = 1.0;
-  dx = (vE->x() - vB->x()) / dt;
-  dy = (vE->y() - vB->y()) / dt;
-  dz = (vE->z() - vB->z()) / dt;
+  //  double dt = 1.0;
+  dx = (vE->x() - vB->x());// / dt;
+  dy = (vE->y() - vB->y());// / dt;
+  dz = (vE->z() - vB->z());// / dt;
 
   SVector3 der(dx, dy, dz);
   return der;
@@ -503,14 +446,13 @@ double discreteEdge::curvature(double par) const
   if( !Curvature::valueAlreadyComputed() ) {
     Msg::Warning("Need to compute discrete curvature (in discreteEdge)");
     Curvature::typeOfCurvature type = Curvature::RUSIN; //RUSIN; //RBF
-    curvature.computeCurvature(model(), type); 
+    curvature.computeCurvature(model(), type);
   }
 
   curvature.edgeNodalValues(lines[iEdge],c0, c1, 1);
   double cv = (1-tLoc)*c0 + tLoc*c1;
 
   return cv;
-
 }
 
 double discreteEdge::curvatures(const double par, SVector3 *dirMax, SVector3 *dirMin,
@@ -527,12 +469,82 @@ double discreteEdge::curvatures(const double par, SVector3 *dirMax, SVector3 *di
 
 Range<double> discreteEdge::parBounds(int i) const
 {
-  return Range<double>(0, lines.size());
+  return Range<double>(0, (double)discrete_lines.size());
+}
+
+void discreteEdge::createGeometry()
+{
+  if (discrete_lines.empty()){
+
+    createTopo();
+    // copy the mesh
+    for (unsigned int i = 0; i < mesh_vertices.size(); i++){
+      MEdgeVertex *v = new MEdgeVertex(mesh_vertices[i]->x(), mesh_vertices[i]->y(),
+                                       mesh_vertices[i]->z(), this, (double)(i+1));
+      v2v[mesh_vertices[i]] = v;
+    }
+
+    std::vector<MLine*> _temp;
+    for (unsigned int i = 0; i < lines.size(); i++){
+      MVertex *v0 = lines[i]->getVertex(0);
+      MVertex *v1 = lines[i]->getVertex(1);
+      MVertex *v00 = (v2v.find(v0) == v2v.end()) ? v0 : v2v[v0];
+      MVertex *v01 = (v2v.find(v1) == v2v.end()) ? v1 : v2v[v1];
+      if (_orientation[i] == 1){
+	discrete_lines.push_back(new MLine(v00,v01));
+      }
+      else{
+        lines[i]->setVertex(0, v1);
+        lines[i]->setVertex(1, v0);
+        _orientation[i] = 1;
+	discrete_lines.push_back(new MLine(v01,v00));
+      }
+    }
+    // compute parameters and recompute the vertices
+    _pars.push_back(0.0);
+    for (unsigned int i = 1; i < discrete_lines.size(); i++){
+      _pars.push_back((double)i);
+      MVertex *newv = discrete_lines[i]->getVertex(0);
+      discrete_vertices.push_back(newv);
+    }
+    _pars.push_back((double)discrete_lines.size());
+  }
+}
+
+MVertex * discreteEdge::getGeometricalVertex (MVertex *v)
+{
+  std::map<MVertex*,MVertex*>::const_iterator it = v2v.find(v);
+  if (it == v2v.end()){
+    Msg::Error("fatality %ld %ld %ld", v2v.size(), mesh_vertices.size(),
+               discrete_vertices.size());
+  }
+  return it->second;
+}
+
+void discreteEdge::interpolateInGeometry(MVertex *v, MVertex **v1,
+                                         MVertex **v2, double &xi) const
+{
+  double t;
+  if (v->onWhat() != this)Msg::Fatal("%s %d",__FILE__,__LINE__);
+  v->getParameter (0,t);
+  int i = (int) t;
+  MLine *l = discrete_lines [i];
+  *v1 = l->getVertex(0);
+  *v2 = l->getVertex(1);
+  xi = t - i;
+}
+
+void discreteEdge::mesh(bool verbose){
+#if defined(HAVE_MESH)
+  if (!CTX::instance()->meshDiscrete) return;
+  meshGEdge mesher;
+  mesher(this);
+#endif
 }
 
 void discreteEdge::writeGEO(FILE *fp)
 {
   if (getBeginVertex() && getEndVertex())
-    fprintf(fp, "Discrete Line(%d) = {%d,%d};\n", tag(), 
-            getBeginVertex()->tag(), getEndVertex()->tag());  
+    fprintf(fp, "Discrete Line(%d) = {%d,%d};\n", tag(),
+            getBeginVertex()->tag(), getEndVertex()->tag());
 }
