@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _DISCRETE_FACE_H_
 #define _DISCRETE_FACE_H_
@@ -11,7 +11,13 @@
 #include "discreteEdge.h"
 #include "MEdge.h"
 
+class discreteDiskFace;
+
 class discreteFace : public GFace {
+  // FIXME we should at the end use a
+  // mesh() functioon that is specific to
+  // discreteFace
+  // we should also SAVE those data's
  public:
   discreteFace(GModel *model, int num);
   virtual ~discreteFace() {}
@@ -24,11 +30,15 @@ class discreteFace : public GFace {
   virtual bool haveParametrization() { return getCompound(); }
   GEntity::GeomType geomType() const { return DiscreteSurface; }
   virtual Pair<SVector3, SVector3> firstDer(const SPoint2 &param) const;
-  virtual void secondDer(const SPoint2 &param, 
+  virtual void secondDer(const SPoint2 &param,
                          SVector3 *dudu, SVector3 *dvdv, SVector3 *dudv) const;
   void setBoundEdges(GModel *gm, std::vector<int> tagEdges);
   void findEdges(std::map<MEdge, std::vector<int>, Less_Edge > &map_edges);
   void writeGEO(FILE *fp);
+  void createGeometry();
+  void gatherMeshes();
+  virtual void mesh (bool verbose);
+  std::vector<discreteDiskFace*> _atlas;
 };
 
 #endif
