@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <sstream>
 #include "GModel.h"
@@ -118,8 +118,13 @@ std::string gmshEdge::getAdditionalInfoString()
 int gmshEdge::minimumMeshSegments () const
 {
   int np;
-  if(geomType() == Line)
+  if(geomType() == Line){
     np = GEdge::minimumMeshSegments();
+    // FIXME FOR QUADS
+    if(List_Nbr(c->Control_Points) > 2){
+      np = 3*(List_Nbr(c->Control_Points))+1;
+    }
+  }
   else if(geomType() == Circle || geomType() == Ellipse)
     np = (int)(fabs(c->Circle.t1 - c->Circle.t2) *
                  (double)CTX::instance()->mesh.minCircPoints / M_PI) - 1;
