@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <stdlib.h>
 #include <stack>
@@ -111,11 +111,12 @@ void GetStatistics(double stat[50], double quality[3][100])
     stat[10] += (*it)->hexahedra.size();
     stat[11] += (*it)->prisms.size();
     stat[12] += (*it)->pyramids.size();
+    stat[13] += (*it)->trihedra.size();
   }
 
-  stat[13] = CTX::instance()->meshTimer[0];
-  stat[14] = CTX::instance()->meshTimer[1];
-  stat[15] = CTX::instance()->meshTimer[2];
+  stat[14] = CTX::instance()->meshTimer[0];
+  stat[15] = CTX::instance()->meshTimer[1];
+  stat[16] = CTX::instance()->meshTimer[2];
 
   if(quality){
     for(int i = 0; i < 3; i++)
@@ -125,7 +126,7 @@ void GetStatistics(double stat[50], double quality[3][100])
     double gamma = 0., gammaMin = 1., gammaMax = 0.;
     double rho = 0., rhoMin = 1., rhoMax = 0.;
 
-    double N = stat[9] + stat[10] + stat[11] + stat[12];
+    double N = stat[9] + stat[10] + stat[11] + stat[12] + stat[13];
     if(N){ // if we have 3D elements
       for(GModel::riter it = m->firstRegion(); it != m->lastRegion(); ++it){
         GetQualityMeasure((*it)->tetrahedra, gamma, gammaMin, gammaMax,
@@ -148,25 +149,25 @@ void GetStatistics(double stat[50], double quality[3][100])
       }
     }
     if(N){
-      stat[17] = minSICN / N; stat[18] = minSICNMin; stat[19] = minSICNMax;
-      stat[20] = gamma / N;   stat[21] = gammaMin;   stat[22] = gammaMax;
-      stat[23] = rho / N;     stat[24] = rhoMin;     stat[25] = rhoMax;
+      stat[18] = minSICN / N; stat[19] = minSICNMin; stat[20] = minSICNMax;
+      stat[21] = gamma / N;   stat[22] = gammaMin;   stat[23] = gammaMax;
+      stat[25] = rho / N;     stat[25] = rhoMin;     stat[26] = rhoMax;
     }
   }
 
 #if defined(HAVE_POST)
-  stat[26] = PView::list.size();
+  stat[27] = PView::list.size();
   for(unsigned int i = 0; i < PView::list.size(); i++) {
     PViewData *data = PView::list[i]->getData(true);
-    stat[27] += data->getNumPoints();
-    stat[28] += data->getNumLines();
-    stat[29] += data->getNumTriangles();
-    stat[30] += data->getNumQuadrangles();
-    stat[31] += data->getNumTetrahedra();
-    stat[32] += data->getNumHexahedra();
-    stat[33] += data->getNumPrisms();
-    stat[34] += data->getNumPyramids();
-    stat[35] += data->getNumStrings2D() + data->getNumStrings3D();
+    stat[28] += data->getNumPoints();
+    stat[29] += data->getNumLines();
+    stat[30] += data->getNumTriangles();
+    stat[31] += data->getNumQuadrangles();
+    stat[32] += data->getNumTetrahedra();
+    stat[33] += data->getNumHexahedra();
+    stat[34] += data->getNumPrisms();
+    stat[35] += data->getNumPyramids();
+    stat[36] += data->getNumStrings2D() + data->getNumStrings3D();
   }
 #endif
 }
@@ -221,8 +222,9 @@ static void Mesh0D(GModel *m)
     GVertex *gv = *it;
     if (gv->meshMaster() != gv){
       if (gv->correspondingVertices.empty()){
-        GVertex *master = dynamic_cast<GVertex*> (gv->meshMaster());
-        if(master)gv->correspondingVertices[gv->mesh_vertices[0]] = (master->mesh_vertices[0]);
+        GVertex *master = dynamic_cast<GVertex*>(gv->meshMaster());
+        if(master)
+          gv->correspondingVertices[gv->mesh_vertices[0]] = master->mesh_vertices[0];
       }
     }
   }
@@ -241,11 +243,11 @@ static void Mesh1D(GModel *m)
 
   int nIter = 0, nTot = m->getNumEdges();
   while(1){
-    meshGEdge mesher;
+    //    meshGEdge mesher;
     int nPending = 0;
     for(GModel::eiter it = m->firstEdge(); it != m->lastEdge(); ++it){
       if ((*it)->meshStatistics.status == GEdge::PENDING){
-        mesher(*it);
+        (*it)->mesh(true);
         nPending++;
       }
       if(!nIter) Msg::ProgressMeter(nPending, nTot, false, "Meshing 1D...");
@@ -294,7 +296,8 @@ static void PrintMesh2dStatistics(GModel *m)
     if((*it)->geomType() != GEntity::DiscreteSurface){
       worst = std::min((*it)->meshStatistics.worst_element_shape, worst);
       best = std::max((*it)->meshStatistics.best_element_shape, best);
-      avg += (*it)->meshStatistics.average_element_shape * (*it)->meshStatistics.nbTriangle;
+      avg += (*it)->meshStatistics.average_element_shape *
+        (*it)->meshStatistics.nbTriangle;
       e_avg += (*it)->meshStatistics.efficiency_index;
       e_long = std::max((*it)->meshStatistics.longest_edge_length, e_long);
       e_short = std::min((*it)->meshStatistics.smallest_edge_length, e_short);
@@ -308,9 +311,11 @@ static void PrintMesh2dStatistics(GModel *m)
     }
   }
 
-  Msg::Info("*** Efficiency index for surface mesh tau=%g ", 100*exp(e_avg/(double)nTotE));
+  Msg::Info("*** Efficiency index for surface mesh tau=%g ",
+            100*exp(e_avg/(double)nTotE));
 
-  fprintf(statreport,"\t%16s\t%d\t\t%d\t\t", m->getName().c_str(), numFaces, nUnmeshed);
+  fprintf(statreport,"\t%16s\t%d\t\t%d\t\t", m->getName().c_str(), numFaces,
+          nUnmeshed);
   fprintf(statreport,"%d\t\t%8.7f\t%8.7f\t%8.7f\t%d\t\t%8.7f\t",
           nTotT, avg / (double)nTotT, best, worst, nTotGoodQuality,
           (double)nTotGoodQuality / nTotT);
@@ -329,9 +334,9 @@ static void Mesh2D(GModel *m)
   for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it)
     (*it)->meshStatistics.status = GFace::PENDING;
 
-  // boundary layers are special: their generation (including vertices
-  // and curve meshes) is global as it depends on a smooth normal
-  // field generated from the surface mesh of the source surfaces
+  // boundary layers are special: their generation (including vertices and curve
+  // meshes) is global as it depends on a smooth normal field generated from the
+  // surface mesh of the source surfaces
   if(!Mesh2DWithBoundaryLayers(m)){
     std::set<GFace*, GEntityLessThan> cf, f;
     for(GModel::fiter it = m->firstFace(); it != m->lastFace(); ++it)
@@ -353,8 +358,8 @@ static void Mesh2D(GModel *m)
       for(size_t K = 0 ; K < temp.size() ; K++){
         if (temp[K]->meshStatistics.status == GFace::PENDING){
           backgroundMesh::current()->unset();
-          meshGFace mesher(true);
-          mesher(temp[K]);
+	  // meshGFace mesher(true);
+          temp[K]->mesh(true);
 #if defined(HAVE_BFGS)
           if(CTX::instance()->mesh.optimizeLloyd){
             if (temp[K]->geomType()==GEntity::CompoundSurface ||
@@ -391,8 +396,8 @@ static void Mesh2D(GModel *m)
           it != cf.end(); ++it){
         if ((*it)->meshStatistics.status == GFace::PENDING){
           backgroundMesh::current()->unset();
-          meshGFace mesher(true);
-          mesher(*it);
+	  //          meshGFace mesher(true);
+          (*it)->mesh(true);
 #if defined(HAVE_BFGS)
           if(CTX::instance()->mesh.optimizeLloyd){
             if ((*it)->geomType()==GEntity::CompoundSurface ||
@@ -466,6 +471,250 @@ static void FindConnectedRegions(std::vector<GRegion*> &delaunay,
             nbVolumes,connected.size());
 }
 
+template <class ITERATOR>
+void fillv_(std::multimap<MVertex*, MElement*> &vertexToElement,
+	    ITERATOR it_beg, ITERATOR it_end)
+{
+  for (ITERATOR IT = it_beg; IT != it_end ; ++IT){
+    MElement *el = *IT;
+    for(int j = 0; j < el->getNumVertices(); j++) {
+      MVertex* e = el->getVertex(j);
+      vertexToElement.insert(std::make_pair(e, el));
+    }
+  }
+}
+
+int LaplaceSmoothing (GRegion *gr)
+{
+  std::multimap<MVertex*, MElement*> vertexToElement;
+  fillv_(vertexToElement, (gr)->tetrahedra.begin(), (gr)->tetrahedra.end());
+  fillv_(vertexToElement, (gr)->hexahedra.begin(),  (gr)->hexahedra.end());
+  fillv_(vertexToElement, (gr)->prisms.begin(),     (gr)->prisms.end());
+  fillv_(vertexToElement, (gr)->pyramids.begin(),   (gr)->pyramids.end());
+  int N=0;
+  for (unsigned int i=0; i<gr->mesh_vertices.size();i++){
+    MVertex *v = gr->mesh_vertices[i];
+    std::multimap<MVertex*, MElement*>::iterator it = vertexToElement.lower_bound(v);
+    std::multimap<MVertex*, MElement*>::iterator it_low = it;
+    std::multimap<MVertex*, MElement*>::iterator it_up  = vertexToElement.upper_bound(v);
+    double minQual = 1.e22;
+    double volTot = 0.0;
+    double xold=v->x(), yold=v->y(), zold=v->z();
+    SPoint3 pNew (0,0,0);
+    for (; it != it_up; ++it) {
+      minQual= std::min(minQual,it->second->minSICNShapeMeasure());
+      double vol = fabs(it->second->getVolume());
+      SPoint3 cog = it->second->barycenter();
+      pNew += cog * vol;
+      volTot += vol;
+    }
+    pNew *= (1./volTot);
+    v->setXYZ (pNew.x(),pNew.y(),pNew.z());
+    double minQual2 = 1.e22;
+    for (it = it_low; it != it_up; ++it) {
+      minQual2 = std::min(minQual2,it->second->minSICNShapeMeasure());
+      if (minQual2 < minQual){
+	v->setXYZ (xold,yold,zold);
+	break;
+      }
+    }
+    if (minQual < minQual2) N++;
+  }
+  return N;
+}
+
+// JFR : use hex-splitting to resolve non conformity
+//     : if howto == 1 ---> split hexes
+//     : if howto == 2 ---> create transition elements
+
+/*
+ v3        v2
+  x--------x
+  |\       |
+  |  \     |
+  |    \   |
+  |      \ |
+  x--------x
+  v0       v1
+ */
+
+void buildUniqueFaces (GRegion *gr, std::set<MFace,Less_Face> &bnd)
+{
+  for (unsigned int i=0;i<gr->getNumMeshElements();i++){
+    MElement *e = gr->getMeshElement(i);
+    for(int j=0;j<e->getNumFaces();j++){
+      MFace f = e->getFace(j);
+      std::set<MFace,Less_Face>::iterator it = bnd.find(f);
+      if (it == bnd.end())bnd.insert(f);
+      else bnd.erase(it);
+    }
+  }
+}
+
+bool MakeMeshConformal(GModel *gm, int howto)
+{
+  fs_cont search;
+  buildFaceSearchStructure(gm, search);
+  std::set<MFace,Less_Face> bnd;
+  for (GModel::riter rit = gm->firstRegion(); rit != gm->lastRegion(); ++rit){
+    GRegion *gr = *rit;
+    buildUniqueFaces (gr,bnd);
+  }
+  // bnd2 contains non conforming faces
+
+   std::set<MFace,Less_Face> bnd2;
+  for (std::set<MFace,Less_Face>::iterator itf = bnd.begin(); itf != bnd.end(); ++itf){
+    GFace *gfound = findInFaceSearchStructure (*itf,search);
+    if (!gfound){
+      bnd2.insert(*itf);
+    }
+  }
+  bnd.clear();
+
+  Msg::Info("%d hanging faces",bnd2.size());
+
+  std::set<MFace,Less_Face> ncf;
+  for (std::set<MFace,Less_Face>::iterator itf = bnd2.begin(); itf != bnd2.end(); ++itf){
+    const MFace &f = *itf;
+    if (f.getNumVertices() == 4){ // quad face
+      std::set<MFace,Less_Face>::iterator it1 =
+        bnd2.find (MFace(f.getVertex(0),f.getVertex(1),f.getVertex(2)));
+      std::set<MFace,Less_Face>::iterator it2 =
+        bnd2.find (MFace(f.getVertex(2),f.getVertex(3),f.getVertex(0)));
+      if (it1 != bnd2.end() && it2 != bnd2.end()){
+	ncf.insert(MFace(f.getVertex(1), f.getVertex(2),
+                         f.getVertex(3),f.getVertex(0)));
+      }
+      else {
+	it1 = bnd2.find (MFace(f.getVertex(0),f.getVertex(1),f.getVertex(3)));
+	it2 = bnd2.find (MFace(f.getVertex(3),f.getVertex(1),f.getVertex(2)));
+	if (it1 != bnd2.end() && it2 != bnd2.end()){
+	  ncf.insert(MFace(f.getVertex(0), f.getVertex(1),
+                           f.getVertex(2), f.getVertex(3)));
+	}
+	else {
+	  Msg::Error("MakeMeshConformal: wrong mesh topology");
+	  return false;
+	}
+      }
+    }
+  }
+  bnd2.clear();
+
+  for (GModel::riter rit = gm->firstRegion(); rit != gm->lastRegion(); ++rit){
+    GRegion *gr = *rit;
+    std::vector<MHexahedron*> remainingHexes;
+    for (unsigned int i=0;i<gr->hexahedra.size();i++){
+      MHexahedron *e = gr->hexahedra[i];
+      std::vector<MFace> faces;
+      for(int j=0;j<e->getNumFaces();j++){
+	MFace f = e->getFace(j);
+	std::set<MFace,Less_Face>::iterator it = ncf.find(f);
+	if (it == ncf.end()){
+	  faces.push_back(f);
+	}
+	else {
+	  faces.push_back(MFace(it->getVertex(0),it->getVertex(1),it->getVertex(3)));
+	  faces.push_back(MFace(it->getVertex(1),it->getVertex(2),it->getVertex(3)));
+	}
+      }
+      // HEX IS ONLY SURROUNED BY COMPATIBLE ELEMENTS
+      if (faces.size() == e->getNumFaces()){
+	remainingHexes.push_back(e);
+      }
+      else {
+	SPoint3 pp = e->barycenter();
+	MVertex *newv = new MVertex (pp.x(),pp.y(),pp.z(),gr);
+	gr->mesh_vertices.push_back(newv);
+	for (unsigned int j=0;j<faces.size();j++){
+	  MFace &f = faces[j];
+	  if (f.getNumVertices() == 4){
+	    gr->pyramids.push_back(new MPyramid(f.getVertex(0), f.getVertex(1),
+                                                f.getVertex(2), f.getVertex(3), newv));
+	  }
+	  else {
+	    gr->tetrahedra.push_back(new MTetrahedron(f.getVertex(0), f.getVertex(1),
+                                                      f.getVertex(2), newv));
+	  }
+	}
+      }
+    }
+    gr->hexahedra = remainingHexes;
+    remainingHexes.clear();
+    std::vector<MPrism*> remainingPrisms;
+    for (unsigned int i=0;i<gr->prisms.size();i++){
+      MPrism *e = gr->prisms[i];
+      std::vector<MFace> faces;
+      for(int j=0;j<e->getNumFaces();j++){
+	MFace f = e->getFace(j);
+	std::set<MFace,Less_Face>::iterator it = ncf.find(f);
+	if (it == ncf.end()){
+	  faces.push_back(f);
+	}
+	else {
+	  faces.push_back(MFace(it->getVertex(0),it->getVertex(1),it->getVertex(3)));
+	  faces.push_back(MFace(it->getVertex(1),it->getVertex(2),it->getVertex(3)));
+	}
+      }
+      // HEX IS ONLY SURROUNED BY COMPATIBLE ELEMENTS
+      if (faces.size() == e->getNumFaces()){
+	remainingPrisms.push_back(e);
+      }
+      else {
+	SPoint3 pp = e->barycenter();
+	MVertex *newv = new MVertex (pp.x(),pp.y(),pp.z(),gr);
+	gr->mesh_vertices.push_back(newv);
+	for (unsigned int j=0;j<faces.size();j++){
+	  MFace &f = faces[j];
+	  if (f.getNumVertices() == 4){
+	    gr->pyramids.push_back(new MPyramid(f.getVertex(0), f.getVertex(1),
+                                                f.getVertex(2), f.getVertex(3), newv));
+	  }
+	  else {
+	    gr->tetrahedra.push_back(new MTetrahedron(f.getVertex(0), f.getVertex(1),
+                                                      f.getVertex(2), newv));
+	  }
+	}
+      }
+    }
+    gr->prisms = remainingPrisms;
+  }
+
+  return true;
+}
+
+void TestConformity(GModel *gm)
+{
+  fs_cont search;
+  buildFaceSearchStructure(gm, search);
+  int count = 0;
+  for (GModel::riter rit = gm->firstRegion(); rit != gm->lastRegion(); ++rit){
+    GRegion *gr = *rit;
+    std::set<MFace,Less_Face> bnd;
+    double vol = 0.0;
+    for (unsigned int i=0;i<gr->getNumMeshElements();i++){
+      MElement *e = gr->getMeshElement(i);
+      vol += fabs(e->getVolume());
+      for(int j=0;j<e->getNumFaces();j++){
+	MFace f = e->getFace(j);
+	std::set<MFace,Less_Face>::iterator it = bnd.find(f);
+	if (it == bnd.end())bnd.insert(f);
+	else bnd.erase(it);
+      }
+    }
+    printf("vol(%d) = %12.5E\n",gr->tag(),vol);
+
+    for (std::set<MFace,Less_Face>::iterator itf = bnd.begin(); itf != bnd.end(); ++itf){
+      GFace *gfound = findInFaceSearchStructure (*itf,search);
+      if (!gfound){
+	count ++;
+      }
+    }
+  }
+  if (!count)Msg::Info("Mesh Conformity: OK");
+  else Msg::Error ("Mesh is not conforming (%d hanging faces)!",count);
+}
+
 static void Mesh3D(GModel *m)
 {
   if(TooManyElements(m, 3)) return;
@@ -530,23 +779,36 @@ static void Mesh3D(GModel *m)
       }
       if(treat_region_ok && (CTX::instance()->mesh.recombine3DAll ||
                              gr->meshAttributes.recombine3D)){
+	if (CTX::instance()->mesh.optimize){
+	  optimizeMeshGRegionGmsh opt;
+	  opt(gr);
+	}
         double a = Cpu();
-        Recombinator rec;
-        rec.execute(gr);
-        Supplementary sup;
-        sup.execute(gr);
+        if (CTX::instance()->mesh.recombine3DLevel >= 0){
+          Recombinator rec;
+          rec.execute(gr);
+        }
+        if (CTX::instance()->mesh.recombine3DLevel >= 1){
+          Supplementary sup;
+          sup.execute(gr);
+        }
         PostOp post;
-        post.execute(gr,0);
+        post.execute(gr,CTX::instance()->mesh.recombine3DLevel,
+                     CTX::instance()->mesh.recombine3DConformity);
+        // 0: no pyramid, 1: single-step, 2: two-steps (conforming),
+        // true: fill non-conformities with trihedra
+	// while(LaplaceSmoothing (gr)){
+	// }
         nb_elements_recombination += post.get_nb_elements();
         nb_hexa_recombination += post.get_nb_hexahedra();
         vol_element_recombination += post.get_vol_elements();
         vol_hexa_recombination += post.get_vol_hexahedra();
         // partial export
-        stringstream ss;
-        ss << "yamakawa_part_";
-        ss << gr->tag();
-        ss << ".msh";
-        export_gregion_mesh(gr, ss.str().c_str());
+	//        stringstream ss;
+	//        ss << "yamakawa_part_";
+	//        ss << gr->tag();
+	//        ss << ".msh";
+	//        export_gregion_mesh(gr, ss.str().c_str());
         time_recombination += (Cpu() - a);
       }
     }
@@ -560,6 +822,8 @@ static void Mesh3D(GModel *m)
               nb_hexa_recombination*100./nb_elements_recombination);
     Msg::Info(".... Percentage of hexahedra (Vol) : %g",
               vol_hexa_recombination*100./vol_element_recombination);
+    //    MakeMeshConformal (m, 1);
+    TestConformity(m);
   }
 
   // Ensure that all volume Jacobians are positive
