@@ -1,10 +1,11 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include "GmshConfig.h"
 #include "GModel.h"
+#include "discreteEdge.h"
 #include "meshGEdge.h"
 #include "GEdge.h"
 #include "MLine.h"
@@ -74,6 +75,13 @@ static double smoothPrimitive(GEdge *ge, double alpha,
   return Points[Points.size() - 1].p;
 }
 
+
+static double F_LcB(GEdge *ge, double t)
+{
+  GPoint p = ge->point(t);
+  return BGM_MeshSize(ge, t, 0, p.x(), p.y(), p.z());
+}
+
 static double F_Lc(GEdge *ge, double t)
 {
   GPoint p = ge->point(t);
@@ -268,14 +276,13 @@ static double Integration(GEdge *ge, double t1, double t2,
 }
 
 void copyMesh(GEdge *from, GEdge *to, int direction)
-{  
+{
   Range<double> u_bounds = from->parBounds(0);
   double u_min = u_bounds.low();
   double u_max = u_bounds.high();
 
   Range<double> to_u_bounds = to->parBounds(0);
   double to_u_min = to_u_bounds.low();
-  double to_u_max = to_u_bounds.high();
 
   for(unsigned int i = 0; i < from->mesh_vertices.size(); i++){
     int index = (direction < 0) ? (from->mesh_vertices.size() - 1 - i) : i;
@@ -298,7 +305,8 @@ void copyMesh(GEdge *from, GEdge *to, int direction)
 
 void deMeshGEdge::operator() (GEdge *ge)
 {
-  if(ge->geomType() == GEntity::DiscreteCurve) return;
+  if(ge->geomType() == GEntity::DiscreteCurve && !CTX::instance()->meshDiscrete)
+    return;
   ge->deleteMesh();
   ge->meshStatistics.status = GEdge::PENDING;
   ge->correspondingVertices.clear();
@@ -328,6 +336,65 @@ static int increaseN (int N)
   return N;
 }
 
+// ensure not to have points that are too close to each other.
+// can be caused by a coarse 1D mesh or by a noisy curve
+static void filterPoints (GEdge*ge) {
+  if (ge->mesh_vertices.empty())return;
+  if(ge->meshAttributes.method == MESH_TRANSFINITE)return;
+  //if (ge->mesh_vertices.size() <=3)return;
+  bool forceOdd = false;
+  if((ge->meshAttributes.method != MESH_TRANSFINITE ||
+      CTX::instance()->mesh.flexibleTransfinite) &&
+     CTX::instance()->mesh.algoRecombine != 0){
+    if(CTX::instance()->mesh.recombineAll){
+      forceOdd = true;
+    }
+  }
+
+  MVertex *v0 = ge->getBeginVertex()->mesh_vertices[0];
+  std::vector<std::pair<double, MVertex*> > lengths;
+  for (unsigned int i=0;i<ge->mesh_vertices.size();i++){
+    MEdgeVertex *v = dynamic_cast<MEdgeVertex*> (ge->mesh_vertices[i]);
+    if (!v)Msg::Fatal("in 1D mesh");
+    double d = distance (v,v0);
+    double t;
+    v->getParameter(0,t);
+    if (i != 0){
+      double t0;
+      v0->getParameter(0,t0);
+      t=0.5*(t+t0);
+    }
+    double lc = F_LcB(ge, t);
+    //    double lc = v->getLc();
+    if (d < lc * .3) {
+      lengths.push_back(std::make_pair(lc/d,v));
+    }
+    else
+      v0=v;
+  }
+  std::sort(lengths.begin(),lengths.end());
+  int last = lengths.size();
+  if (forceOdd) {
+    while (last %2 != 0)last--;
+  }
+  /*
+	if (CTX::instance()->mesh.algoRecombine == 2){
+	if (last < 4)last = 0;
+	while (last %4 != 0)last--;
+	}
+	else {
+	while (last %2 != 0)last--;
+	}
+	}
+  */
+
+  for (int i=0;i<last;i++){
+    std::vector<MVertex*>::iterator it = std::find(ge->mesh_vertices.begin(),ge->mesh_vertices.end(),lengths[i].second);
+    ge->mesh_vertices.erase(it);
+    delete lengths[i].second;
+  }
+}
+
 void meshGEdge::operator() (GEdge *ge)
 {
 #if defined(HAVE_ANN)
@@ -341,7 +408,7 @@ void meshGEdge::operator() (GEdge *ge)
 
   ge->model()->setCurrentMeshEntity(ge);
 
-  if(ge->geomType() == GEntity::DiscreteCurve) return;
+  //  if(ge->geomType() == GEntity::DiscreteCurve) return;
   if(ge->geomType() == GEntity::BoundaryLayerCurve) return;
   if(ge->meshAttributes.method == MESH_NONE) return;
   if(CTX::instance()->mesh.meshOnlyVisible && !ge->getVisibility()) return;
@@ -490,6 +557,7 @@ void meshGEdge::operator() (GEdge *ge)
         const double d = norm(der);
         double lc  = d/(P1.lc + dlc / dp * (d - P1.p));
         GPoint V = ge->point(t);
+	//	printf("%d %g\n",NUMP-1,t);
         mesh_vertices[NUMP - 1] = new MEdgeVertex(V.x(), V.y(), V.z(), ge, t, lc);
         NUMP++;
       }
@@ -500,6 +568,10 @@ void meshGEdge::operator() (GEdge *ge)
     mesh_vertices.resize(NUMP - 1);
   }
 
+  //  printf("%ld ----> ", ge->mesh_vertices.size());
+  filterPoints (ge);
+  //  printf("%ld \n", ge->mesh_vertices.size());
+
   for(unsigned int i = 0; i < mesh_vertices.size() + 1; i++){
     MVertex *v0 = (i == 0) ?
       ge->getBeginVertex()->mesh_vertices[0] : mesh_vertices[i - 1];
