@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <sstream>
 #include <stdlib.h>
@@ -18,6 +18,7 @@
 #include "GEdge.h"
 #include "GEdgeCompound.h"
 #include "robustPredicates.h"
+#include "discreteFace.h"
 #include "GFace.h"
 #include "GModel.h"
 #include "MVertex.h"
@@ -97,6 +98,9 @@ public:
 	    MVertex *v1 = (*ite)->lines[i]->getVertex(0);
 	    MVertex *v2 = (*ite)->lines[i]->getVertex(1);
 	    MVertex *v3 = (*ite)->lines[i+1]->getVertex(1);
+	    v2->x() = 0.5*(v1->x()+v3->x());
+	    v2->y() = 0.5*(v1->y()+v3->y());
+	    v2->z() = 0.5*(v1->z()+v3->z());
 	    temp.push_back(new MLine(v1,v3));
 	    if (v1->onWhat() == *ite) (*ite)->mesh_vertices.push_back(v1);
 	    _middle[MEdge(v1,v3)] = v2;
@@ -107,8 +111,8 @@ public:
 	}
 	++ite;
       }
+      CTX::instance()->mesh.lcFactor *=2.0;
     }
-    backgroundMesh::setSizeFactor(2.0);
   }
   void subdivide ()
   {
@@ -128,7 +132,10 @@ public:
 	m[j] = p1;
 	if (it == _middle.end() && it2 == eds.end()){
 	  GPoint gp = _gf->point((p1+p2)*0.5);
-	  v[j] = new MFaceVertex (gp.x(),gp.y(),gp.z(),_gf,gp.u(),gp.v());
+	  double XX = 0.5*(E.getVertex(0)->x()+E.getVertex(1)->x());
+	  double YY = 0.5*(E.getVertex(0)->y()+E.getVertex(1)->y());
+	  double ZZ = 0.5*(E.getVertex(0)->z()+E.getVertex(1)->z());
+	  v[j] = new MFaceVertex (XX,YY,ZZ,_gf,gp.u(),gp.v());
 	  _gf->mesh_vertices.push_back(v[j]);
 	  eds[E] = v[j];
 	}
@@ -141,7 +148,10 @@ public:
 	}
       }
       GPoint gp    = _gf->point((m[0]+m[1]+m[2])*(1./3.));
-      MFaceVertex *vmid = new MFaceVertex (gp.x(),gp.y(),gp.z(),_gf,gp.u(),gp.v());
+      double XX = (v[0]->x()+v[1]->x()+v[2]->x())*(1./3.);
+      double YY = (v[0]->y()+v[1]->y()+v[2]->y())*(1./3.);
+      double ZZ = (v[0]->z()+v[1]->z()+v[2]->z())*(1./3.);
+      MFaceVertex *vmid = new MFaceVertex (XX,YY,ZZ,_gf,gp.u(),gp.v());
       _gf->mesh_vertices.push_back(vmid);
       qnew.push_back(new MQuadrangle(_gf->triangles[i]->getVertex(0),v[0],vmid,v[2]));
       qnew.push_back(new MQuadrangle(_gf->triangles[i]->getVertex(1),v[1],vmid,v[0]));
@@ -161,7 +171,10 @@ public:
 	m[j] = p1;
 	if (it == _middle.end() && it2 == eds.end()){
 	  GPoint gp = _gf->point((p1+p2)*0.5);
-	  v[j] = new MFaceVertex (gp.x(),gp.y(),gp.z(),_gf,gp.u(),gp.v());
+	  double XX = 0.5*(E.getVertex(0)->x()+E.getVertex(1)->x());
+	  double YY = 0.5*(E.getVertex(0)->y()+E.getVertex(1)->y());
+	  double ZZ = 0.5*(E.getVertex(0)->z()+E.getVertex(1)->z());
+	  v[j] = new MFaceVertex (XX,YY,ZZ,_gf,gp.u(),gp.v());
 	  _gf->mesh_vertices.push_back(v[j]);
 	  eds[E] = v[j];
 	}
@@ -174,7 +187,11 @@ public:
 	}
       }
       GPoint gp    = _gf->point((m[0]+m[1]+m[2]+m[3])*0.25);
-      MVertex *vmid = new MFaceVertex (gp.x(),gp.y(),gp.z(),_gf,gp.u(),gp.v());
+      // FIXME : NOT EXACTLY CORRECT, BUT THAT'S THE PLACE WE WANT THE POINT TO RESIDE
+      double XX = 0.25*(v[0]->x()+v[1]->x()+v[2]->x()+v[3]->x());
+      double YY = 0.25*(v[0]->y()+v[1]->y()+v[2]->y()+v[3]->y());
+      double ZZ = 0.25*(v[0]->z()+v[1]->z()+v[2]->z()+v[3]->z());
+      MVertex *vmid = new MFaceVertex (XX,YY,ZZ,_gf,gp.u(),gp.v());
       _gf->mesh_vertices.push_back(vmid);
       qnew.push_back(new MQuadrangle(_gf->quadrangles[i]->getVertex(0),v[0],vmid,v[3]));
       qnew.push_back(new MQuadrangle(_gf->quadrangles[i]->getVertex(1),v[1],vmid,v[0]));
@@ -187,17 +204,16 @@ public:
   }
   void finish ()
   {
-    backgroundMesh::setSizeFactor(1.0);
     if((CTX::instance()->mesh.recombineAll || _gf->meshAttributes.recombine) &&
        CTX::instance()->mesh.algoRecombine == 2){
       // recombine the elements on the half mesh
-      recombineIntoQuads(_gf,true,true,.1);
-      Msg::Info("subdividing");
-      //      _gf->model()->writeMSH("hop1.msh");
+      CTX::instance()->mesh.lcFactor /=2.0;
+      recombineIntoQuads(_gf,true,true,.1,true);
+      //      Msg::Info("subdividing");
       subdivide();
       //      _gf->model()->writeMSH("hop2.msh");
       restore();
-      recombineIntoQuads(_gf,true,true,0.1);
+      recombineIntoQuads(_gf,true,true,1.e-3,false);
       computeElementShapes(_gf,
 			   _gf->meshStatistics.worst_element_shape,
 			   _gf->meshStatistics.average_element_shape,
@@ -496,6 +512,7 @@ static bool algoDelaunay2D(GFace *gf)
      gf->getMeshingAlgo() == ALGO_2D_FRONTAL ||
      gf->getMeshingAlgo() == ALGO_2D_FRONTAL_QUAD ||
      gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS ||
+     gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS_CSTR ||
      gf->getMeshingAlgo() == ALGO_2D_BAMG)
     return true;
 
@@ -985,10 +1002,11 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
       }
     }
     else
-      Msg::Info("Degenerated mesh on edge %d", (*ite)->tag());
+      Msg::Debug("Degenerated mesh on edge %d", (*ite)->tag());
     ++ite;
   }
 
+
   if(boundary.size()){
     Msg::Error("The 1D mesh seems not to be forming a closed loop");
     gf->meshStatistics.status = GFace::FAILED;
@@ -1071,8 +1089,6 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
     count++;
   }
 
-  // here check if some boundary layer nodes should be added
-
   bbox.makeCube();
 
   // use a divide & conquer type algorithm to create a triangulation.
@@ -1117,6 +1133,8 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
       doc.points[points.size() + ip].data = pp;
     }
 
+
+
     // Use "fast" inhouse recursive algo to generate the triangulation.
     // At this stage the triangulation is not what we need
     //   -) It does not necessary recover the boundaries
@@ -1131,6 +1149,7 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
     }
     Msg::Debug("Meshing of the convex hull (%d points) done", points.size());
 
+
     for(int i = 0; i < doc.numTriangles; i++) {
       int a = doc.triangles[i].a;
       int b = doc.triangles[i].b;
@@ -1498,7 +1517,8 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
 
   bool infty = false;
   if (gf->getMeshingAlgo() == ALGO_2D_FRONTAL_QUAD ||
-      gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS)
+      gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS ||
+      gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS_CSTR)
     infty = true;
   if (!onlyInitialMesh) {
     if (infty)
@@ -1519,9 +1539,13 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
     else if(gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS){
       bowyerWatsonParallelograms(gf);
     }
+    else if(gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS_CSTR){
+      bowyerWatsonParallelogramsConstrained(gf,gf->constr_vertices);
+    }
     else if(gf->getMeshingAlgo() == ALGO_2D_DELAUNAY ||
-            gf->getMeshingAlgo() == ALGO_2D_AUTO)
+            gf->getMeshingAlgo() == ALGO_2D_AUTO){
       bowyerWatson(gf);
+    }
     else {
       bowyerWatson(gf,15000);
       meshGFaceBamg(gf);
@@ -1537,21 +1561,7 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
     sprintf(name, "param%d.pos", gf->tag());
     outputScalarField(m->triangles, name,1);
   }
-  if(CTX::instance()->mesh.remove4triangles)
-    removeFourTrianglesNodes(gf,false);
 
-  //Emi print efficiency index
-  /*
-  gf->computeMeshSizeFieldAccuracy(gf->meshStatistics.efficiency_index,
-  				   gf->meshStatistics.longest_edge_length,
-  				   gf->meshStatistics.smallest_edge_length,
-  				   gf->meshStatistics.nbEdge,
-  				   gf->meshStatistics.nbGoodLength);
-  */
-  //printf("----- Efficiency index is tau=%g\n", gf->meshStatistics.efficiency_index);
-
-
-  // delete the mesh
   delete m;
 
   if((CTX::instance()->mesh.recombineAll || gf->meshAttributes.recombine) &&
@@ -2313,7 +2323,8 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
 
   bool infty = false;
   if (gf->getMeshingAlgo() == ALGO_2D_FRONTAL_QUAD ||
-      gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS)
+      gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS ||
+      gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS_CSTR)
     infty = true;
   if (infty)
     buildBackGroundMesh (gf, &equivalence, &parametricCoordinates);
@@ -2328,6 +2339,8 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
       bowyerWatsonFrontalLayers(gf,true, &equivalence, &parametricCoordinates);
     else if(gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS)
       bowyerWatsonParallelograms(gf,&equivalence, &parametricCoordinates);
+    else if(gf->getMeshingAlgo() == ALGO_2D_PACK_PRLGRMS_CSTR)
+      bowyerWatsonParallelogramsConstrained(gf,gf->constr_vertices,&equivalence, &parametricCoordinates);
     else if(gf->getMeshingAlgo() == ALGO_2D_DELAUNAY ||
             gf->getMeshingAlgo() == ALGO_2D_AUTO)
       bowyerWatson(gf,1000000000, &equivalence, &parametricCoordinates);
@@ -2342,7 +2355,7 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
 
   if((CTX::instance()->mesh.recombineAll || gf->meshAttributes.recombine) &&
      !CTX::instance()->mesh.optimizeLloyd && CTX::instance()->mesh.algoRecombine != 2)
-    recombineIntoQuads(gf);
+    recombineIntoQuads(gf,true,false);
 
   computeElementShapes(gf, gf->meshStatistics.worst_element_shape,
                        gf->meshStatistics.average_element_shape,
@@ -2355,7 +2368,7 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
 
 void deMeshGFace::operator() (GFace *gf)
 {
-  if(gf->geomType() == GEntity::DiscreteSurface) return;
+  if(gf->geomType() == GEntity::DiscreteSurface && !CTX::instance()->meshDiscrete)return;
   gf->deleteMesh();
   gf->meshStatistics.status = GFace::PENDING;
   gf->meshStatistics.nbTriangle = gf->meshStatistics.nbEdge = 0;
@@ -2367,7 +2380,6 @@ int debugSurface = -1; //-1;
 
 void meshGFace::operator() (GFace *gf, bool print)
 {
-
   gf->model()->setCurrentMeshEntity(gf);
 
   if(debugSurface >= 0 && gf->tag() != debugSurface){
@@ -2375,7 +2387,7 @@ void meshGFace::operator() (GFace *gf, bool print)
     return;
   }
 
-  if(gf->geomType() == GEntity::DiscreteSurface) return;
+  //  if(gf->geomType() == GEntity::DiscreteFace) return;
   if(gf->geomType() == GEntity::ProjectionFace) return;
   if(gf->meshAttributes.method == MESH_NONE) return;
   if(CTX::instance()->mesh.meshOnlyVisible && !gf->getVisibility()) return;
