@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include "qualityMeasures.h"
 #include "BDS.h"
@@ -365,7 +365,7 @@ void qmTriangle::NCJAndGradients(const SPoint3 &p0, const SPoint3 &p1, const SPo
 
 
 double qmQuadrangle::eta(MQuadrangle *el) {
-  double AR = 1;//(minEdge()/maxEdge());
+  double AR = 1;//pow(el->minEdge()/el->maxEdge(),.25);
 
   MVertex *_v[4] = {el->getVertex(0), el->getVertex(1), el->getVertex(2), el->getVertex(3)};
 
