@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 //
 // Contributed by Tristan Carrier
 
@@ -609,7 +609,7 @@ public:
 
 class PostOp{
 private:
-  int nbr,nbr8,nbr6,nbr5,nbr4;
+  int nbr,nbr8,nbr6,nbr5,nbr4,nbr4Trih;
   double vol,vol8,vol6,vol5,vol4;
   int estimate1;
   int estimate2;
@@ -619,12 +619,15 @@ private:
   std::map<MVertex*,std::set<MElement*> > vertex_to_pyramids;
   std::multiset<Tuple> tuples;
   std::set<MElement*> triangles;
+
 public:
   PostOp();
   ~PostOp();
 
-  void execute(bool);
-  void execute(GRegion*,bool);
+  void execute(int, int);
+  //level - 0: hex, 1: hex+prisms, 2: hex+prism+pyramids
+  //conformity - 0: nonconforming, 1: trihedra, 2: pyramids+trihedra, 3:pyramids+hexPrismSplit+trihedra, 4:hexPrismSplit+trihedra
+  void execute(GRegion*,int level, int conformity);
 
   inline int get_nb_hexahedra()const{return nbr8;};
   inline double get_vol_hexahedra()const{return vol8;};
@@ -632,17 +635,30 @@ public:
   inline double get_vol_elements()const{return vol;};
 
   void init_markings(GRegion*);
+  void init_markings_hex(GRegion*);
+  void init_markings_pri(GRegion*);
+  void init_markings_pyr(GRegion*);
   void pyramids1(GRegion*);
-  void pyramids2(GRegion*);
+  void pyramids2(GRegion*, bool allowNonConforming=false);
+  void trihedra(GRegion*);
+  void split_hexahedra(GRegion*);
+  void split_prisms(GRegion*);
+  void split_pyramids(GRegion*);  
+  int nonConformDiag(MVertex* a,MVertex* b,MVertex* c,MVertex* d,GRegion* gr);
   void pyramids1(MVertex*,MVertex*,MVertex*,MVertex*,GRegion*);
-  void pyramids2(MVertex*,MVertex*,MVertex*,MVertex*,GRegion*);
+  void pyramids2(MVertex*,MVertex*,MVertex*,MVertex*,GRegion*, bool allowNonConforming);
+  void trihedra(MVertex*,MVertex*,MVertex*,MVertex*,GRegion*);
   void rearrange(GRegion*);
   void statistics(GRegion*);
   void build_tuples(GRegion*);
   void modify_surfaces(GRegion*);
   void modify_surfaces(MVertex*,MVertex*,MVertex*,MVertex*);
 
+  //returns the geometrical validity of the pyramid
+  bool valid(MPyramid *pyr);
+  
   bool four(MElement*);
+  bool fourTrih(MElement*);
   bool five(MElement*);
   bool six(MElement*);
   bool eight(MElement*);
@@ -655,7 +671,10 @@ public:
   double workaround(MElement*);
 
   MVertex* find(MVertex*,MVertex*,MVertex*,MVertex*,MElement*);
+  MVertex* findInTriFace(MVertex* in0,MVertex* in1,MVertex* out0,MVertex* out1,MElement* element);
   void find_tetrahedra(MVertex*,MVertex*,std::set<MElement*>&);
+  void find_tetrahedra(MVertex*,MVertex*,MVertex*,std::set<MElement*>&);
+  void find_pyramids_from_tri(MVertex*,MVertex*,MVertex*,std::set<MElement*>&);
   void find_pyramids(MVertex*,MVertex*,std::set<MElement*>&);
 
   void intersection(const std::set<MElement*>&,const std::set<MElement*>&,std::set<MElement*>&);
