@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include "ElementType.h"
 #include "GmshDefines.h"
@@ -90,9 +90,15 @@ int ElementType::ParentTypeFromTag(int tag)
     case(MSH_PNT_SUB):  case(MSH_LIN_SUB):
     case(MSH_TRI_SUB):  case(MSH_TET_SUB):
       return TYPE_XFEM;
+    case(MSH_TRI_MINI):
+    case(MSH_TET_MINI):
+      return TYPE_MINI;
+    case(MSH_TRIH_4):
+      return TYPE_TRIH;
+
     default:
-      Msg::Error("Unknown element tag %i, assuming tetrahedron.", tag);
-      return TYPE_TET;
+      Msg::Error("Unknown element tag %i for parent type, returning -1.", tag);
+      return -1;
   }
 }
 
@@ -130,6 +136,9 @@ int ElementType::OrderFromTag(int tag)
   case MSH_TRI_24  : return 8;
   case MSH_TRI_27  : return 9;
   case MSH_TRI_30  : return 10;
+  case MSH_TRI_MINI  : return 1;
+  case MSH_POLYG_  : return 1;
+  case MSH_POLYG_B : return 1;
   case MSH_TET_1   : return 0;
   case MSH_TET_4   : return 1;
   case MSH_TET_10  : return 2;
@@ -149,6 +158,7 @@ int ElementType::OrderFromTag(int tag)
   case MSH_TET_46  : return 8;
   case MSH_TET_52  : return 9;
   case MSH_TET_58  : return 10;
+  case MSH_TET_MINI  : return 1;
   case MSH_QUA_1   : return 0;
   case MSH_QUA_4   : return 1;
   case MSH_QUA_9   : return 2;
@@ -223,8 +233,10 @@ int ElementType::OrderFromTag(int tag)
   case MSH_PYR_53 : return 7;
   case MSH_PYR_61 : return 8;
   case MSH_PYR_69 : return 9;
+  case MSH_TRIH_4 : return 1;
+  case MSH_POLYH_ : return 1;
   default :
-    Msg::Error("Unknown element tag %d, assuming order 1.",tag);
+    Msg::Warning("Unknown element tag %d, assuming order 1.",tag);
     return 1;
   }
 }
@@ -254,7 +266,7 @@ int ElementType::DimensionFromTag(int tag)
     case(MSH_TRI_21I):  case(MSH_TRI_24):
     case(MSH_TRI_27):   case(MSH_TRI_30):
     case(MSH_TRI_B):    case(MSH_TRI_1):
-    case(MSH_TRI_SUB):
+    case(MSH_TRI_SUB):  case(MSH_TRI_MINI):
 
     case(MSH_QUA_4):    case(MSH_QUA_9):
     case(MSH_QUA_8):    case(MSH_QUA_16):
@@ -280,6 +292,7 @@ int ElementType::DimensionFromTag(int tag)
     case(MSH_TET_46):   case(MSH_TET_52):
     case(MSH_TET_58):   case(MSH_TET_1):
     case(MSH_TET_16):   case(MSH_TET_SUB):
+    case(MSH_TET_MINI):
 
     case(MSH_PYR_5):    case(MSH_PYR_14):
     case(MSH_PYR_13):   case(MSH_PYR_30):
@@ -311,12 +324,14 @@ int ElementType::DimensionFromTag(int tag)
     case(MSH_HEX_68):   case(MSH_HEX_80):
     case(MSH_HEX_92):   case(MSH_HEX_104):
 
+    case(MSH_TRIH_4):
+      
     case(MSH_POLYH_):
       return 3;
 
     default:
-      Msg::Error("Unknown element tag %i, assuming dimension 3.", tag);
-      return 3;
+      Msg::Error("Unknown element tag %i for dimension, returning -1.", tag);
+      return -1;
   }
 }
 
@@ -347,6 +362,9 @@ int ElementType::SerendipityFromTag(int tag)
 
   case MSH_PYR_1   : case MSH_PYR_5   :
 
+  case MSH_POLYG_  : case MSH_POLYG_B :
+  case MSH_POLYH_  :
+
     return 1; // Serendipity or not
 
 
@@ -381,6 +399,8 @@ int ElementType::SerendipityFromTag(int tag)
   case MSH_PYR_140 : case MSH_PYR_204 :
   case MSH_PYR_285 : case MSH_PYR_385 :
 
+  case MSH_TRIH_4 :
+    
     return 0; // Not Serendipity
 
 
@@ -418,7 +438,7 @@ int ElementType::SerendipityFromTag(int tag)
     return 2; // Only Serendipity
 
   default :
-    Msg::Error("Unknown element tag %d, assuming not serendipity.",tag);
+    Msg::Warning("Unknown element tag %d, assuming not serendipity.",tag);
     return 0;
   }
 }
@@ -537,6 +557,8 @@ int ElementType::getTag(int parentTag, int order, bool serendip)
     default : Msg::Error("pyramid order %i unknown", order); return 0;
     }
     break;
-  default : Msg::Error("unknown element type %i, returning 0", parentTag); return 0;
+  case TYPE_TRIH :
+    return MSH_TRIH_4;
+  default : Msg::Warning("unknown element type %i, returning 0", parentTag); return 0;
   }
 }
