@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <string.h>
 #include <algorithm>
@@ -15,10 +15,14 @@
 #include "GmshMessage.h"
 
 int PView::_globalTag = 0;
+bool PView::_removeInnerBorders = false;
+bool PView::_x3dCompatibility = false;
+double PView::_transparency=0.;
+double PView::_precision=pow(10.,-9.);
 std::vector<PView*> PView::list;
-
 void PView::_init(int tag)
 {
+ _removeInnerBorders = false;
   if(tag >= 0){
     _tag = tag;
     _globalTag = std::max(_globalTag, _tag) + 1;
@@ -354,3 +358,4 @@ double PView::getMemoryInMb()
   mem += getData()->getMemoryInMb();
   return mem;
 }
+
