@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2016 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include "GmshConfig.h"
 #include "GmshMessage.h"
@@ -89,6 +89,7 @@ bool PViewDataGModel::readMSH(const std::string &viewName, const std::string &fi
     else{
       if(fscanf(fp, "%d", &num) != 1) return false;
     }
+    if(num < 0) return false;
     int mult = 1;
     if(_type == ElementNodeData || _type == GaussPointData){
       if(binary){
@@ -134,7 +135,8 @@ bool PViewDataGModel::readMSH(const std::string &viewName, const std::string &fi
 
 bool PViewDataGModel::writeMSH(const std::string &fileName, double version, bool binary,
                                bool saveMesh, bool multipleView, int partitionNum,
-                               bool saveInterpolationMatrices, bool forceNodeData)
+                               bool saveInterpolationMatrices, bool forceNodeData,
+                               bool forceElementData)
 {
   if(_steps.empty()) return true;
 
@@ -147,6 +149,10 @@ bool PViewDataGModel::writeMSH(const std::string &fileName, double version, bool
     Msg::Warning("Cannot force NodeData for this dataset: saving native data");
   }
 
+  if(forceElementData && _type != ElementData){
+    Msg::Warning("Cannot force ElementData for this dataset: saving native data");
+  }
+
   GModel *model = _steps[0]->getModel();
 
   FILE *fp;
