@@ -2,17 +2,18 @@
 
 DATE=`date "+%d/%m/%Y"`
 
-echo "This archive was built on ${DATE} with the latest nightly snapshots of
-Gmsh (http://gmsh.info) and GetDP (http://getdp.info).
+echo "This ONELAB bundle was built on ${DATE} with the latest nightly snapshots
+of Gmsh (http://gmsh.info) and GetDP (http://getdp.info).
 
-Both Gmsh and GetDP are distributed under the terms of the GNU General Public
-License. See the LICENSE and CREDITS files for more information. 
+The software is distributed under the terms of the GNU General Public License. 
+See the LICENSE and CREDITS files for more information. 
 
-The Gmsh tutorial and sample GetDP models are provided in the tutorial and
-models directories.
+The 'models' directory contains a selection of ready-to-use examples from
+http://onelab.info. The 'template' directory contains GetDP templates. The
+'tutorial' directory contains the Gmsh tutorial.
 
-See the official websites for up-to-date versions, documentation and additional
-examples." > /tmp/README.txt
+See http://onelab.info, http://gmsh.info and http://getdp.info for additional
+examples, up-to-date versions and documentation." > /tmp/README.txt
 
 GMSH=svn
 GETDP=svn
@@ -32,8 +33,8 @@ done
 
 rm -rf gmsh-getdp-source*
 mkdir gmsh-getdp-source
-curl -O http://geuz.org/gmsh/src/gmsh-${GMSH}-source.tgz
-curl -O http://geuz.org/getdp/src/getdp-${GETDP}-source.tgz
+curl -O http://gmsh.info/src/gmsh-${GMSH}-source.tgz
+curl -O http://getdp.info/src/getdp-${GETDP}-source.tgz
 mv gmsh-${GMSH}-source.tgz /tmp
 mv getdp-${GETDP}-source.tgz /tmp
 tar zxvf /tmp/gmsh-${GMSH}-source.tgz -C /tmp
@@ -55,8 +56,8 @@ rm -rf gmsh-getdp-source
 
 rm -rf gmsh-getdp-Windows64*
 mkdir gmsh-getdp-Windows64
-curl -O http://geuz.org/gmsh/bin/Windows/gmsh-${GMSH}-Windows64.zip
-curl -O http://geuz.org/getdp/bin/Windows/getdp-${GETDP}-Windows64c.zip
+curl -O http://gmsh.info/bin/Windows/gmsh-${GMSH}-Windows64.zip
+curl -O http://getdp.info/bin/Windows/getdp-${GETDP}-Windows64c.zip
 mv gmsh-${GMSH}-Windows64.zip /tmp
 mv getdp-${GETDP}-Windows64c.zip /tmp
 unzip -q -o /tmp/gmsh-${GMSH}-Windows64.zip -d /tmp
@@ -71,6 +72,7 @@ cat /tmp/getdp-*${GETDP}-Windows/LICENSE.txt >> gmsh-getdp-Windows64/LICENSE.txt
 mv /tmp/gmsh-*${GMSH}-Windows/CREDITS.txt gmsh-getdp-Windows64/CREDITS.txt
 echo "\n\n" >> gmsh-getdp-Windows64/CREDITS.txt
 cat /tmp/getdp-*${GETDP}-Windows/CREDITS.txt >> gmsh-getdp-Windows64/CREDITS.txt
+mv /tmp/getdp-*${GETDP}-Windows/templates gmsh-getdp-Windows64
 mv /tmp/gmsh-*${GMSH}-Windows/tutorial gmsh-getdp-Windows64
 cp -R /tmp/models gmsh-getdp-Windows64
 rm -rf /tmp/gmsh-*
@@ -80,8 +82,8 @@ rm -rf gmsh-getdp-Windows64
 
 rm -rf gmsh-getdp-Windows32*
 mkdir gmsh-getdp-Windows32
-curl -O http://geuz.org/gmsh/bin/Windows/gmsh-${GMSH}-Windows32.zip
-curl -O http://geuz.org/getdp/bin/Windows/getdp-${GETDP}-Windows32c.zip
+curl -O http://gmsh.info/bin/Windows/gmsh-${GMSH}-Windows32.zip
+curl -O http://getdp.info/bin/Windows/getdp-${GETDP}-Windows32c.zip
 mv gmsh-${GMSH}-Windows32.zip /tmp
 mv getdp-${GETDP}-Windows32c.zip /tmp
 unzip -q -o /tmp/gmsh-${GMSH}-Windows32.zip -d /tmp
@@ -96,6 +98,7 @@ cat /tmp/getdp-*${GETDP}-Windows/LICENSE.txt >> gmsh-getdp-Windows32/LICENSE.txt
 mv /tmp/gmsh-*${GMSH}-Windows/CREDITS.txt gmsh-getdp-Windows32/CREDITS.txt
 echo "\n\n" >> gmsh-getdp-Windows32/CREDITS.txt
 cat /tmp/getdp-*${GETDP}-Windows/CREDITS.txt >> gmsh-getdp-Windows32/CREDITS.txt
+mv /tmp/getdp-*${GETDP}-Windows/templates gmsh-getdp-Windows32
 mv /tmp/gmsh-*${GMSH}-Windows/tutorial gmsh-getdp-Windows32
 cp -R /tmp/models gmsh-getdp-Windows32
 rm -rf /tmp/gmsh-*
@@ -105,8 +108,8 @@ rm -rf gmsh-getdp-Windows32
 
 rm -rf gmsh-getdp-Linux32*
 mkdir gmsh-getdp-Linux32
-curl -O http://geuz.org/gmsh/bin/Linux/gmsh-${GMSH}-Linux32.tgz
-curl -O http://geuz.org/getdp/bin/Linux/getdp-${GETDP}-Linux32c.tgz
+curl -O http://gmsh.info/bin/Linux/gmsh-${GMSH}-Linux32.tgz
+curl -O http://getdp.info/bin/Linux/getdp-${GETDP}-Linux32c.tgz
 mv gmsh-${GMSH}-Linux32.tgz /tmp
 mv getdp-${GETDP}-Linux32c.tgz /tmp
 tar zxvf /tmp/gmsh-${GMSH}-Linux32.tgz -C /tmp
@@ -121,6 +124,7 @@ cat /tmp/getdp-*${GETDP}-Linux/share/doc/getdp/LICENSE.txt >> gmsh-getdp-Linux32
 mv /tmp/gmsh-*${GMSH}-Linux/share/doc/gmsh/CREDITS.txt gmsh-getdp-Linux32/CREDITS.txt
 echo "\n\n" >> gmsh-getdp-Linux32/CREDITS.txt
 cat /tmp/getdp-*${GETDP}-Linux/share/doc/getdp/CREDITS.txt >> gmsh-getdp-Linux32/CREDITS.txt
+mv /tmp/getdp-*${GETDP}-Linux/share/doc/getdp/templates gmsh-getdp-Linux32
 mv /tmp/gmsh-*${GMSH}-Linux/share/doc/gmsh/tutorial gmsh-getdp-Linux32
 cp -R /tmp/models gmsh-getdp-Linux32
 rm -rf /tmp/gmsh-*
@@ -130,8 +134,8 @@ rm -rf gmsh-getdp-Linux32
 
 rm -rf gmsh-getdp-Linux64*
 mkdir gmsh-getdp-Linux64
-curl -O http://geuz.org/gmsh/bin/Linux/gmsh-${GMSH}-Linux64.tgz
-curl -O http://geuz.org/getdp/bin/Linux/getdp-${GETDP}-Linux64c.tgz
+curl -O http://gmsh.info/bin/Linux/gmsh-${GMSH}-Linux64.tgz
+curl -O http://getdp.info/bin/Linux/getdp-${GETDP}-Linux64c.tgz
 mv gmsh-${GMSH}-Linux64.tgz /tmp
 mv getdp-${GETDP}-Linux64c.tgz /tmp
 tar zxvf /tmp/gmsh-${GMSH}-Linux64.tgz -C /tmp
@@ -146,6 +150,7 @@ cat /tmp/getdp-*${GETDP}-Linux/share/doc/getdp/LICENSE.txt >> gmsh-getdp-Linux64
 mv /tmp/gmsh-*${GMSH}-Linux/share/doc/gmsh/CREDITS.txt gmsh-getdp-Linux64/CREDITS.txt
 echo "\n\n" >> gmsh-getdp-Linux64/CREDITS.txt
 cat /tmp/getdp-*${GETDP}-Linux/share/doc/getdp/CREDITS.txt >> gmsh-getdp-Linux64/CREDITS.txt
+mv /tmp/getdp-*${GETDP}-Linux/share/doc/getdp/templates gmsh-getdp-Linux64
 mv /tmp/gmsh-*${GMSH}-Linux/share/doc/gmsh/tutorial gmsh-getdp-Linux64
 cp -R /tmp/models gmsh-getdp-Linux64
 rm -rf /tmp/gmsh-*
@@ -155,8 +160,8 @@ rm -rf gmsh-getdp-Linux64
 
 rm -rf gmsh-getdp-MacOSX*
 mkdir gmsh-getdp-MacOSX
-curl -O http://geuz.org/gmsh/bin/MacOSX/gmsh-${GMSH}-MacOSX.dmg
-curl -O http://geuz.org/getdp/bin/MacOSX/getdp-${GETDP}-MacOSX64c.tgz
+curl -O http://gmsh.info/bin/MacOSX/gmsh-${GMSH}-MacOSX.dmg
+curl -O http://getdp.info/bin/MacOSX/getdp-${GETDP}-MacOSX64c.tgz
 mv gmsh-${GMSH}-MacOSX.dmg /tmp
 mv getdp-${GETDP}-MacOSX64c.tgz /tmp
 # open -W /tmp/gmsh-${GMSH}-MacOSX.dmg
@@ -172,6 +177,7 @@ cat /tmp/getdp-*${GETDP}-MacOSX/share/doc/getdp/LICENSE.txt >> gmsh-getdp-MacOSX
 cp gmsh_mount/CREDITS.txt gmsh-getdp-MacOSX/CREDITS.txt
 echo "\n\n" >> gmsh-getdp-MacOSX/CREDITS.txt
 cat /tmp/getdp-*${GETDP}-MacOSX/share/doc/getdp/CREDITS.txt >> gmsh-getdp-MacOSX/CREDITS.txt
+mv /tmp/getdp-*${GETDP}-MacOSX/doc/getdp/templates gmsh-getdp-MacOSX
 cp -R gmsh_mount/tutorial gmsh-getdp-MacOSX
 cp -R /tmp/models gmsh-getdp-MacOSX
 umount gmsh_mount
