@@ -1,4 +1,4 @@
-// ONELAB - Copyright (C) 2011-2014 ULg-UCL
+// ONELAB - Copyright (C) 2011-2016 ULg-UCL
 //
 // Permission is hereby granted, free of charge, to any person
 // obtaining a copy of this software and associated documentation
@@ -23,7 +23,7 @@
 // OF THIS SOFTWARE.
 //
 // Please report all bugs and problems to the public mailing list
-// <gmsh@geuz.org>.
+// <gmsh@onelab.info>.
 
 #ifndef _ONELAB_H_
 #define _ONELAB_H_
@@ -38,19 +38,10 @@
 #include <sstream>
 #include "GmshSocket.h"
 
-#ifdef HAVE_ONELAB2
-#include "NetworkUtils.h"
-#include "OnelabAttributes.h"
-#endif
-
 namespace onelab{
 
   // The base parameter class.
-#ifdef HAVE_ONELAB2
-  class parameter : public OnelabAttr{
-#else
   class parameter {
-#endif
   private:
     // the name of the parameter, including its '/'-separated path in the
     // parameter hierarchy. Parameters or subpaths can start with numbers to
@@ -140,7 +131,15 @@ namespace onelab{
       std::string::size_type last = _name.find_last_of('/');
       if(last != std::string::npos)
         s = _name.substr(last + 1);
-      // remove starting numbers
+      // remove starting white space
+      while(s.size() && s[0] == ' ')
+        s = s.substr(1);
+      // remove starting braces: can be used to order parameters 'from the end',
+      // as the ASCII code is after numbers and letters
+      while(s.size() && (s[0] == '}' || s[0] == '{'))
+        s = s.substr(1);
+      // remove starting numbers: can be used to order parameters 'from the
+      // start'
       while(s.size() && s[0] >= '0' && s[0] <= '9')
         s = s.substr(1);
       return s;
@@ -301,86 +300,6 @@ namespace onelab{
       }
       return true;
     }
-#ifdef HAVE_ONELAB2
-    static UInt16 attributeType() {return 0x05;}
-    virtual inline UInt16 getAttributeType() const {return this->attributeType();}
-    virtual inline UInt16 getAttributeLength() const {
-      UInt16 len = _name.length()+_label.length()+_help.length()+10;
-      for(std::map<std::string, bool>::const_iterator it = getClients().begin(); it != getClients().end(); it++)
-        len += it->first.size()+2;
-      for(std::map<std::string, std::string>::const_iterator it = _attributes.begin(); it != _attributes.end(); it++)
-        len += it->first.size()+it->second.size()+2;
-      return len;
-    }
-    virtual UInt8 *encodeAttribute(UInt8 *dst)
-    {
-      dst = encode(dst, getAttributeType());
-      dst = encode(dst, getAttributeLength());
-
-      dst = encode(dst, (UInt8 *)_name.c_str(), this->_name.length()+1);
-      dst = encode(dst, (UInt8 *)_label.c_str(), this->_label.length()+1);
-      dst = encode(dst, (UInt8 *)_help.c_str(), this->_help.length()+1);
-      dst = encode(dst, (UInt8)_readOnly);
-      dst = encode(dst, (UInt8)_neverChanged);
-      dst = encode(dst, (UInt8)_visible);
-
-      dst = encode(dst, (UInt16)_attributes.size());
-      for(std::map<std::string, std::string>::const_iterator it = _attributes.begin(); it != _attributes.end(); it++) {
-        dst = encode(dst, (UInt8 *)it->first.c_str(), it->first.size()+1);
-        dst = encode(dst, (UInt8 *)it->second.c_str(), it->second.size()+1);
-      }
-
-      dst = encode(dst, (UInt16)_clients.size());
-      for(std::map<std::string, bool>::const_iterator it = getClients().begin(); it != getClients().end(); it++) {
-        dst = encode(dst, (UInt8 *)it->first.c_str(), it->first.size()+1);
-        dst = encode(dst, (UInt8)it->second);
-      }
-
-      return dst;
-    }
-    virtual UInt8 *parseAttribute(UInt8 *src, UInt32 length)
-    {
-      UInt8 tmp;
-      UInt16 n;
-
-      src = parse(src, _name, '\0');
-      src = parse(src, _label, '\0');
-      src = parse(src, _help, '\0');
-      src = parse(src, tmp);
-      this->_readOnly = (bool)tmp;
-      src = parse(src, tmp);
-      this->_neverChanged = (bool)tmp;
-      src = parse(src, tmp);
-      this->_visible = (bool)tmp;
-
-      src = parse(src, n);
-      for(int i=0; i<n; i++) {
-        std::string key, value;
-        src = parse(src, key, '\0');
-        src = parse(src, value, '\0');
-        setAttribute(key, value);
-      }
-
-      src = parse(src, n);
-      for(int i=0; i<n; i++) {
-        std::string client;
-        src = parse(src, client, '\0');
-        src = parse(src, tmp);
-        addClient(client, (bool)tmp);
-      }
-
-      return src;
-    }
-    void showAttribute() const
-    {
-      std::cout << "Name: " << getName() << std::endl
-        << "Label: " << getLabel() << std::endl
-        << "Help: " << getHelp() << std::endl
-        << "Never changed: " << getNeverChanged() << std::endl
-        << "Changed: " << getChanged() << std::endl
-        << "Visible: " << getVisible() << std::endl;
-    }
-#endif
   };
 
   class parameterLessThan{
@@ -392,8 +311,8 @@ namespace onelab{
   };
 
   // The number class. Numbers are stored internally as double precision real
-  // numbers. All more complicated types (complex numbers, vectors, etc.) are
-  // supposed to be either exchanged as strings or encapsulated in functions.
+  // numbers. All more complicated types (complex numbers, vectors, expressions,
+  // functions, etc.) are supposed to be exchanged as strings.
   class number : public parameter{
   private:
     double _value, _min, _max, _step;
@@ -501,73 +420,6 @@ namespace onelab{
       }
       return pos;
     }
-#ifdef HAVE_ONELAB2
-    static UInt16 attributeType() {return 0x06;}
-    virtual inline UInt16 getAttributeType() const {return this->attributeType();}
-    virtual inline UInt16 getAttributeLength() const {
-      UInt16 len = parameter::getAttributeLength()+sizeof(double)*4+8+sizeof(double)*_choices.size();
-      for(std::map<double, std::string>::const_iterator it = _valueLabels.begin(); it != _valueLabels.end(); it++)
-        len += it->second.size()+1+sizeof(double);
-      return len;
-    }
-    UInt8 *encodeAttribute(UInt8 *dst)
-    {
-      dst = parameter::encodeAttribute(dst);
-
-      dst = encode(dst, _value);
-      dst = encode(dst, _min);
-      dst = encode(dst, _max);
-      dst = encode(dst, _step);
-      dst = encode(dst, (UInt32)_index);
-
-      dst = encode(dst, (UInt16)_choices.size());
-      for(unsigned int i = 0; i < _choices.size(); i++)
-        dst = encode(dst, _choices[i]);
-
-      dst = encode(dst, (UInt16)_valueLabels.size());
-      for(std::map<double, std::string>::const_iterator it = _valueLabels.begin(); it != _valueLabels.end(); it++) {
-        dst = encode(dst, it->first);
-        dst = encode(dst, (UInt8 *)it->second.c_str(), it->second.size()+1);
-      }
-
-      return dst;
-    }
-    UInt8 *parseAttribute(UInt8 *src, UInt32 length)
-    {
-      UInt16 n;
-
-      src = parameter::parseAttribute(src, length);
-
-      src = parse(src, _value);
-      src = parse(src, _min);
-      src = parse(src, _max);
-      src = parse(src, _step);
-      src = parse(src, *(UInt32 *)&_index);
-
-      src = parse(src, n);
-      _choices.resize(n);
-      for(unsigned int i = 0; i < n; i++)
-        src = parse(src, _choices[i]);
-
-      src = parse(src, n);
-      for(int i=0; i<n; i++) {
-        double value;
-        std::string label;
-        src = parse(src, value);
-        src = parse(src, label, '\0');
-        setValueLabel(value, label);
-      }
-
-      return src;
-    }
-    void showAttribute() const
-    {
-      parameter::showAttribute();
-      std::cout << "Value: " << this->_value << std::endl
-        << "Min: " << this->_min << std::endl
-        << "Max: " << this->_max << std::endl;
-    }
-#endif
   };
 
   // The string class. A string has a mutable "kind": we do not derive
@@ -632,337 +484,6 @@ namespace onelab{
         _choices[i] = getNextToken(msg, pos);
       return pos;
     }
-#ifdef HAVE_ONELAB2
-    static UInt16 attributeType() {return 0x07;}
-    virtual inline UInt16 getAttributeType() const {return this->attributeType();}
-    virtual inline UInt16 getAttributeLength() const
-    {
-      UInt16 len =  parameter::getAttributeLength();
-      len += _value.size()+_kind.size()+4;
-      for(unsigned int i = 0; i < _choices.size(); i++)
-        len += _choices[i].size()+1;
-      return len;
-    }
-    UInt8 *encodeAttribute(UInt8 *dst)
-    {
-      dst = parameter::encodeAttribute(dst);
-
-      dst = encode(dst, (UInt8 *)_value.c_str(), _value.size()+1);
-      dst = encode(dst, (UInt8 *)_kind.c_str(), _kind.size()+1);
-
-      dst = encode(dst, (UInt16)_choices.size());
-      for(unsigned int i = 0; i < _choices.size(); i++)
-        dst = encode(dst, (UInt8 *)_choices[i].c_str(), _choices[i].size()+1);
-
-      return dst;
-     }
-    UInt8 *parseAttribute(UInt8 *src, UInt32 length)
-    {
-      UInt16 n;
-      src = parameter::parseAttribute(src, length);
-      src = parse(src, _value, '\0');
-      src = parse(src, _kind, '\0');
-
-      src = parse(src, n);
-      _choices.resize(n);
-      for(unsigned int i=0; i<n; i++) {
-        src = parse(src, _choices[i], '\0');
-      }
-
-      return src;
-    }
-    void showAttribute() const
-    {
-      parameter::showAttribute();
-      std::cout << "Value: " << this->_value << std::endl;
-    }
-#endif
-  };
-
-  // The region class. A region can be any kind of geometrical entity,
-  // represented as identifiers of physical regions. Operations on regions will
-  // include union, intersection, etc.
-  class region : public parameter{
-  private:
-    std::set<std::string> _value;
-    // optional geometrical dimension
-    int _dimension;
-    std::vector<std::set<std::string> > _choices;
-  public:
-    region(const std::string &name="",
-           const std::set<std::string> &value = std::set<std::string>(),
-           const std::string &label="", const std::string &help="")
-      : parameter(name, label, help), _value(value), _dimension(-1) {}
-    region(const std::string &name, const std::string &value,
-           const std::string &label="", const std::string &help="")
-      : parameter(name, label, help), _dimension(-1)
-    {
-      if(value.size()) _value.insert(value);
-    }
-    void setValue(const std::set<std::string> &value){ _value = value; }
-    void setDimension(int dim){ _dimension = dim; }
-    void setChoices(const std::vector<std::set<std::string> > &choices)
-    {
-      _choices = choices;
-    }
-    std::string getType() const { return "region"; }
-    const std::set<std::string> &getValue() const { return _value; }
-    int getDimension() const { return _dimension; }
-    const std::vector<std::set<std::string> > &getChoices() const
-    {
-      return _choices;
-    }
-    void update(const region &p)
-    {
-      addClients(p.getClients());
-      setLabel(p.getLabel());
-      setHelp(p.getHelp());
-      setAttributes(p.getAttributes());
-      if(p.getValue() != getValue()){
-        setValue(p.getValue());
-        setChanged(true);
-      }
-      setDimension(p.getDimension());
-      setChoices(p.getChoices());
-      if(getNeverChanged()) setChanged(false);
-    }
-    std::string toChar() const
-    {
-      std::ostringstream sstream;
-      sstream << parameter::toChar() << _value.size() << charSep();
-      for(std::set<std::string>::const_iterator it = _value.begin();
-          it != _value.end(); it++)
-        sstream << sanitize(*it) << charSep();
-      sstream << _dimension << charSep();
-      sstream << _choices.size() << charSep();
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        sstream << _choices[i].size() << charSep();
-        for(std::set<std::string>::const_iterator it = _choices[i].begin();
-            it != _choices[i].end(); it++)
-          sstream << sanitize(*it) << charSep();
-      }
-      return sstream.str();
-    }
-    std::string::size_type fromChar(const std::string &msg)
-    {
-      std::string::size_type pos = parameter::fromChar(msg);
-      if(!pos) return 0;
-      int n = atoi(getNextToken(msg, pos).c_str());
-      for(int i = 0; i < n; i++)
-        _value.insert(getNextToken(msg, pos));
-      setDimension(atoi(getNextToken(msg, pos).c_str()));
-      _choices.resize(atoi(getNextToken(msg, pos).c_str()));
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        n = atoi(getNextToken(msg, pos).c_str());
-        for(int i = 0; i < n; i++)
-          _choices[i].insert(getNextToken(msg, pos));
-      }
-      return pos;
-    }
-#ifdef HAVE_ONELAB2
-    static UInt16 attributeType() {return 0x08;}
-    virtual inline UInt16 getAttributeType() const {return this->attributeType();}
-    virtual inline UInt16 getAttributeLength() const {
-      UInt16 len = parameter::getAttributeLength();
-      len += 2;
-      for(std::set<std::string>::const_iterator it = _value.begin(); it != _value.end(); it++)
-        len += it->size()+1;
-      len += 4;
-      len += 2;
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        len += 2;
-        for(std::set<std::string>::const_iterator it = _choices[i].begin(); it != _choices[i].end(); it++)
-          len += it->size()+1;
-      }
-      return len;
-    }
-    UInt8 *encodeAttribute(UInt8 *dst)
-    {
-      dst = parameter::encodeAttribute(dst);
-      dst = encode(dst, (UInt16)this->_value.size());
-      for(std::set<std::string>::const_iterator it = _value.begin(); it != _value.end(); it++)
-        dst = encode(dst, (UInt8 *)it->c_str(), it->size()+1);
-      dst = encode(dst, (UInt32)_dimension);
-      dst = encode(dst, (UInt16)_choices.size());
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        dst = encode(dst, (UInt16)_choices[i].size());
-        for(std::set<std::string>::const_iterator it = _choices[i].begin(); it != _choices[i].end(); it++)
-          dst = encode(dst, (UInt8 *)it->c_str(), it->size()+1);
-      }
-      return dst;
-    }
-    UInt8 *parseAttribute(UInt8 *src, UInt32 len)
-    {
-      src = parameter::parseAttribute(src, len);
-      UInt16 m = 0, n = 0;
-      std::string value;
-      src = parse(src, n);
-      for(int i=0; i<n; i++) {
-        src = parse(src, value, '\0');
-      	_value.insert(value);
-      }
-      src = parse(src, *(UInt32 *)&_dimension);
-      src = parse(src, n);
-      _choices.resize(n);
-      for(int i=0; i<n; i++) {
-        src = parse(src, m);
-        for(int j=0; j<m; j++) {
-          src = parse(src, value, '\0');
-          _choices[i].insert(value);
-        }
-      }
-      return src;
-    }
-    void showAttribute() const {}
-#endif
-
-  };
-
-  // The (possibly piece-wise defined on regions) function class. Functions are
-  // entirely client-dependent: they are just represented internally as onelab
-  // strings, defined on onelab regions.
-  class function : public parameter{
-  private:
-    std::map<std::string, std::string> _value;
-    std::vector<std::map<std::string, std::string> > _choices;
-  public:
-    function(const std::string &name="") : parameter(name, "", "") {}
-    function(const std::string &name, const std::map<std::string, std::string> &value,
-             const std::string &label="", const std::string &help="")
-      : parameter(name, label, help), _value(value) {}
-    void setValue(const std::map<std::string, std::string> &value)
-    {
-      _value = value;
-    }
-    void setChoices(const std::vector<std::map<std::string, std::string> > &choices)
-    {
-      _choices = choices;
-    }
-    std::string getType() const { return "function"; }
-    const std::map<std::string, std::string> &getValue() const { return _value; }
-    const std::string getValue(const std::string &region) const
-    {
-      std::map<std::string, std::string>::const_iterator it = _value.find(region);
-      if(it != _value.end()) return it->second;
-      return "";
-    }
-    const std::vector<std::map<std::string, std::string> > &getChoices() const
-    {
-      return _choices;
-    }
-    void update(const function &p)
-    {
-      addClients(p.getClients());
-      setLabel(p.getLabel());
-      setHelp(p.getHelp());
-      setAttributes(p.getAttributes());
-      if(p.getValue() != getValue()){
-        setValue(p.getValue());
-        setChanged(true);
-      }
-      setChoices(p.getChoices());
-      if(getNeverChanged()) setChanged(false);
-    }
-    std::string toChar() const
-    {
-      std::ostringstream sstream;
-      sstream << parameter::toChar() << _value.size() << charSep();
-      for(std::map<std::string, std::string>::const_iterator it = _value.begin();
-          it != _value.end(); it++)
-        sstream << sanitize(it->first) << charSep()
-                << sanitize(it->second) << charSep();
-      sstream << _choices.size() << charSep();
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        sstream << _choices[i].size() << charSep();
-        for(std::map<std::string, std::string>::const_iterator it = _choices[i].begin();
-            it != _choices[i].end(); it++)
-          sstream << sanitize(it->first) << charSep()
-                  << sanitize(it->second) << charSep();
-      }
-      return sstream.str();
-    }
-    std::string::size_type fromChar(const std::string &msg)
-    {
-      std::string::size_type pos = parameter::fromChar(msg);
-      if(!pos) return 0;
-      int n = atoi(getNextToken(msg, pos).c_str());
-      for(int i = 0; i < n; i++){
-        std::string key = getNextToken(msg, pos);
-        _value[key] = getNextToken(msg, pos);
-      }
-      _choices.resize(atoi(getNextToken(msg, pos).c_str()));
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        n = atoi(getNextToken(msg, pos).c_str());
-        for(int i = 0; i < n; i++){
-          std::string key = getNextToken(msg, pos);
-          _choices[i][key] = getNextToken(msg, pos);
-        }
-      }
-      return pos;
-    }
-#ifdef HAVE_ONELAB2
-    static UInt16 attributeType() {return 0x09;}
-    virtual inline UInt16 getAttributeType() const {return this->attributeType();}
-    virtual inline UInt16 getAttributeLength() const {
-      UInt16 len = parameter::getAttributeLength();
-      len += 2;
-      for(std::map<std::string, std::string>::const_iterator it = _value.begin();
-          it != _value.end(); it++)
-        len += 2+it->first.size()+it->second.size();
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        len += 2;
-        for(std::map<std::string, std::string>::const_iterator it = _choices[i].begin();
-            it != _choices[i].end(); it++) {
-          len += 2+it->first.size()+it->second.size();
-        }
-      }
-      return len;
-    }
-    UInt8 *encodeAttribute(UInt8 *dst)
-    {
-      dst = parameter::encodeAttribute(dst),
-      dst = encode(dst, (UInt16)this->_value.size());
-      for(std::map<std::string, std::string>::const_iterator it = _value.begin();
-          it != _value.end(); it++) {
-        dst = encode(dst, (UInt8 *)it->first.c_str(), it->first.size()+1);
-        dst = encode(dst, (UInt8 *)it->second.c_str(), it->second.size()+1);
-      }
-      dst = encode(dst, (UInt16)_choices.size());
-      for(unsigned int i = 0; i < _choices.size(); i++){
-        dst = encode(dst, (UInt16)_choices[i].size());
-        for(std::map<std::string, std::string>::const_iterator it = _choices[i].begin();
-            it != _choices[i].end(); it++) {
-          dst = encode(dst, (UInt8 *)it->first.c_str(), it->first.size()+1);
-          dst = encode(dst, (UInt8 *)it->second.c_str(), it->second.size()+1);
-        }
-      }
-      return dst;
-    }
-    UInt8 *parseAttribute(UInt8 *src, UInt32 len)
-    {
-      src = parameter::parseAttribute(src, len);
-      UInt16 m = 0, n = 0;
-      std::string key, value;
-      src = parse(src, n);
-      for(int i=0; i<n; i++) {
-        src = parse(src, key, '\0');
-        src = parse(src, value, '\0');
-      	_value[key] = value;
-      }
-      src = parse(src, n);
-      _choices.resize(n);
-      for(int i=0; i<n; i++) {
-        src = parse(src, m);
-        for(int j=0; j<m; j++) {
-          src = parse(src, key, '\0');
-          src = parse(src, value, '\0');
-          _choices[i][key] = value;
-        }
-      }
-      return src;
-    }
-    void showAttribute() const {}
-#endif
   };
 
   // The parameter space, i.e., the set of parameters stored and handled by the
@@ -971,21 +492,23 @@ namespace onelab{
   private:
     std::set<number*, parameterLessThan> _numbers;
     std::set<string*, parameterLessThan> _strings;
-    std::set<region*, parameterLessThan> _regions;
-    std::set<function*, parameterLessThan> _functions;
     // delete a parameter from the parameter space
     template <class T> bool _clear(const std::string &name,
                                    const std::string &client,
                                    std::set<T*, parameterLessThan> &ps)
     {
       if(name.empty() && client.size()){
+        std::vector<T*> toDelete;
         for(typename std::set<T*, parameterLessThan>::iterator it = ps.begin();
-            it != ps.end(); it++){
+            it != ps.end(); ){
           T *p = *it;
           if(p->hasClient(client)){
-            ps.erase(it);
+            ps.erase(it++); // to avoid invalid iterator
             delete p;
           }
+          else{
+            it++;
+          }
         }
       }
       else{
@@ -1016,9 +539,6 @@ namespace onelab{
       }
       else{
         T* newp = new T(p);
-#ifdef HAVE_ONELAB2
-        newp->isInDatabase(true);
-#endif
         if(client.size()) newp->addClient(client, true);
         ps.insert(newp);
       }
@@ -1048,7 +568,8 @@ namespace onelab{
       }
       return true;
     }
-    template <class T> T* _getPtr(std::string name, const std::string client, std::set<T*, parameterLessThan> ps)
+    template <class T> T* _getPtr(std::string name, const std::string client,
+                                  std::set<T*, parameterLessThan> ps)
     {
       T tmp(name);
       typename std::set<T*, parameterLessThan>::iterator it = ps.find(&tmp);
@@ -1062,8 +583,6 @@ namespace onelab{
     {
       ps.insert(_numbers.begin(), _numbers.end());
       ps.insert(_strings.begin(), _strings.end());
-      ps.insert(_regions.begin(), _regions.end());
-      ps.insert(_functions.begin(), _functions.end());
     }
   public:
     parameterSpace(){}
@@ -1078,50 +597,36 @@ namespace onelab{
           delete *it;
         _numbers.clear();
         _strings.clear();
-        _regions.clear();
-        _functions.clear();
       }
       else{
         bool done = _clear(name, client, _numbers);
         if(!done) done = _clear(name, client, _strings);
-        if(!done) done = _clear(name, client, _regions);
-        if(!done) done = _clear(name, client, _functions);
       }
     }
     bool set(const number &p,
              const std::string &client=""){ return _set(p, client, _numbers); }
     bool set(const string &p,
              const std::string &client=""){ return _set(p, client, _strings); }
-    bool set(const region &p,
-             const std::string &client=""){ return _set(p, client, _regions); }
-    bool set(const function &p,
-             const std::string &client=""){ return _set(p, client, _functions); }
     bool get(std::vector<number> &ps, const std::string &name="",
              const std::string &client=""){ return _get(ps, name, client, _numbers); }
     bool get(std::vector<onelab::string> &ps, const std::string &name="",
              const std::string &client=""){ return _get(ps, name, client, _strings); }
-    bool get(std::vector<region> &ps, const std::string &name="",
-             const std::string &client=""){ return _get(ps, name, client, _regions); }
-    bool get(std::vector<function> &ps, const std::string &name="",
-             const std::string &client=""){ return _get(ps, name, client, _functions); }
     void getPtr(number **ptr, const std::string name, const std::string client="")
-      {*ptr = _getPtr(name, client, _numbers);}
+    {
+      *ptr = _getPtr(name, client, _numbers);
+    }
     void getPtr(string **ptr, const std::string name, const std::string client="")
-      {*ptr = _getPtr(name, client, _strings);}
-    void getPtr(region **ptr, const std::string name, const std::string client="")
-      {*ptr = _getPtr(name, client, _regions);}
-    void getPtr(function **ptr, const std::string name, const std::string client="")
-      {*ptr = _getPtr(name, client, _functions);}
+    {
+      *ptr = _getPtr(name, client, _strings);
+    }
     void getAllParameters(std::set<parameter*, parameterLessThan> &ps) const
     {
       ps.insert(_numbers.begin(), _numbers.end());
       ps.insert(_strings.begin(), _strings.end());
-      ps.insert(_regions.begin(), _regions.end());
-      ps.insert(_functions.begin(), _functions.end());
     }
     unsigned int getNumParameters()
     {
-      return (int)(_numbers.size() + _strings.size() + _regions.size() + _functions.size());
+      return (int)(_numbers.size() + _strings.size());
     }
     // check if at least one parameter depends on the given client
     bool hasClient(const std::string &client) const
@@ -1185,12 +690,6 @@ namespace onelab{
         else if(type == "string"){
           onelab::string p; p.fromChar(msg[i]); set(p, client);
         }
-        else if(type == "region"){
-          onelab::region p; p.fromChar(msg[i]); set(p, client);
-        }
-        else if(type == "function"){
-          onelab::function p; p.fromChar(msg[i]); set(p, client);
-        }
         else
           return false;
       }
@@ -1231,23 +730,15 @@ namespace onelab{
     virtual bool clear(const std::string &name) = 0;
     virtual bool set(const number &p) = 0;
     virtual bool set(const string &p) = 0;
-    virtual bool set(const region &p) = 0;
-    virtual bool set(const function &p) = 0;
     virtual bool get(std::vector<number> &ps, const std::string &name="") = 0;
     virtual bool get(std::vector<onelab::string> &ps, const std::string &name="") = 0;
-    virtual bool get(std::vector<region> &ps, const std::string &name="") = 0;
-    virtual bool get(std::vector<function> &ps, const std::string &name="") = 0;
     std::vector<std::string> toChar()
     {
       std::vector<std::string> out;
       std::vector<number> n; get(n);
       for(unsigned int i = 0; i < n.size(); i++) out.push_back(n[i].toChar());
-      std::vector<number> s; get(s);
+      std::vector<string> s; get(s);
       for(unsigned int i = 0; i < s.size(); i++) out.push_back(s[i].toChar());
-      std::vector<region> r; get(r);
-      for(unsigned int i = 0; i < r.size(); i++) out.push_back(r[i].toChar());
-      std::vector<region> f; get(f);
-      for(unsigned int i = 0; i < f.size(); i++) out.push_back(f[i].toChar());
       return out;
     }
     bool fromChar(const std::vector<std::string> &msg)
@@ -1262,12 +753,6 @@ namespace onelab{
         else if(type == "string"){
           onelab::string p; p.fromChar(msg[i]); set(p);
         }
-        else if(type == "region"){
-          onelab::region p; p.fromChar(msg[i]); set(p);
-        }
-        else if(type == "function"){
-          onelab::function p; p.fromChar(msg[i]); set(p);
-        }
         else
           return false;
       }
@@ -1292,8 +777,8 @@ namespace onelab{
     static server *_server;
     // the address of the server
     std::string _address;
-    // the connected clients, indexed by name
-    std::map<std::string, client*> _clients;
+    // the connected clients
+    std::set<client*> _clients;
     // the parameter space
     parameterSpace _parameterSpace;
   public:
@@ -1318,17 +803,22 @@ namespace onelab{
     {
       return _parameterSpace.get(ps, name, client);
     }
-    typedef std::map<std::string, client*>::iterator citer;
+    typedef std::set<client*>::iterator citer;
     citer firstClient(){ return _clients.begin(); }
     citer lastClient(){ return _clients.end(); }
     int getNumClients() { return (int)_clients.size(); };
-    citer findClient(const std::string &name){ return _clients.find(name); }
+    citer findClient(const std::string &name)
+    {
+      for(citer it = _clients.begin(); it != _clients.end(); it++)
+        if((*it)->getName() == name) return it;
+      return _clients.end();
+    }
     void registerClient(client *c)
     {
-      _clients[c->getName()] = c;
+      _clients.insert(c);
       c->setId(_clients.size());
     }
-    void unregisterClient(client *c){ _clients.erase(c->getName()); }
+    void unregisterClient(client *c){ _clients.erase(c); }
     void setChanged(bool changed, const std::string &client="")
     {
       _parameterSpace.setChanged(changed, client);
@@ -1337,10 +827,6 @@ namespace onelab{
     {
       return _parameterSpace.getChanged(client);
     }
-    bool isRegistered(const std::string &client)
-    {
-      return _clients.count(client);
-    }
     unsigned int getNumParameters(){ return _parameterSpace.getNumParameters(); }
     std::vector<std::string> toChar(const std::string &client="")
     {
@@ -1392,16 +878,10 @@ namespace onelab{
     }
     virtual bool set(const number &p){ return _set(p); }
     virtual bool set(const string &p){ return _set(p); }
-    virtual bool set(const function &p){ return _set(p); }
-    virtual bool set(const region &p){ return _set(p); }
     virtual bool get(std::vector<number> &ps,
                      const std::string &name=""){ return _get(ps, name); }
     virtual bool get(std::vector<onelab::string> &ps,
                      const std::string &name=""){ return _get(ps, name); }
-    virtual bool get(std::vector<function> &ps,
-                     const std::string &name=""){ return _get(ps, name); }
-    virtual bool get(std::vector<region> &ps,
-                     const std::string &name=""){ return _get(ps, name); }
   };
 
   // The local part of a network client.
@@ -1434,7 +914,7 @@ namespace onelab{
     const std::string &getExecutable(){ return _executable; }
     void setExecutable(const std::string &s){ _executable = s; }
     const std::string &getRemoteLogin(){ return _remoteLogin; }
-    const bool treatExecutableAsFullCommandLine()
+    bool treatExecutableAsFullCommandLine() const
     {
       return _treatExecutableAsFullCommandLine;
     }
@@ -1591,16 +1071,10 @@ namespace onelab{
     }
     virtual bool set(const number &p){ return _set(p); }
     virtual bool set(const string &p){ return _set(p); }
-    virtual bool set(const function &p){ return _set(p); }
-    virtual bool set(const region &p){ return _set(p); }
     virtual bool get(std::vector<number> &ps,
                      const std::string &name=""){ return _get(ps, name); }
     virtual bool get(std::vector<onelab::string> &ps,
                      const std::string &name=""){ return _get(ps, name); }
-    virtual bool get(std::vector<function> &ps,
-                     const std::string &name=""){ return _get(ps, name); }
-    virtual bool get(std::vector<region> &ps,
-                     const std::string &name=""){ return _get(ps, name); }
     void sendInfo(const std::string &msg)
     {
       if(_gmshClient) _gmshClient->Info(msg.c_str());
